@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6460 $:
+$Revision: 6998 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-09-20 17:37:06 +0200 (Thu, 20 Sep 2012) $
+$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -321,8 +321,6 @@ ConnectorItem::ConnectorItem( Connector * connector, ItemBase * attachedTo )
 	m_groundFillSeed = false;
 	m_connectorDetectT = m_connectorDrawT = 0;
 	m_draggingCurve = m_draggingLeg = m_rubberBandLeg = m_bigDot = m_hybrid = false;
-	m_marked = false;
-	m_checkedEffectively = false;
 	m_hoverEnterSpaceBarWasPressed = m_spaceBarWasPressed = false;
 	m_overConnectorItem = NULL;
 	m_connectorHovering = false;
@@ -355,6 +353,7 @@ ConnectorItem::~ConnectorItem() {
 
 void ConnectorItem::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 
+    //debugInfo("connector hoverEnter");
 	/*
 	QRectF sbr = this->sceneBoundingRect();
 	QPointF p = event->scenePos();
@@ -406,7 +405,8 @@ void ConnectorItem::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
 		return;
 	}
 
-	restoreColor(false, 0, true);
+    QList<ConnectorItem *> visited;
+	restoreColor(visited);
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
 		infoGraphicsView->hoverLeaveConnectorItem(event, this);
@@ -451,7 +451,8 @@ void ConnectorItem::connectorHover(ItemBase * itemBase, bool hovering) {
 		setHoverColor();			// could make this light up buses as well
 	}
 	else {
-		restoreColor(false, 0, true);
+        QList<ConnectorItem *> visited;
+		restoreColor(visited);
 	}
 	if (this->m_attachedTo != NULL) {
 		m_attachedTo->connectorHover(this, itemBase, hovering);
@@ -467,13 +468,15 @@ void ConnectorItem::connectTo(ConnectorItem * connected) {
 
 	m_connectedTo.append(connected);
 	//DebugDialog::debug(QString("connect to cc:%4 this:%1 to:%2 %3").arg((long) this, 0, 16).arg((long) connected, 0, 16).arg(connected->attachedTo()->modelPartShared()->title()).arg(m_connectedTo.count()) );
-	restoreColor(true, 0, true);
+    QList<ConnectorItem *> visited;
+	restoreColor(visited);
 	if (m_attachedTo != NULL) {
 		m_attachedTo->connectionChange(this, connected, true);
 	}
 }
 
 ConnectorItem * ConnectorItem::removeConnection(ItemBase * itemBase) {
+    QList<ConnectorItem *> visited;
 	for (int i = 0; i < m_connectedTo.count(); i++) {
 		if (m_connectedTo[i]->attachedTo() == itemBase) {
 			ConnectorItem * removed = m_connectedTo[i];
@@ -481,7 +484,7 @@ ConnectorItem * ConnectorItem::removeConnection(ItemBase * itemBase) {
 			if (m_attachedTo != NULL) {
 				m_attachedTo->connectionChange(this, removed, false);
 			}
-			restoreColor(true, 0, true);
+			restoreColor(visited);
 			DebugDialog::debug(QString("remove from:%1 to:%2 count%3")
 				.arg((long) this, 0, 16)
 				.arg(itemBase->modelPartShared()->title())
@@ -497,7 +500,8 @@ void ConnectorItem::removeConnection(ConnectorItem * connectedItem, bool emitCha
 	if (connectedItem == NULL) return;
 
 	m_connectedTo.removeOne(connectedItem);
-	restoreColor(true, 0, true);
+    QList<ConnectorItem *> visited;
+	restoreColor(visited);
 	if (emitChange) {
 		m_attachedTo->connectionChange(this, connectedItem, false);
 	}
@@ -506,82 +510,62 @@ void ConnectorItem::removeConnection(ConnectorItem * connectedItem, bool emitCha
 void ConnectorItem::tempConnectTo(ConnectorItem * item, bool applyColor) {
 	if (!m_connectedTo.contains(item)) m_connectedTo.append(item);
 
-	if(applyColor) restoreColor(true, 0, true);
+	if(applyColor) {
+        QList<ConnectorItem *> visited;
+        restoreColor(visited);
+    }
 }
 
 void ConnectorItem::tempRemove(ConnectorItem * item, bool applyColor) {
 	m_connectedTo.removeOne(item);
 
-	if(applyColor) restoreColor(true, 0, true);
+	if(applyColor) {    
+        QList<ConnectorItem *> visited;
+        restoreColor(visited);
+    }
 }
 
-void ConnectorItem::restoreColor(bool doBuses, int busConnectionCount, bool doCross) 
+void ConnectorItem::restoreColor(QList<ConnectorItem *> & visited) 
 {
-	setMarked(true);
+    if (visited.contains(this)) return;
+    visited.append(this);
+
 	if (!attachedTo()->isEverVisible()) return;
 
-	/*
-		DebugDialog::debug(QString("restore color dobus:%1 bccount:%2 docross:%3 cid:'%4' '%5' id:%6 '%7' vid:%8 vlid:%9")
-		.arg(doBuses)
-		.arg(busConnectionCount)
-		.arg(doCross)
-		.arg(this->connectorSharedID())
-		.arg(this->connectorSharedName())
-		.arg(this->attachedToID())
-		.arg(this->attachedToInstanceTitle())
-		.arg(this->attachedToViewIdentifier())
-		.arg(this->attachedToViewLayerID())
-		);
-		*/
+    QList<ConnectorItem *> connectorItems;
+    connectorItems.append(this);
+    collectEqualPotential(connectorItems, true, getSkipFlags());
+    visited.append(connectorItems);
+    QSet<ItemBase *> attachedTo;
+    foreach (ConnectorItem * connectorItem, connectorItems) {
+        if (connectorItem->isEverVisible()) {
+            if (connectorItem->attachedToItemType() != ModelPart::Wire) {
+                attachedTo.insert(connectorItem->attachedTo()->layerKinChief());
+            }
+        }
+    }
 
-	int connectedToCount = busConnectionCount;
-	if (attachedToItemType() == ModelPart::Wire) {
-		doBuses = doCross = false;
-		foreach (ConnectorItem * toConnectorItem, m_connectedTo) {
-			if (toConnectorItem->attachedTo()->isEverVisible()) {
-				connectedToCount = 1;
-				break;
-			}
-		}
-	}
-	else if (connectedToCount == 0) {
-		connectedToCount = isConnectedToPart() ? 1 : 0;
-	}
+    foreach (ConnectorItem * connectorItem, connectorItems) {
+        if (connectorItem->isEverVisible()) {
+	        //QString how;
+	        if (attachedTo.count() <= 1) {
+		        if (connectorItem->connectorType() == Connector::Female) {
+                    if (connectorItem->connectionsCount() > 0) connectorItem->setUnconnectedColor();
+			        else connectorItem->setNormalColor();
+			        //how = "normal";	
+		        }
+		        else {
+			        connectorItem->setUnconnectedColor();
+			        //how = "unconnected";
+		        }
+	        }
+	        else {
+		        connectorItem->setConnectedColor();
+		        //how = "connected";
+	        }
+        }
+    }
 
-	if (doBuses) {
-		Bus * b = bus();
-		if (b != NULL) {
-			QList<ConnectorItem *> busConnectedItems;
-			attachedTo()->busConnectorItems(b, busConnectedItems);
-			busConnectedItems.removeOne(this);
-			foreach (ConnectorItem * busConnectorItem, busConnectedItems) {
-				busConnectorItem->restoreColor(false, connectedToCount, true);
-			}
-		}
-	}
-
-	if (doCross) {
-		ConnectorItem * crossConnectorItem = getCrossLayerConnectorItem();
-		if (crossConnectorItem) {
-			crossConnectorItem->restoreColor(false, connectedToCount, false);
-		}
-	}
-
-	QString how;
-	if (connectedToCount <= 0) {
-		if (connectorType() == Connector::Female) {
-			setNormalColor();
-			how = "normal";	
-		}
-		else {
-			setUnconnectedColor();
-			how = "unconnected";
-		}
-	}
-	else {
-		setConnectedColor();
-		how = "connected";
-	}
 
 	/*
 	DebugDialog::debug(QString("restore color dobus:%1 bccount:%2 docross:%3 cid:'%4' '%5' id:%6 '%7' vid:%8 vlid:%9 %10")
@@ -592,7 +576,7 @@ void ConnectorItem::restoreColor(bool doBuses, int busConnectionCount, bool doCr
 		.arg(this->connectorSharedName())
 		.arg(this->attachedToID())
 		.arg(this->attachedToInstanceTitle())
-		.arg(this->attachedToViewIdentifier())
+		.arg(this->attachedToViewID())
 		.arg(this->attachedToViewLayerID())
 		.arg(how)
 	);
@@ -840,8 +824,9 @@ void ConnectorItem::mousePressEvent(QGraphicsSceneMouseEvent *event) {
 	m_equalPotentialDisplayItems.append(this);
 	collectEqualPotential(m_equalPotentialDisplayItems, true, ViewGeometry::NoFlag);
 	//m_equalPotentialDisplayItems.removeAt(0);									// not sure whether to leave the clicked one in or out of the list
-	foreach (ConnectorItem * connectorItem, m_equalPotentialDisplayItems) {
-		connectorItem->showEqualPotential(true);
+	QList<ConnectorItem *> visited;
+    foreach (ConnectorItem * connectorItem, m_equalPotentialDisplayItems) {
+		connectorItem->showEqualPotential(true, visited);
 	}
 	
 	if (m_rubberBandLeg && this->m_attachedTo != NULL && m_attachedTo->acceptsMousePressLegEvent(this, event)) {
@@ -860,7 +845,7 @@ int ConnectorItem::connectionsCount() {
 	return m_connectedTo.count();
 }
 
-void ConnectorItem::attachedMoved() {
+void ConnectorItem::attachedMoved(bool includeRatsnest, QList<ConnectorItem *> & already) {
 	//DebugDialog::debug("attached moved");
     if (!this->isVisible()) return;
 
@@ -881,11 +866,11 @@ void ConnectorItem::attachedMoved() {
 			//itemBase->debugInfo("    ");
 			continue;
 		}
-        if (itemBase->getRatsnest()) {
+        if (itemBase->getRatsnest() && !includeRatsnest) {
             continue;
         }
 
-        toConnectorItem->attachedTo()->connectedMoved(this, toConnectorItem);
+        toConnectorItem->attachedTo()->connectedMoved(this, toConnectorItem, already);
     }
 }
 
@@ -1007,7 +992,7 @@ void ConnectorItem::setInactive(bool inactivate) {
 }
 
 void ConnectorItem::setHiddenOrInactive() {
-	if (m_hidden || m_inactive || m_hybrid) {
+	if (m_hidden || m_inactive || m_hybrid || m_layerHidden) {
 		this->setAcceptedMouseButtons(Qt::NoButton);
 		this->unsetCursor();
 		setAcceptHoverEvents(false);
@@ -1035,6 +1020,12 @@ const QString & ConnectorItem::connectorSharedID() {
 	return m_connector->connectorSharedID();
 }
 
+const QString & ConnectorItem::connectorSharedReplacedby() {
+	if (m_connector == NULL) return ___emptyString___;
+
+	return m_connector->connectorSharedReplacedby();
+}
+
 ErcData * ConnectorItem::connectorSharedErcData() {
 	if (m_connector == NULL) return NULL;
 
@@ -1077,28 +1068,22 @@ Bus * ConnectorItem::bus() {
 	return m_connector->bus();
 }
 
-int ConnectorItem::attachedToItemType() {
-	if (m_attachedTo == NULL) return ModelPart::Unknown;
-
-	return m_attachedTo->itemType();
-}
-
 ViewLayer::ViewLayerID ConnectorItem::attachedToViewLayerID() {
 	if (m_attachedTo == NULL) return ViewLayer::UnknownLayer;
 
 	return m_attachedTo->viewLayerID();
 }
 
-ViewLayer::ViewLayerSpec ConnectorItem::attachedToViewLayerSpec() {
-	if (m_attachedTo == NULL) return ViewLayer::UnknownSpec;
+ViewLayer::ViewLayerPlacement ConnectorItem::attachedToViewLayerPlacement() {
+	if (m_attachedTo == NULL) return ViewLayer::UnknownPlacement;
 
-	return m_attachedTo->viewLayerSpec();
+	return m_attachedTo->viewLayerPlacement();
 }
 
-ViewLayer::ViewIdentifier ConnectorItem::attachedToViewIdentifier() {
+ViewLayer::ViewID ConnectorItem::attachedToViewID() {
 	if (m_attachedTo == NULL) return ViewLayer::UnknownView;
 
-	return m_attachedTo->viewIdentifier();
+	return m_attachedTo->viewID();
 }
 
 Connector::ConnectorType ConnectorItem::connectorType() {
@@ -1261,7 +1246,7 @@ bool ConnectorItem::isConnectedToPart() {
 	QList<ConnectorItem *> busConnectedItems;
 	Bus * b = bus();
 	if (b != NULL) {
-		attachedTo()->busConnectorItems(b, busConnectedItems);
+		attachedTo()->busConnectorItems(b, this, busConnectedItems);
 	}
 
 	for (int i = 0; i < tempItems.count(); i++) {
@@ -1273,6 +1258,7 @@ bool ConnectorItem::isConnectedToPart() {
 		{
 			switch (connectorItem->attachedToItemType()) {
 				case ModelPart::Symbol:
+				case ModelPart::SchematicSubpart:
 				case ModelPart::Jumper:
 				case ModelPart::Part:
 				case ModelPart::CopperFill:
@@ -1305,7 +1291,7 @@ bool ConnectorItem::isConnectedToPart() {
 		Bus * bus = connectorItem->bus();
 		if (bus != NULL) {
 			QList<ConnectorItem *> busConnectedItems;
-			connectorItem->attachedTo()->busConnectorItems(bus, busConnectedItems);
+			connectorItem->attachedTo()->busConnectorItems(bus, connectorItem, busConnectedItems);
 			foreach (ConnectorItem * busConnectedItem, busConnectedItems) {
 				if (!tempItems.contains(busConnectedItem)) {
 					tempItems.append(busConnectedItem);
@@ -1350,6 +1336,7 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 
 		// this one's a keeper
 		connectorItems.append(connectorItem);
+        //connectorItem->debugInfo("collect");
 
 		foreach (ConnectorItem * cto, connectorItem->connectedToItems()) {
 			if (tempItems.contains(cto)) continue;
@@ -1365,7 +1352,7 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 		Bus * bus = connectorItem->bus();
 		if (bus != NULL) {
 			QList<ConnectorItem *> busConnectedItems;
-			connectorItem->attachedTo()->busConnectorItems(bus, busConnectedItems);
+			connectorItem->attachedTo()->busConnectorItems(bus, connectorItem, busConnectedItems);
 #ifndef QT_NO_DEBUG
 			if (connectorItem->attachedToItemType() == ModelPart::Wire && busConnectedItems.count() != 2) {
 				connectorItem->debugInfo("bus is missing");
@@ -1382,19 +1369,19 @@ void ConnectorItem::collectEqualPotential(QList<ConnectorItem *> & connectorItem
 	}
 }
 
-void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerSpec viewLayerSpec)
+void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<ConnectorItem *> & partsConnectors, bool includeSymbols, ViewLayer::ViewLayerPlacement viewLayerPlacement)
 {
 	if (connectorItems.count() == 0) return;
 
 	//DebugDialog::debug("___________________________");
-	switch (viewLayerSpec) {
-		case ViewLayer::Top:
-		case ViewLayer::Bottom:
-		case ViewLayer::TopAndBottom:
+	switch (viewLayerPlacement) {
+		case ViewLayer::NewTop:
+		case ViewLayer::NewBottom:
+		case ViewLayer::NewTopAndBottom:
 			break;
 		default:
-			DebugDialog::debug(QString("collect parts unknown spec %1").arg(viewLayerSpec));
-			viewLayerSpec = ViewLayer::TopAndBottom;
+			DebugDialog::debug(QString("collect parts unknown spec %1").arg(viewLayerPlacement));
+			viewLayerPlacement = ViewLayer::NewTopAndBottom;
 			break;
 	}
 	
@@ -1406,6 +1393,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 		ItemBase * candidate = connectorItem->attachedTo();
 		switch (candidate->itemType()) {
 			case ModelPart::Symbol:
+			case ModelPart::SchematicSubpart:
 				if (!includeSymbols) break;
 			case ModelPart::Jumper:
 			case ModelPart::Part:
@@ -1413,7 +1401,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 			case ModelPart::Board:
 			case ModelPart::ResizableBoard:
 			case ModelPart::Via:
-				collectPart(connectorItem, partsConnectors, viewLayerSpec);
+				collectPart(connectorItem, partsConnectors, viewLayerPlacement);
 				break;
 			default:
 				break;
@@ -1421,7 +1409,7 @@ void ConnectorItem::collectParts(QList<ConnectorItem *> & connectorItems, QList<
 	}
 }
 
-void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerSpec viewLayerSpec) {
+void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorItem *> & partsConnectors, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	if (partsConnectors.contains(connectorItem)) return;
 				
 	ConnectorItem * crossConnectorItem = connectorItem->getCrossLayerConnectorItem();
@@ -1430,7 +1418,7 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 			return;
 		}
 		
-		if (viewLayerSpec == ViewLayer::TopAndBottom) {
+		if (viewLayerPlacement == ViewLayer::NewTopAndBottom) {
 			partsConnectors.append(crossConnectorItem);
 			
 			/*
@@ -1442,14 +1430,14 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 			*/
 				
 		}
-		else if (viewLayerSpec == ViewLayer::Top) {
+		else if (viewLayerPlacement == ViewLayer::NewTop) {
 			if (connectorItem->attachedToViewLayerID() == ViewLayer::Copper1) {
 			}
 			else {
 				connectorItem = crossConnectorItem;
 			}
 		}
-		else if (viewLayerSpec == ViewLayer::Bottom) {
+		else if (viewLayerPlacement == ViewLayer::NewBottom) {
 			if (connectorItem->attachedToViewLayerID() == ViewLayer::Copper0) {
 			}
 			else {
@@ -1472,8 +1460,10 @@ void ConnectorItem::collectPart(ConnectorItem * connectorItem, QList<ConnectorIt
 void ConnectorItem::updateTooltip() {
 	if (attachedToItemType() != ModelPart::Wire) {
 		QString name = connectorSharedName();
+        bool isInt = false;
+        name.toInt(&isInt);
 		QString descr = connectorSharedDescription();
-        if (name == descr) {
+        if (name.compare(descr, Qt::CaseInsensitive) == 0) {
             descr = "";
         }
         else {
@@ -1481,19 +1471,22 @@ void ConnectorItem::updateTooltip() {
         }
         QString id = connectorSharedID();
         int ix = IntegerFinder.indexIn(id);
-        if (ix < 0) {
+        if (ix < 0 || isInt) {
             id = "";
         }
         else {
             if (attachedTo()->modelPart()->hasZeroConnector()) {
-                id = QString::number(IntegerFinder.cap(0).toInt() + 1) + " ";
+                id = QString::number(IntegerFinder.cap(0).toInt() + 1);
             }
             else {
-                id = IntegerFinder.cap(0) + " ";
+                id = IntegerFinder.cap(0);
+            }
+            if (!id.isEmpty()) {
+                id = " <span style='color:#909090;'>(" + id + ")</span>";
             }
         }
        
-        QString tt = QString("%1<b>%2</b>%3<br /><span style='font-size:small;'>%4</span>")
+        QString tt = QString("<b>%2</b>%3%1<br /><span style='font-size:small;'>%4</span>")
                 .arg(id)
                 .arg(name)
                 .arg(descr)
@@ -1542,9 +1535,9 @@ bool ConnectorItem::connectionIsAllowed(ConnectorItem * other) {
 	return true;
 }
 
-void ConnectorItem::showEqualPotential(bool show) {
+void ConnectorItem::showEqualPotential(bool show, QList<ConnectorItem *> & visited) {
 	if (!show) {
-		restoreColor(false, 0, true);
+		restoreColor(visited);
 		return;
 	}
 
@@ -1557,8 +1550,9 @@ void ConnectorItem::showEqualPotential(bool show) {
 }
 
 void ConnectorItem::clearEqualPotentialDisplay() {
+    QList<ConnectorItem *> visited;
 	foreach (ConnectorItem * connectorItem, m_equalPotentialDisplayItems) {
-		connectorItem->showEqualPotential(false);
+		connectorItem->restoreColor(visited);
 	}
 	m_equalPotentialDisplayItems.clear();
 }
@@ -1597,21 +1591,21 @@ bool ConnectorItem::isGrounded() {
 ConnectorItem * ConnectorItem::getCrossLayerConnectorItem() {
 	if (m_connector == NULL) return NULL;
 	if (m_attachedTo == NULL) return NULL;
-	if (m_attachedTo->viewIdentifier() != ViewLayer::PCBView) return NULL;
+	if (m_attachedTo->viewID() != ViewLayer::PCBView) return NULL;
 
 	ViewLayer::ViewLayerID viewLayerID = attachedToViewLayerID();
 	if (viewLayerID == ViewLayer::Copper0) {
-		return m_connector->connectorItemByViewLayerID(this->attachedToViewIdentifier(), ViewLayer::Copper1);
+		return m_connector->connectorItemByViewLayerID(this->attachedToViewID(), ViewLayer::Copper1);
 	}
 	if (viewLayerID == ViewLayer::Copper1) {
-		return m_connector->connectorItemByViewLayerID(this->attachedToViewIdentifier(), ViewLayer::Copper0);
+		return m_connector->connectorItemByViewLayerID(this->attachedToViewID(), ViewLayer::Copper0);
 	}
 
 	return NULL;
 }
 
-bool ConnectorItem::isInLayers(ViewLayer::ViewLayerSpec viewLayerSpec) {
-	return ViewLayer::copperLayers(viewLayerSpec).contains(attachedToViewLayerID());
+bool ConnectorItem::isInLayers(ViewLayer::ViewLayerPlacement viewLayerPlacement) {
+	return ViewLayer::copperLayers(viewLayerPlacement).contains(attachedToViewLayerID());
 }
 
 bool ConnectorItem::isCrossLayerConnectorItem(ConnectorItem * candidate) {
@@ -1647,7 +1641,7 @@ void ConnectorItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * o
 
 	if (m_effectively == EffectivelyUnknown) {
 		if (!m_circular && m_shape.isEmpty()) {
-			if (this->attachedTo()->viewIdentifier() == ViewLayer::PCBView) {
+			if (this->attachedTo()->viewID() == ViewLayer::PCBView) {
 				QRectF r = rect();
 				if (qAbs(r.width() - r.height()) < 0.01) m_effectively = EffectivelyCircular;
 				else m_effectively = EffectivelyRectangular;
@@ -1754,20 +1748,20 @@ void ConnectorItem::paintLeg(QPainter * painter, bool hasCurves)
 }
 
 
-ConnectorItem * ConnectorItem::chooseFromSpec(ViewLayer::ViewLayerSpec viewLayerSpec) {
+ConnectorItem * ConnectorItem::chooseFromSpec(ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	ConnectorItem * crossConnectorItem = getCrossLayerConnectorItem();
 	if (crossConnectorItem == NULL) return this;
 
 	ViewLayer::ViewLayerID basis = ViewLayer::Copper0;
-	switch (viewLayerSpec) {
-		case ViewLayer::Top:
+	switch (viewLayerPlacement) {
+		case ViewLayer::NewTop:
 			basis = ViewLayer::Copper1;
 			break;
-		case ViewLayer::Bottom:
+		case ViewLayer::NewBottom:
 			basis = ViewLayer::Copper0;
 			break;
 		default:
-			DebugDialog::debug(QString("unusual viewLayerSpec %1").arg(viewLayerSpec));
+			DebugDialog::debug(QString("unusual viewLayerPlacement %1").arg(viewLayerPlacement));
 			basis = ViewLayer::Copper0;
 			break;
 	}
@@ -1879,7 +1873,7 @@ void ConnectorItem::clearRatsnestDisplay(QList<ConnectorItem *> & connectorItems
 			c2->tempRemove(vw->connector1(), false);
 		}
 
-		vw->debugInfo("removing rat 1");
+		//vw->debugInfo("removing rat 1");
 		vw->scene()->removeItem(vw);
 		delete vw;
 	}
@@ -1896,14 +1890,6 @@ void ConnectorItem::collectConnectorNames(QList<ConnectorItem *> & connectorItem
 	}
 }
 
-bool ConnectorItem::marked() {
-	return m_marked;
-}
-
-void ConnectorItem::setMarked(bool m) {
-	m_marked = m;
-}
-
 double ConnectorItem::calcClipRadius() {
 	if (m_circular) {
 		return radius() - (strokeWidth() / 2.0);
@@ -1925,17 +1911,17 @@ void ConnectorItem::debugInfo(const QString & msg)
 {
 
 #ifndef QT_NO_DEBUG
-	QString s = QString("%1 cid:%2 %3 %4 id:%5 %6 vlid:%7 vid:%8 spec:%9 flg:%10 hy:%11 bus:%12 r:%13 sw:%14")
-	//QString s = QString("%1 %2 %3 %4 id:%5 %6 vlid:%7 vid:%8")
+	QString s = QString("%1 cid:%2 cname:%3 title:%4 id:%5 type:%6 inst:%7 vlid:%8 vid:%9 spec:%10 flg:%11 hy:%12 bus:%13 r:%14 sw:%15")
 			.arg(msg)
 			.arg(this->connectorSharedID())
 			.arg(this->connectorSharedName())
 			.arg(this->attachedToTitle())
 			.arg(this->attachedToID())
+			.arg(this->attachedToItemType())
 			.arg(this->attachedToInstanceTitle())
 			.arg(this->attachedToViewLayerID())
-			.arg(this->attachedToViewIdentifier())
-			.arg(this->attachedToViewLayerSpec())
+			.arg(this->attachedToViewID())
+			.arg(this->attachedToViewLayerPlacement())
 			.arg(this->attachedTo()->wireFlags()) 
 			.arg(this->m_hybrid)
 			.arg((long) this->bus(), 0, 16)
@@ -2065,8 +2051,9 @@ ConnectorItem * ConnectorItem::releaseDrag() {
 
 		// clean up
 		setOverConnectorItem(NULL);
-		clearConnectorHover();
-		restoreColor(false, 0, true);
+		clearConnectorHover();    
+        QList<ConnectorItem *> visited;
+		restoreColor(visited);
 	}
 	attachedTo()->clearConnectorHover();
 	return result;
@@ -2349,7 +2336,7 @@ double ConnectorItem::findT(Bezier * bezier, double blen, double length)
 	return t;
 }
 
-const QString & ConnectorItem::legID(ViewLayer::ViewIdentifier viewID, ViewLayer::ViewLayerID viewLayerID) {
+const QString & ConnectorItem::legID(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
 	if (m_connector) return m_connector->legID(viewID, viewLayerID);
 
 	return ___emptyString___;
@@ -2853,3 +2840,9 @@ void ConnectorItem::setGroundFillSeed(bool seed)
 	if (cross) cross->m_groundFillSeed = seed;
 }
 
+ViewGeometry::WireFlags ConnectorItem::getSkipFlags() {
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+    if (infoGraphicsView == NULL) return ViewGeometry::RatsnestFlag;
+
+    return (ViewGeometry::RatsnestFlag | ViewGeometry::NormalFlag | ViewGeometry::PCBTraceFlag | ViewGeometry::SchematicTraceFlag) ^ infoGraphicsView->getTraceFlag();
+}
