@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6503 $:
+$Revision: 6975 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-02 17:20:22 +0200 (Tue, 02 Oct 2012) $
+$Date: 2013-04-19 12:51:22 +0200 (Fr, 19. Apr 2013) $
 
 ********************************************************************/
 
@@ -62,10 +62,10 @@ $Date: 2012-10-02 17:20:22 +0200 (Tue, 02 Oct 2012) $
 #include "items/partfactory.h"
 #include "items/propertydef.h"
 #include "dialogs/recoverydialog.h"
-#include "lib/qtsysteminfo/QtSystemInfo.h"
 #include "processeventblocker.h"
-#include "autoroute/cmrouter/panelizer.h"
+#include "autoroute/panelizer.h"
 #include "sketch/sketchwidget.h"
+#include "sketch/pcbsketchwidget.h"
 
 // dependency injection :P
 #include "referencemodel/sqlitereferencemodel.h"
@@ -86,8 +86,6 @@ $Date: 2012-10-02 17:20:22 +0200 (Tue, 02 Oct 2012) $
 #include <QNetworkRequest>
 #include <QMultiHash>
 
-static QNetworkAccessManager * NetworkAccessManager = NULL;
-
 #ifdef LINUX_32
 #define PLATFORM_NAME "linux-32bit"
 #endif
@@ -111,9 +109,6 @@ static QNetworkAccessManager * NetworkAccessManager = NULL;
 #endif
 #endif
 
-QSet<QString> InstalledFonts::InstalledFontsList;
-QMultiHash<QString, QString> InstalledFonts::InstalledFontsNameMapper;   // family name to filename; SVG files seem to have to use filename
-
 static const double LoadProgressStart = 0.085;
 static const double LoadProgressEnd = 0.6;
 
@@ -353,6 +348,13 @@ bool FApplication::init() {
 			toRemove << i << i + 1;
 		}
 
+		if ((m_arguments[i].compare("-drc", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("--drc", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = DRCService;
+			m_outputFolder = m_arguments[i + 1];
+			toRemove << i << i + 1;
+		}
+
 		if ((m_arguments[i].compare("-db", Qt::CaseInsensitive) == 0) ||
             (m_arguments[i].compare("-database", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--database", Qt::CaseInsensitive) == 0)) {
@@ -565,6 +567,7 @@ bool FApplication::spaceBarIsPressed() {
 bool FApplication::eventFilter(QObject *obj, QEvent *event)
 {
 	// check whether the space bar is down.
+    //qDebug() << "event" << event->type();
 
 	Q_UNUSED(obj);
 
@@ -650,7 +653,8 @@ bool FApplication::findTranslator(const QString & translationsPath) {
 		QLocale::setDefault(QLocale(suffix));
 	}
 
-    bool loaded = m_translator.load(QString("fritzing_") + suffix, translationsPath);
+    bool loaded = m_translator.load(QString("fritzing_") + suffix.toLower(), translationsPath);
+    DebugDialog::debug(QString("translation %1 loaded %2 from %3").arg(suffix).arg(loaded).arg(translationsPath));
 	if (loaded) {
 		QApplication::installTranslator(&m_translator);
 	}
@@ -662,8 +666,7 @@ void FApplication::registerFonts() {
 	registerFont(":/resources/fonts/DroidSans.ttf", true);
 	registerFont(":/resources/fonts/DroidSans-Bold.ttf", false);
 	registerFont(":/resources/fonts/DroidSansMono.ttf", false);
-	//registerFont(":/resources/fonts/ocra10.ttf", true);
-	registerFont(":/resources/fonts/OCRA.otf", true);
+	registerFont(":/resources/fonts/OCRA.ttf", true);
 
 	/*	
 		QFontDatabase database;
@@ -679,19 +682,27 @@ ReferenceModel * FApplication::loadReferenceModel(const QString & databaseName,
 	ItemBase::setReferenceModel(m_referenceModel);
 	connect(m_referenceModel, SIGNAL(loadedPart(int, int)), this, SLOT(loadedPart(int, int)));
 
-	bool ok = m_referenceModel->loadAll(databaseName, fullLoad);		// loads local parts, resource parts, and any other parts in files not in the db--these part override db parts with the same moduleID
+    bool dbExists = false;
+    QDir * dir = FolderUtils::getApplicationSubFolder("parts");
+    QString dbPath;
+    if (dir) {
+        dbPath = dir->absoluteFilePath("parts.db");
+        QFileInfo info(dbPath);
+        dbExists = info.exists();
+    }
+
+	bool ok = m_referenceModel->loadAll(databaseName, fullLoad, dbExists);		// loads local parts, resource parts, and any other parts in files not in the db--these part override db parts with the same moduleID
     if (ok && databaseName.isEmpty()) {
-        QDir * dir = FolderUtils::getApplicationSubFolder("parts");
         if (dir == NULL) {
         }
         else {
             QFile file(dir->absoluteFilePath("parts.db"));
             if (file.exists()) {
-                m_referenceModel->loadFromDB(dir->absoluteFilePath("parts.db"));
+                m_referenceModel->loadFromDB(dbPath);
             }
-            delete dir;
         }
     }
+    delete dir;
 
 	return m_referenceModel;
 }
@@ -716,6 +727,10 @@ int FApplication::serviceStartup() {
 			runGedaService();
 			return 0;
 	
+		case DRCService:
+			runDRCService();
+			return 0;
+	
 		case DatabaseService:
 			runDatabaseService();
 			return 0;
@@ -811,10 +826,10 @@ void FApplication::runSvgServiceAux()
 		FolderUtils::setOpenSaveFolderAux(m_outputFolder);
 		if (mainWindow->loadWhich(filepath, false, false, "")) {
             QFileInfo info(filepath);
-            QList<ViewLayer::ViewIdentifier> ids;
+            QList<ViewLayer::ViewID> ids;
             ids << ViewLayer::BreadboardView << ViewLayer::SchematicView << ViewLayer::PCBView;
-            foreach (ViewLayer::ViewIdentifier id, ids) {
-                QString fn = QString("%1_%2.svg").arg(info.completeBaseName()).arg(ViewLayer::viewIdentifierNaturalName(id));
+            foreach (ViewLayer::ViewID id, ids) {
+                QString fn = QString("%1_%2.svg").arg(info.completeBaseName()).arg(ViewLayer::viewIDNaturalName(id));
                 QString svgPath = dir.absoluteFilePath(fn);
                 mainWindow->setCurrentView(id);
                 mainWindow->exportSvg(GraphicsUtils::StandardFritzingDPI, false, false, svgPath);
@@ -828,6 +843,8 @@ void FApplication::runSvgServiceAux()
 
 void FApplication::runDatabaseService()
 {
+	createUserDataStoreFolderStructure();
+
     DebugDialog::setEnabled(true);
     QDir * parent = FolderUtils::getApplicationSubFolder("pdb");
     QFileInfoList dirs = parent->entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
@@ -876,6 +893,62 @@ void FApplication::runGedaService() {
 	}
 }
 
+
+void FApplication::runDRCService() {
+    m_started = true;
+    initService();
+    DebugDialog::setEnabled(true);
+
+    try {
+		QDir dir(m_outputFolder);
+		QStringList filters;
+		filters << "*.fzz";
+		QStringList filenames = dir.entryList(filters, QDir::Files);
+		foreach (QString filename, filenames) {
+			QString filepath = dir.absoluteFilePath(filename);
+	        MainWindow * mainWindow = openWindowForService(false);
+            if (mainWindow == NULL) continue;
+
+            mainWindow->setCloseSilently(true);
+
+	        if (!mainWindow->loadWhich(filepath, false, false, "")) {
+		        DebugDialog::debug(QString("failed to load '%1'").arg(filepath));
+		        mainWindow->close();
+                delete mainWindow;
+                continue;
+	        }
+
+	        mainWindow->showPCBView();
+
+            int moved = mainWindow->pcbView()->checkLoadedTraces();
+            if (moved > 0) {
+                QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("%1 wires moved from their saved position in %2.").arg(moved).arg(filepath));
+                DebugDialog::debug(QString("\ncheckloadedtraces %1\n").arg(filepath)); 
+            }
+	
+
+            Panelizer::checkDonuts(mainWindow, true);
+            Panelizer::checkText(mainWindow, true);
+
+            QList<ItemBase *> boards = mainWindow->pcbView()->findBoard();	
+	        foreach (ItemBase * boardItem, boards) {
+                mainWindow->pcbView()->selectAllItems(false, false);
+                boardItem->setSelected(true);
+                mainWindow->newDesignRulesCheck(false);
+            }
+
+            mainWindow->close();
+            delete mainWindow;
+		}
+	}
+	catch (const QString & msg) {
+		DebugDialog::debug(msg);
+	}
+	catch (...) {
+		DebugDialog::debug("who knows");
+	}
+}
+
 void FApplication::runKicadFootprintService() {
 	QDir dir(m_outputFolder);
 	QStringList filters;
@@ -1000,6 +1073,8 @@ int FApplication::startup()
 
 	cleanFzzs();
 
+    createUserDataStoreFolderStructure();
+
 	loadReferenceModel("", false);
 
 	QString prevVersion;
@@ -1010,24 +1085,27 @@ int FApplication::startup()
 		QString currVersion = Version::versionString();
 		if(prevVersion != currVersion) {
 			QVariant pid = settings.value("pid");
+            QVariant language = settings.value("language");
 			settings.clear();
 			if (!pid.isNull()) {
 				settings.setValue("pid", pid);
 			}
+			if (!language.isNull()) {
+				settings.setValue("language", language);
+			}
 		}
 	}
 
 	//bool fabEnabled = settings.value(ORDERFABENABLED, QVariant(false)).toBool();
 	//if (!fabEnabled) {
-		NetworkAccessManager = new QNetworkAccessManager(this);
-		connect(NetworkAccessManager, SIGNAL(finished(QNetworkReply *)), this, SLOT(gotOrderFab(QNetworkReply *)));
-
-		NetworkAccessManager->get(QNetworkRequest(QUrl(QString("http://fab.fritzing.org/launched%1").arg(makeRequestParamsString()))));
+		QNetworkAccessManager * manager = new QNetworkAccessManager(this);
+		connect(manager, SIGNAL(finished(QNetworkReply *)), this, SLOT(gotOrderFab(QNetworkReply *)));
+		manager->get(QNetworkRequest(QUrl(QString("http://fab.fritzing.org/launched%1").arg(Version::makeRequestParamsString(true)))));
 	//}
 
 	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressEnd);
 
-	createUserDataStoreFolderStructure();
+
 
 	//DebugDialog::debug("after createUserDataStoreFolderStructure");
 
@@ -1060,7 +1138,7 @@ void FApplication::registerFont(const QString &fontFile, bool reallyRegister) {
 		QStringList familyNames = QFontDatabase::applicationFontFamilies(id);
 		QFileInfo finfo(fontFile);
 		foreach (QString family, familyNames) {
-			InstalledFonts::InstalledFontsNameMapper.insert(family, finfo.baseName());
+			InstalledFonts::InstalledFontsNameMapper.insert(family, finfo.completeBaseName());
 			InstalledFonts::InstalledFontsList << family;
 			//DebugDialog::debug("registering font family: "+family);
 		}
@@ -1219,7 +1297,7 @@ void FApplication::initSplash(FSplashScreen & splash) {
 
 struct Thing {
         QString moduleID;
-        ViewLayer::ViewIdentifier viewIdentifier;
+        ViewLayer::ViewID viewID;
         ViewLayer::ViewLayerID viewLayerID;
 };
 
@@ -1252,7 +1330,7 @@ void FApplication::checkForUpdates(bool atUserRequest)
 
     QString atom = QString("http://fritzing.org/download/feed/atom/%1/%2")
 		.arg(PLATFORM_NAME)
-		.arg(makeRequestParamsString());
+		.arg(Version::makeRequestParamsString(true));
     DebugDialog::debug(atom);
     versionChecker->setUrl(atom);
 	m_updateDialog->setAtUserRequest(atUserRequest);
@@ -1265,14 +1343,14 @@ void FApplication::checkForUpdates(bool atUserRequest)
 
 void FApplication::enableCheckUpdates(bool enabled)
 {
-	DebugDialog::debug("before enable check updates");
+	//DebugDialog::debug("before enable check updates");
     foreach (QWidget *widget, QApplication::topLevelWidgets()) {
         MainWindow *mainWindow = qobject_cast<MainWindow *>(widget);
 		if (mainWindow) {
 			mainWindow->enableCheckUpdates(enabled);
 		}
 	}
-	DebugDialog::debug("after enable check updates");
+	//DebugDialog::debug("after enable check updates");
 }
 
 
@@ -1481,6 +1559,11 @@ void FApplication::loadSomething(const QString & prevVersion) {
             DebugDialog::debug(QString("Loading non-service file %1").arg(filename));
             MainWindow *mainWindow = MainWindow::newMainWindow(m_referenceModel, filename, true, true);
             mainWindow->loadWhich(filename, true, true, "");
+            if (filename.endsWith(FritzingSketchExtension) || filename.endsWith(FritzingBundleExtension)) {
+            }
+            else {
+                mainWindow->addDefaultParts();
+            }
             sketchesToLoad << mainWindow;
         }
 	}
@@ -1643,30 +1726,8 @@ void FApplication::gotOrderFab(QNetworkReply * networkReply) {
 		QSettings settings;
 		settings.setValue(ORDERFABENABLED, QVariant(true));
 	}
-}
-
-QString FApplication::makeRequestParamsString() {
-	QSettings settings;
-	if (settings.value("pid").isNull()) {
-		settings.setValue("pid", FolderUtils::getRandText());
-	}
-
-	QtSystemInfo systemInfo(this);
-	QString siVersion(QUrl::toPercentEncoding(Version::versionString()));
-	QString siSystemName(QUrl::toPercentEncoding(systemInfo.systemName()));
-	QString siSystemVersion(QUrl::toPercentEncoding(systemInfo.systemVersion()));
-	QString siKernelName(QUrl::toPercentEncoding(systemInfo.kernelName()));
-	QString siKernelVersion(QUrl::toPercentEncoding(systemInfo.kernelVersion()));
-	QString siArchitecture(QUrl::toPercentEncoding(systemInfo.architectureName()));
-    QString string = QString("?pid=%1&version=%2&sysname=%3&kernname=%4&kernversion=%5arch=%6&sysversion=%7")
-		.arg(settings.value("pid").toString())
-		.arg(siVersion)
-		.arg(siSystemName)
-		.arg(siKernelName)
-		.arg(siKernelVersion)
-		.arg(siArchitecture)
-		.arg(siSystemVersion);
-	return string;
+    networkReply->manager()->deleteLater();
+    networkReply->deleteLater();
 }
 
 void FApplication::runPanelizerService()
@@ -1678,7 +1739,13 @@ void FApplication::runPanelizerService()
 void FApplication::runInscriptionService()
 {	
 	m_started = true;
-	Panelizer::inscribe(this, m_panelFilename);
+    bool drc = false;
+    foreach (QString arg, m_arguments) {
+        if (arg.compare("-drc", Qt::CaseInsensitive) == 0) {
+            drc = true;
+        }
+    }
+	Panelizer::inscribe(this, m_panelFilename, drc);
 }
 
 QList<MainWindow *> FApplication::orderedTopLevelMainWindows() {
@@ -1764,7 +1831,7 @@ void FApplication::doCommand(const QString & command, const QString & params, QS
 
 	QString subfolder = params;
 	if (command.endsWith("tcp")) {
-		subfolder = FolderUtils::getRandText();
+		subfolder = TextUtils::getRandText();
 		dir.mkdir(subfolder);
 	}
 
