@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5767 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-01-21 15:50:32 +0100 (Sat, 21 Jan 2012) $
+$Revision: 6968 $:
+$Author: irascibl@gmail.com $:
+$Date: 2013-04-15 17:44:14 +0200 (Mo, 15. Apr 2013) $
 
 ********************************************************************/
 
@@ -158,10 +158,16 @@ QString("<br /><br /><br /><br /><br /><br /><br /><br /><br />") +
 	tr("to Hiroshi Suzuki for the Japanese translation, ") +
 	tr("to Robert Lee for the Chinese (Traditional) translation, ") +
 	tr("to Vladimir Savinov for the Russian translation, " ) +
-	tr("to Steven Noppe for the Dutch translation, " ) +
+	tr("to Steven Noppe and Davy Uittenbogerd for the Dutch translation, " ) +
 	tr("to Josef Dustira for the Czech translation, " ) +
 	tr("to Jinbuhm Kim for the Korean translation, " ) +
-	tr("and to Arthur Zanona, Nuno Pessanha Santos, and Leandro Nunes for the Portuguese (Brazilian) translation. " ) +
+	tr("to &#313;ubom&iacute;r Ducho&#328; for the Slovak translation, " ) + 
+    tr("to Alexander Kaltsas for the Greek translation, " ) +
+    tr("to Lionel Michel and Yvan Kloster for the French translation, " ) +
+    tr("to Cihan Mete Bahad&#x0131;r for the Turkish translation, " ) +
+    tr("to Nikolay Stankov for the Bulgarian translation, " ) +
+   
+	tr("and to Arthur Zanona, Nuno Pessanha Santos, Leandro Nunes, and Gabriel Ferreira for the Portuguese (Brazilian) translation. " ) +
 "</p>" +
 
 "<p>" +
@@ -169,7 +175,7 @@ QString("<br /><br /><br /><br /><br /><br /><br /><br /><br />") +
 	tr("MWFK Brandenburg, the sponsorship of the Design ") +
 	tr("Department of Bauhaus-University Weimar, ") +
 	tr("IxDS, an anonymous donor, Parallax, Picaxe, Sparkfun, ") +
-	tr("and from each purchase of a Fritzing Starter Kit.") +
+	tr("and from each purchase of a Fritzing Starter Kit or a PCB from Fritzing Fab.") +
 "</p>" +
 
 "<p>" +
