@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6506 $:
+$Revision: 6984 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-03 13:11:12 +0200 (Wed, 03 Oct 2012) $
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -40,6 +40,8 @@ $Date: 2012-10-03 13:11:12 +0200 (Wed, 03 Oct 2012) $
 #include "../utils/flineedit.h"
 #include "../items/moduleidnames.h"
 #include "../items/paletteitem.h"
+#include "../utils/clickablelabel.h"
+#include "../utils/textutils.h"
 
 
 #define HTML_EOF "</body>\n</html>"
@@ -50,6 +52,9 @@ const int HtmlInfoView::STANDARD_ICON_IMG_WIDTH = 32;
 const int HtmlInfoView::STANDARD_ICON_IMG_HEIGHT = 32;
 const int IconSpace = 0;
 
+static const int MaxSpinBoxWidth = 55;
+static const int AfterSpinBoxWidth = 5;
+
 /////////////////////////////////////
 
 QLabel * addLabel(QHBoxLayout * hboxLayout, QPixmap * pixmap) {
@@ -63,6 +68,11 @@ QLabel * addLabel(QHBoxLayout * hboxLayout, QPixmap * pixmap) {
 
 	return label;
 }
+
+QString format3(double d) {
+    return QString("%1").arg(d, 0, 'f', 3);
+}
+
 //////////////////////////////////////
 
 
@@ -100,8 +110,6 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 {
     this->setWidgetResizable(true);
 	this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    QFrame * mainFrame = new QFrame(this);
-	mainFrame->setObjectName("infoViewMainFrame");
 
 	m_lastTitleItemBase = NULL;
 	m_lastTagsModelPart = NULL;
@@ -110,6 +118,8 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_lastPropsModelPart = NULL;
 	m_lastPropsItemBase = NULL;
 
+    m_tinyMode = false;
+
 	m_partTitle = NULL;
 	m_partUrl = NULL;
 	m_partVersion = NULL;
@@ -119,11 +129,32 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_tagsTextLabel = NULL;
 	m_lastSwappingEnabled = false;
 	m_lastItemBase = NULL;
-	m_infoGraphicsView = NULL;
 	m_setContentTimer.setSingleShot(true);
 	m_setContentTimer.setInterval(10);
 	connect(&m_setContentTimer, SIGNAL(timeout()), this, SLOT(setContent()));
-	QVBoxLayout *vlo = new QVBoxLayout(mainFrame);
+
+	m_currentItem = NULL;
+	m_currentSwappingEnabled = false;
+
+    m_layerWidget = NULL;
+
+}
+
+
+HtmlInfoView::~HtmlInfoView() {
+	foreach (PropThing * propThing, m_propThings) {
+		delete propThing;
+	}
+	m_propThings.clear();
+}
+
+void HtmlInfoView::init(bool tinyMode) {
+    m_tinyMode = tinyMode;
+
+    QFrame * mainFrame = new QFrame(this);
+	mainFrame->setObjectName("infoViewMainFrame");
+
+    QVBoxLayout *vlo = new QVBoxLayout(mainFrame);
 	vlo->setSpacing(0);
 	vlo->setContentsMargins(0, 0, 0, 0);
 	vlo->setSizeConstraint( QLayout::SetMinAndMaxSize );
@@ -142,9 +173,11 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 
 	setInstanceTitleColors(m_titleEdit, QColor(0xaf, 0xaf, 0xb4), QColor(0x00, 0x00, 0x00)); //b3b3b3, 575757
 	m_titleEdit->setAutoFillBackground(true);
-	vlo->addWidget(m_titleEdit);
+	
+    vlo->addWidget(m_titleEdit);
+    if (tinyMode) m_titleEdit->setVisible(false);
 
-        /* Part Icons */
+    /* Part Icons */
 
 	if (NoIcon == NULL) {
 		NoIcon = new QPixmap(":/resources/images/icons/noicon.png");
@@ -176,22 +209,6 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	subVersionLayout->addStretch(1);
 	versionLayout->addLayout(subVersionLayout);
 
-    QHBoxLayout * cbLayout = new QHBoxLayout();
-	
-	m_lockCheckbox = new QCheckBox(tr("Locked"));
-	m_lockCheckbox->setObjectName("infoViewLockCheckbox");
-	m_lockCheckbox->setToolTip(tr("Change the locked state of the part in this view. A locked part can't be moved."));
-	connect(m_lockCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeLock(bool)));
-	cbLayout->addWidget(m_lockCheckbox);
-
-	m_stickyCheckbox = new QCheckBox(tr("Sticky"));
-	m_stickyCheckbox->setObjectName("infoViewLockCheckbox");
-	m_stickyCheckbox->setToolTip(tr("Change the \"sticky\" state of the part in this view. When a sticky part is moved, objects on top of it also move."));
-	connect(m_stickyCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeSticky(bool)));
-	cbLayout->addWidget(m_stickyCheckbox);
-
-    versionLayout->addLayout(cbLayout);
-
 	hboxLayout->addLayout(versionLayout);
 	
 	hboxLayout->addSpacerItem(new QSpacerItem(IconSpace, 1, QSizePolicy::Expanding));
@@ -203,16 +220,58 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_partUrl->setObjectName("infoViewPartUrl");
 	m_partUrl->setOpenExternalLinks(true);
 	vlo->addWidget(m_partUrl);
+    if (tinyMode) m_partUrl->setVisible(false);
 
 	m_partTitle = new TagLabel(this);
 	m_partTitle->setWordWrap(true);
 	m_partTitle->setObjectName("infoViewPartTitle");
 	m_partTitle->setOpenExternalLinks(true);
 	vlo->addWidget(m_partTitle);
+    if (tinyMode) m_partTitle->setVisible(false);
 
-	m_proplabel = new QLabel(tr("Properties"), NULL);
-	m_proplabel->setObjectName("expandableViewLabel");
-	vlo->addWidget(m_proplabel);
+    m_placementLabel = new QLabel(tr("Placement"), NULL);
+	m_placementLabel->setObjectName("expandableViewLabel");
+	vlo->addWidget(m_placementLabel);
+
+    m_placementFrame = new QFrame(this);
+	m_placementFrame->setObjectName("infoViewPropertyFrame");
+	m_placementLayout = new QGridLayout(m_placementFrame);
+	m_placementLayout->setSpacing(0);
+	m_placementLayout->setContentsMargins(0, 0, 0, 0);
+
+	m_layerLabel = new QLabel(tr("pcb layer"), this);
+	m_layerLabel->setObjectName("propNameLabel");
+	m_layerLabel->setWordWrap(true);
+	m_placementLayout->addWidget(m_layerLabel, 0, 0);
+
+	m_layerFrame = new QFrame(this);
+	m_layerFrame->setObjectName("propValueFrame");
+	m_layerLayout = new QVBoxLayout(m_layerFrame);
+	m_layerLayout->setSpacing(0);
+	m_layerLayout->setContentsMargins(0, 0, 0, 0);
+    m_layerFrame->setLayout(m_layerLayout);
+
+    m_placementLayout->addWidget(m_layerFrame, 0, 1);
+
+    makeLocationFrame();
+    m_placementLayout->addWidget(m_locationLabel, 1, 0);
+    m_placementLayout->addWidget(m_locationFrame, 1, 1);
+
+    makeRotationFrame();
+    m_placementLayout->addWidget(m_rotationLabel, 2, 0);
+    m_placementLayout->addWidget(m_rotationFrame, 2, 1);
+
+    makeLockFrame();
+    m_placementLayout->addWidget(m_lockLabel, 3, 0);
+    m_placementLayout->addWidget(m_lockFrame, 3, 1);
+
+
+	m_placementFrame->setLayout(m_placementLayout);
+	vlo->addWidget(m_placementFrame);
+
+	m_propLabel = new QLabel(tr("Properties"), NULL);
+	m_propLabel->setObjectName("expandableViewLabel");
+	vlo->addWidget(m_propLabel);
 
 	m_propFrame = new QFrame(this);
 	m_propFrame->setObjectName("infoViewPropertyFrame");
@@ -222,18 +281,21 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_propFrame->setLayout(m_propLayout);
 	vlo->addWidget(m_propFrame);
 
-	m_taglabel = new QLabel(tr("Tags"), NULL);
-	m_taglabel->setObjectName("expandableViewLabel");
-	vlo->addWidget(m_taglabel);
+	m_tagLabel = new QLabel(tr("Tags"), NULL);
+	m_tagLabel->setObjectName("expandableViewLabel");
+	vlo->addWidget(m_tagLabel);
+    if (tinyMode) m_tagLabel->setVisible(false);
 
 	m_tagsTextLabel = new TagLabel(this);
 	m_tagsTextLabel->setWordWrap(true);
 	m_tagsTextLabel->setObjectName("tagsValue");
 	vlo->addWidget(m_tagsTextLabel);
+    if (tinyMode) m_tagsTextLabel->setVisible(false);
 
 	m_connLabel = new QLabel(tr("Connections"), NULL);
 	m_connLabel->setObjectName("expandableViewLabel");
 	vlo->addWidget(m_connLabel);
+    if (tinyMode) m_connLabel->setVisible(false);
 
 	m_connFrame = new QFrame(this);
 	m_connFrame->setObjectName("connectionsFrame");
@@ -266,23 +328,15 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 
 	vlo->addWidget(m_connFrame);
 
+    if (m_tinyMode) m_connFrame->setVisible(false);
+
 	vlo->addSpacerItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding));
 
 	mainFrame->setLayout(vlo);
 
 	this->setWidget(mainFrame);
-
-	m_currentItem = NULL;
-	m_currentSwappingEnabled = false;
-
 }
 
-HtmlInfoView::~HtmlInfoView() {
-	foreach (PropThing * propThing, m_propThings) {
-		delete propThing;
-	}
-	m_propThings.clear();
-}
 
 void HtmlInfoView::cleanup() {
 	if (NoIcon) {
@@ -291,28 +345,25 @@ void HtmlInfoView::cleanup() {
 	}
 }
 
-void HtmlInfoView::viewItemInfo(InfoGraphicsView * infoGraphicsView, ItemBase* item, bool swappingEnabled) 
+void HtmlInfoView::viewItemInfo(InfoGraphicsView *, ItemBase* item, bool swappingEnabled) 
 {
 	m_setContentTimer.stop();
-	m_pendingInfoGraphicsView = infoGraphicsView;
 	m_lastItemBase = m_pendingItemBase = item;
 	m_lastSwappingEnabled = m_pendingSwappingEnabled = swappingEnabled;
 	m_setContentTimer.start();
 }
 
-void HtmlInfoView::hoverEnterItem(InfoGraphicsView * infoGraphicsView, QGraphicsSceneHoverEvent *, ItemBase * item, bool swappingEnabled) {
+void HtmlInfoView::hoverEnterItem(InfoGraphicsView *, QGraphicsSceneHoverEvent *, ItemBase * item, bool swappingEnabled) {
 	m_setContentTimer.stop();
-	m_pendingInfoGraphicsView = infoGraphicsView;
 	m_pendingItemBase = item;
 	m_pendingSwappingEnabled = swappingEnabled;
 	m_setContentTimer.start();
 }
 
-void HtmlInfoView::hoverLeaveItem(InfoGraphicsView * infoGraphicsView, QGraphicsSceneHoverEvent *, ItemBase * itemBase){
+void HtmlInfoView::hoverLeaveItem(InfoGraphicsView *, QGraphicsSceneHoverEvent *, ItemBase * itemBase){
 	Q_UNUSED(itemBase);
 	//DebugDialog::debug(QString("hoverLeaveItem itembase %1").arg(itemBase ? itemBase->instanceTitle() : "NULL"));
 	m_setContentTimer.stop();
-	m_pendingInfoGraphicsView = infoGraphicsView;
 	m_pendingItemBase = m_lastItemBase;
 	m_pendingSwappingEnabled = m_lastSwappingEnabled;
 	m_setContentTimer.start();
@@ -387,13 +438,22 @@ void HtmlInfoView::appendWireStuff(Wire* wire, bool swappingEnabled) {
 		 nameString = modelPart->description();
 	}
 	partTitle(nameString, modelPart->version(), modelPart->url(), modelPart->isObsolete());
-	m_lockCheckbox->setVisible(false);
-	m_stickyCheckbox->setVisible(false);
+    m_lockFrame->setVisible(false);
+    m_lockLabel->setVisible(false);
+    m_locationFrame->setVisible(false);
+    m_locationLabel->setVisible(false);
+    m_rotationFrame->setVisible(false);
+    m_rotationLabel->setVisible(false);
 
 	setUpTitle(wire);
 	setUpIcons(wire, swappingEnabled);
 
 	displayProps(modelPart, wire, swappingEnabled);
+
+    bool hasLayer = (wire->viewID() == ViewLayer::PCBView) && wire->getViewGeometry().getPCBTrace();
+    m_placementFrame->setVisible(hasLayer);
+	m_placementLabel->setVisible(hasLayer);
+
 	addTags(modelPart);
 }
 
@@ -420,8 +480,21 @@ void HtmlInfoView::appendItemStuff(ItemBase * itemBase, ModelPart * modelPart, b
 		nameString = modelPart->description();
 	}
 	partTitle(nameString, modelPart->version(), modelPart->url(), modelPart->isObsolete());
-	m_lockCheckbox->setVisible(true);
+    m_lockFrame->setVisible(swappingEnabled);
+    m_lockLabel->setVisible(swappingEnabled);
 	m_lockCheckbox->setChecked(itemBase->moveLock());
+
+    m_locationFrame->setVisible(swappingEnabled);
+    m_locationLabel->setVisible(swappingEnabled);
+    m_locationFrame->setDisabled(itemBase->moveLock());
+
+    m_rotationFrame->setVisible(swappingEnabled);
+    m_rotationLabel->setVisible(swappingEnabled);
+    m_rotationFrame->setEnabled(itemBase->rotationAllowed());
+
+    setLocation(itemBase);
+    setRotation(itemBase);
+
     if (itemBase->isBaseSticky()) {
 	    m_stickyCheckbox->setVisible(true);
         m_stickyCheckbox->setChecked(itemBase->isSticky());
@@ -432,6 +505,10 @@ void HtmlInfoView::appendItemStuff(ItemBase * itemBase, ModelPart * modelPart, b
 
 	displayProps(modelPart, itemBase, swappingEnabled);
 	addTags(modelPart);
+
+    m_placementLabel->setVisible(swappingEnabled);
+    m_placementFrame->setVisible(swappingEnabled);
+
 }
 
 void HtmlInfoView::setContent()
@@ -449,12 +526,13 @@ void HtmlInfoView::setContent()
 
 	appendStuff(m_pendingItemBase, m_pendingSwappingEnabled);
 	setCurrentItem(m_pendingItemBase);
-	m_infoGraphicsView = m_pendingInfoGraphicsView;
 
-	m_connFrame->setVisible(true);
-	m_proplabel->setVisible(true);
-	m_taglabel->setVisible(true);
-	m_connLabel->setVisible(true);
+    if (!m_tinyMode) {
+	    m_connFrame->setVisible(m_pendingSwappingEnabled);
+	    m_tagLabel->setVisible(true);
+	    m_connLabel->setVisible(m_pendingSwappingEnabled);
+    }
+	m_propLabel->setVisible(true);
 	m_propFrame->setVisible(true);
 
 	m_setContentTimer.stop();
@@ -500,16 +578,16 @@ void HtmlInfoView::setNullContent()
 {
 	setUpTitle(NULL);
 	partTitle("", "", "", false);
-	m_lockCheckbox->setVisible(false);
-	m_stickyCheckbox->setVisible(false);
 	setUpIcons(NULL, false);
 	displayProps(NULL, NULL, false);
 	addTags(NULL);
 	viewConnectorItemInfo(NULL, NULL);
 	m_connFrame->setVisible(false);
 	m_propFrame->setVisible(false);
-	m_proplabel->setVisible(false);
-	m_taglabel->setVisible(false);
+	m_propLabel->setVisible(false);
+	m_placementFrame->setVisible(false);
+	m_placementLabel->setVisible(false);
+	m_tagLabel->setVisible(false);
 	m_connLabel->setVisible(false);
 }
 
@@ -517,11 +595,13 @@ void HtmlInfoView::setInstanceTitle() {
 	FLineEdit * edit = qobject_cast<FLineEdit *>(sender());
 	if (edit == NULL) return;
 	if (!edit->isEnabled()) return;
-	if (m_infoGraphicsView == NULL) return;
 	if (m_currentItem == NULL) return;
 
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(m_currentItem);
+    if (infoGraphicsView == NULL) return;
+
 	DebugDialog::debug(QString("set instance title to %1").arg(edit->text()));
-	m_infoGraphicsView->setInstanceTitle(m_currentItem->id(), m_partTitle->text(), edit->text(), true, false);
+	infoGraphicsView->setInstanceTitle(m_currentItem->id(), m_partTitle->text(), edit->text(), true, false);
 }
 
 void HtmlInfoView::instanceTitleEnter() {
@@ -638,14 +718,16 @@ void HtmlInfoView::partTitle(const QString & title, const QString & version, con
 	m_lastPartTitle = title;
 	m_lastPartVersion = version;
 
-	if (url.isEmpty()) {
-		m_partUrl->setVisible(false);	
-		m_partUrl->setText("");
-	}
-	else {
-		m_partUrl->setText(QString("<a href=\"%1\">%1</a>").arg(url));
-		m_partUrl->setVisible(true);
-	}
+    if (!m_tinyMode) {
+	    if (url.isEmpty()) {
+		    m_partUrl->setVisible(false);	
+		    m_partUrl->setText("");
+	    }
+	    else {
+		    m_partUrl->setText(QString("<a href=\"%1\">%1</a>").arg(url));
+		    m_partUrl->setVisible(true);
+	    }
+    }
 
 	m_partTitle->setText(title);
 	if (!version.isEmpty()) {
@@ -677,6 +759,14 @@ void HtmlInfoView::displayProps(ModelPart * modelPart, ItemBase * itemBase, bool
 	QHash<QString, QString> properties = getPartProperties(modelPart, itemBase, wantDebug, keys);
 	QString family = properties.value("family", "").toLower();
 
+    bool sl = false;
+    if (keys.contains("layer")) {
+        keys.removeOne("layer");
+        sl = (itemBase->viewID() == ViewLayer::PCBView);
+    }
+    
+    showLayers(sl, itemBase, family, properties.value("layer", ""), swappingEnabled);
+
 	int ix = 0;
 	foreach(QString key, keys) {
 		if (ix >= m_propThings.count()) {
@@ -717,14 +807,15 @@ void HtmlInfoView::displayProps(ModelPart * modelPart, ItemBase * itemBase, bool
 		QString resultKey, resultValue;
 		QWidget * resultWidget = oldPlugin;
 		bool result = false;
+        bool hide = false;
 		if (itemBase != NULL) {
-			result = itemBase->collectExtraInfo(propThing->m_name->parentWidget(), family, key, value, swappingEnabled, resultKey, resultValue, resultWidget);
+			result = itemBase->collectExtraInfo(propThing->m_name->parentWidget(), family, key, value, swappingEnabled, resultKey, resultValue, resultWidget, hide);
 		}
 
 		QString newName;
 		QString newValue;
 		QWidget * newWidget = NULL;
-		if (result) {
+		if (result && !hide) {
 			newName = resultKey;
 			if (resultWidget) {
 				newWidget = resultWidget;
@@ -753,13 +844,14 @@ void HtmlInfoView::displayProps(ModelPart * modelPart, ItemBase * itemBase, bool
 		if (propThing->m_name->text().compare(newName) != 0) {
 			propThing->m_name->setText(newName);
 		}
-		propThing->m_name->setVisible(true);
-		propThing->m_frame->setVisible(true);
+
+		propThing->m_name->setVisible(!hide);
+		propThing->m_frame->setVisible(!hide);
 
 		if (newWidget == NULL && propThing->m_value->text().compare(newValue) != 0) {
 			propThing->m_value->setText(newValue);
 		}
-		propThing->m_value->setVisible(newWidget == NULL);
+		propThing->m_value->setVisible(newWidget == NULL && !hide);
 	}
 
 	for (int jx = ix; jx < m_propThings.count(); jx++) {
@@ -809,60 +901,8 @@ QHash<QString, QString> HtmlInfoView::getPartProperties(ModelPart * modelPart, I
 	QHash<QString, QString> properties;
 	QString family;
 	QString partNumber;
-	if (modelPart) {
-        // TODO: someday get local props
-		properties = modelPart->properties();
-		family = properties.value("family", "").toLower();
-		if (itemBase) {
-			itemBase->prepareProps();
-		}
-
-		// ensure family is first;
-		keys = properties.keys();
-		keys.removeOne("family");
-		keys.push_front("family");
-
-		// ensure part number  is last
-		partNumber = properties.value(ModelPartShared::PartNumberPropertyName, "").toLower();
-		keys.removeOne(ModelPartShared::PartNumberPropertyName);
-		
-		if (wantDebug) {
-			properties.insert("id", QString("%1 %2 %3")
-				.arg(itemBase ? QString::number(itemBase->id()) : "")
-				.arg(modelPart->moduleID())
-				.arg(itemBase ? ViewLayer::viewLayerNameFromID(itemBase->viewLayerID()) : "")
-			);
-			keys.insert(1, "id");
-
-			int insertAt = 2;
-
-			if (itemBase) {
-				PaletteItemBase * paletteItemBase = qobject_cast<PaletteItemBase *>(itemBase);
-				if (paletteItemBase != NULL) {
-					properties.insert("svg", paletteItemBase->filename());
-					keys.insert(insertAt++, "svg");
-				}
-				properties.insert("class", itemBase->metaObject()->className());
-				keys.insert(insertAt++, "class");
-			}
-			else {
-                DebugDialog::debug("htmlinfoview has modelpart without itembase?");
-				//FSvgRenderer * renderer = FSvgRenderer::getByModuleID(modelPart->moduleID(), ViewLayer::Icon);
-				//if (renderer != NULL) {
-				//	properties.insert("svg", renderer->filename());
-				//	keys.insert(insertAt++, "svg");
-				//}
-			}
-			if (modelPart->modelPartShared()) {
-				properties.insert("fzp",  modelPart->path());
-				keys.insert(insertAt++, "fzp");
-			}	
-		}
-	}
-
-	// ensure part number is last
-	if (itemBase != NULL && itemBase->hasPartNumberProperty()) {
-		keys.append(ModelPartShared::PartNumberPropertyName);
+	if (modelPart != NULL && itemBase != NULL) {
+        properties = itemBase->prepareProps(modelPart, wantDebug, keys);
 	}
 
 	return properties;
@@ -874,6 +914,8 @@ void HtmlInfoView::changeLock(bool lockState)
 	if (m_currentItem->itemType() == ModelPart::Wire) return;
 
 	m_currentItem->setMoveLock(lockState);
+    m_locationFrame->setDisabled(lockState);
+    m_rotationFrame->setDisabled(lockState);
 }
 
 
@@ -888,3 +930,289 @@ void HtmlInfoView::changeSticky(bool lockState)
 void HtmlInfoView::clickObsolete(const QString &) {
     emit clickObsoleteSignal();
 }
+
+void HtmlInfoView::showLayers(bool show, ItemBase * itemBase, const QString & family, const QString & value, bool swappingEnabled) {
+
+    if (m_layerWidget) {
+	    m_layerLayout->removeWidget(m_layerWidget);
+        m_layerWidget->blockSignals(true);     
+	    m_layerWidget->setVisible(false);          // seems to trigger an unwanted focus out signal
+	    m_layerWidget->deleteLater();
+        m_layerWidget = NULL;
+    }
+
+    if (itemBase == NULL) show = false;
+    m_layerFrame->setVisible(show);
+    m_layerLabel->setVisible(show);
+    if (!show) return;
+
+    QString resultKey, resultValue;
+    bool hide;
+    bool result = itemBase->collectExtraInfo(m_layerLabel->parentWidget(), family, "layer", value, swappingEnabled, resultKey, resultValue, m_layerWidget, hide);
+    if (result && m_layerWidget != NULL) {
+        m_layerLayout->addWidget(m_layerWidget);
+    }
+}
+
+void HtmlInfoView::makeLockFrame() {
+	m_lockLabel = new QLabel("    ", this);
+	m_lockLabel->setObjectName("propNameLabel");
+	m_lockLabel->setWordWrap(true);
+
+    m_lockFrame = new QFrame(this);
+    QHBoxLayout * lockLayout = new QHBoxLayout();
+	lockLayout->setSpacing(0);
+	lockLayout->setContentsMargins(0, 0, 0, 0);
+    m_lockFrame->setObjectName("propValueFrame");
+
+	m_lockCheckbox = new QCheckBox(tr("Locked"));
+	m_lockCheckbox->setObjectName("infoViewLockCheckbox");
+	m_lockCheckbox->setToolTip(tr("Change the locked state of the part in this view. A locked part can't be moved."));
+	connect(m_lockCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeLock(bool)));
+	lockLayout->addWidget(m_lockCheckbox);
+
+    lockLayout->addSpacing(10);
+
+	m_stickyCheckbox = new QCheckBox(tr("Sticky"));
+	m_stickyCheckbox->setObjectName("infoViewLockCheckbox");
+	m_stickyCheckbox->setToolTip(tr("Change the \"sticky\" state of the part in this view. When a sticky part is moved, objects on top of it also move."));
+	connect(m_stickyCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeSticky(bool)));
+	lockLayout->addWidget(m_stickyCheckbox);
+
+    lockLayout->addSpacerItem(new QSpacerItem(1,1, QSizePolicy::Expanding));
+
+    m_lockFrame->setLayout(lockLayout);
+}
+
+void HtmlInfoView::makeLocationFrame() {
+	m_locationLabel = new QLabel(tr("location"), this);
+	m_locationLabel->setObjectName("propNameLabel");
+	m_locationLabel->setWordWrap(true);
+
+    m_locationFrame = new QFrame(this);
+    QHBoxLayout * locationLayout = new QHBoxLayout();
+	locationLayout->setSpacing(0);
+	locationLayout->setContentsMargins(0, 0, 0, 0);
+    m_locationFrame->setObjectName("propValueFrame");
+
+	m_xEdit = new QDoubleSpinBox;
+    m_xEdit->setDecimals(3);
+    m_xEdit->setRange(-99999.999, 99999.999);
+    m_xEdit->setKeyboardTracking(false);
+    m_xEdit->setObjectName("infoViewDoubleSpinBox");
+    m_xEdit->setMaximumWidth(MaxSpinBoxWidth);
+    m_xEdit->setMinimumWidth(MaxSpinBoxWidth);
+    locationLayout->addWidget(m_xEdit);
+
+    locationLayout->addSpacing(AfterSpinBoxWidth);
+
+	m_yEdit = new QDoubleSpinBox;
+    m_yEdit->setDecimals(3);
+    m_yEdit->setRange(-99999.999, 99999.999);
+    m_yEdit->setKeyboardTracking(false);
+    m_yEdit->setObjectName("infoViewDoubleSpinBox");
+    m_yEdit->setMaximumWidth(MaxSpinBoxWidth);
+    m_yEdit->setMinimumWidth(MaxSpinBoxWidth);
+    locationLayout->addWidget(m_yEdit);
+
+    locationLayout->addSpacing(3);
+
+	m_unitsLabel = new ClickableLabel("px", this);
+    m_unitsLabel->setObjectName("infoViewSpinBoxLabel");
+    m_unitsLabel->setCursor(Qt::PointingHandCursor);
+    locationLayout->addWidget(m_unitsLabel);
+
+    locationLayout->addSpacerItem(new QSpacerItem(1,1, QSizePolicy::Expanding));
+
+    m_locationFrame->setLayout(locationLayout);
+
+    connect(m_xEdit, SIGNAL(valueChanged(double)), this, SLOT(xyEntry()));
+    connect(m_yEdit, SIGNAL(valueChanged(double)), this, SLOT(xyEntry()));
+	connect(m_unitsLabel, SIGNAL(clicked()), this, SLOT(unitsClicked()));
+    unitsClicked();   // increments from px so that default is inches
+}
+
+void HtmlInfoView::makeRotationFrame() {
+	m_rotationLabel = new QLabel(tr("rotation"), this);
+	m_rotationLabel->setObjectName("propNameLabel");
+	m_rotationLabel->setWordWrap(true);
+
+    m_rotationFrame = new QFrame(this);
+    QHBoxLayout * rotationLayout = new QHBoxLayout();
+	rotationLayout->setSpacing(0);
+	rotationLayout->setContentsMargins(0, 0, 0, 0);
+    m_rotationFrame->setObjectName("propValueFrame");
+
+	m_rotEdit = new QDoubleSpinBox;
+    m_rotEdit->setDecimals(1);
+    m_rotEdit->setRange(-360, 360);
+    m_rotEdit->setKeyboardTracking(false);
+    m_rotEdit->setObjectName("infoViewDoubleSpinBox");
+    m_rotEdit->setMaximumWidth(MaxSpinBoxWidth);
+    m_rotEdit->setMinimumWidth(MaxSpinBoxWidth);
+    rotationLayout->addWidget(m_rotEdit);
+
+    rotationLayout->addSpacing(AfterSpinBoxWidth);
+
+	QLabel * label = new QLabel(tr("degrees"), this);
+    label->setObjectName("infoViewSpinBoxLabel");
+    rotationLayout->addWidget(label);
+
+    rotationLayout->addSpacerItem(new QSpacerItem(1,1, QSizePolicy::Expanding));
+
+    m_rotationFrame->setLayout(rotationLayout);
+
+    connect(m_rotEdit, SIGNAL(valueChanged(double)), this, SLOT(rotEntry()));
+}
+
+void HtmlInfoView::unitsClicked() {
+    QString units = m_unitsLabel->text();
+
+    QString xs = m_xEdit->text();
+    QString ys = m_yEdit->text();
+    double x = TextUtils::convertToInches(xs + units);
+    double y = TextUtils::convertToInches(ys + units);
+
+    double dpi = 1;
+    double step = 1;
+	if (units.compare("mm") == 0) {
+		units = "px";
+        dpi = 90;
+        step = 1;
+	}
+	else if (units.compare("px") == 0) {
+		units = "in";
+        dpi = 1;
+        step = 0.1;
+	}
+	else if (units.compare("in") == 0) {
+		units = "mm";
+        dpi = 25.4;
+        step = 1;
+	}
+	else {
+		units = "in";
+        dpi = 1;
+        step = 0.1;
+	}
+    m_unitsLabel->setText(units);
+
+    m_xEdit->blockSignals(true);
+    m_yEdit->blockSignals(true);
+    m_xEdit->setSingleStep(step);
+    m_yEdit->setSingleStep(step);
+    if (!xs.isEmpty()) m_xEdit->setValue(x * dpi);
+    if (!ys.isEmpty()) m_yEdit->setValue(y * dpi);
+    m_xEdit->blockSignals(false);
+    m_yEdit->blockSignals(false);
+}
+
+void HtmlInfoView::setLocation(ItemBase * itemBase) {
+    if (itemBase == NULL) {
+        m_xEdit->blockSignals(true);
+        m_yEdit->blockSignals(true);
+        m_xEdit->setEnabled(false);
+        m_yEdit->setEnabled(false);
+        m_xEdit->setValue(0);
+        m_yEdit->setValue(0);
+        m_xEdit->blockSignals(false);
+        m_yEdit->blockSignals(false);
+        return;
+    }
+
+    m_xEdit->setEnabled(!itemBase->moveLock());
+    m_yEdit->setEnabled(!itemBase->moveLock());
+
+    QString units = m_unitsLabel->text();
+    QPointF loc = itemBase->pos();
+    if (units == "px") {
+    }
+    else if (units == "in") {
+        loc /= 90;
+    }
+    else if (units == "mm") {
+        loc /= (90 / 25.4);
+    }
+
+    m_xEdit->blockSignals(true);
+    m_yEdit->blockSignals(true);
+    m_xEdit->setValue(loc.x());
+    m_yEdit->setValue(loc.y());
+    m_xEdit->blockSignals(false);
+    m_yEdit->blockSignals(false);
+}
+
+void HtmlInfoView::setRotation(ItemBase * itemBase) {
+    if (itemBase == NULL) {
+        m_rotEdit->blockSignals(true);
+        m_rotEdit->setEnabled(false);
+        m_rotEdit->setValue(0);
+        m_rotEdit->blockSignals(false);
+        return;
+    }
+
+    double step = 90;
+    bool enabled = true;
+    if (itemBase->moveLock()) enabled = false;
+    else if (!itemBase->rotationAllowed()) enabled = false;
+    else {
+        if (itemBase->rotation45Allowed()) step = 45;
+        if (itemBase->freeRotationAllowed()) step = 1;
+    }
+
+    m_rotEdit->setEnabled(enabled);
+    m_rotEdit->setSingleStep(step);
+
+    QTransform transform = itemBase->transform();
+    double angle = atan2(transform.m12(), transform.m11()) * 180 / M_PI;
+
+    m_rotEdit->blockSignals(true);
+    m_rotEdit->setValue(angle);
+    m_rotEdit->blockSignals(false);
+}
+
+void HtmlInfoView::updateLocation(ItemBase * itemBase) {
+    if (itemBase == NULL) return;
+    if (itemBase != m_lastItemBase) return;
+
+    setLocation(itemBase);
+}
+
+void HtmlInfoView::updateRotation(ItemBase * itemBase) {
+    if (itemBase == NULL) return;
+    if (itemBase != m_lastItemBase) return;
+
+    setRotation(itemBase);
+}
+
+void HtmlInfoView::xyEntry() {
+    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(m_lastItemBase);
+    if (infoGraphicsView == NULL) return;
+
+    DebugDialog::debug(QString("xedit %1 %2 %3").arg(m_xEdit->text()).arg(m_yEdit->text()).arg(sender() == m_xEdit));
+    double x = TextUtils::convertToInches(m_xEdit->text() + m_unitsLabel->text());
+    double y = TextUtils::convertToInches(m_yEdit->text() + m_unitsLabel->text());
+    if (infoGraphicsView != NULL && m_lastItemBase != NULL) {
+        infoGraphicsView->moveItem(m_lastItemBase, x * 90, y * 90);
+    }
+}
+
+void HtmlInfoView::rotEntry() {
+    if (m_lastItemBase != NULL) {
+        InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(m_lastItemBase);
+        if (infoGraphicsView == NULL) return;
+
+        double newAngle = m_rotEdit->value();
+
+        if (m_rotEdit->singleStep() == 1) {
+        }
+        else {
+            newAngle = qFloor(newAngle / m_rotEdit->singleStep()) * m_rotEdit->singleStep();
+        }
+
+        QTransform transform = m_lastItemBase->transform();
+        double angle = atan2(transform.m12(), transform.m11()) * 180 / M_PI;
+        infoGraphicsView->rotateX(newAngle - angle, false, m_lastItemBase);
+    }
+}
+
