@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5801 $:
-$Author: cohen@irascible.com $:
-$Date: 2012-02-01 10:30:37 +0100 (Wed, 01 Feb 2012) $
+$Revision: 6609 $:
+$Author: irascibl@gmail.com $:
+$Date: 2012-10-30 14:21:08 +0100 (Di, 30. Okt 2012) $
 
 ********************************************************************/
 
@@ -38,6 +38,7 @@ $Date: 2012-02-01 10:30:37 +0100 (Wed, 01 Feb 2012) $
 #include <QVBoxLayout>
 #include <QComboBox>
 #include <QCheckBox>
+#include <QDoubleSpinBox>
 
 #include "../items/itembase.h"
 #include "../items/wire.h"
@@ -76,6 +77,8 @@ public:
 	void reloadContent(class InfoGraphicsView *);
 
 	void viewItemInfo(class InfoGraphicsView *, ItemBase* item, bool swappingEnabled);
+    void updateLocation(ItemBase *);
+    void updateRotation(ItemBase *);
 
 	void hoverEnterItem(class InfoGraphicsView *, QGraphicsSceneHoverEvent * event, ItemBase * item, bool swappingEnabled);
 	void hoverLeaveItem(class InfoGraphicsView *, QGraphicsSceneHoverEvent * event, ItemBase * item);
@@ -86,6 +89,8 @@ public:
 	void unregisterCurrentItem();
 	void unregisterCurrentItemIf(long id);
 
+    void init(bool tinyMode);
+
 public:
 	static const int STANDARD_ICON_IMG_WIDTH;
 	static const int STANDARD_ICON_IMG_HEIGHT;
@@ -93,6 +98,9 @@ public:
 	static void cleanup();
 	static QHash<QString, QString> getPartProperties(ModelPart * modelPart, ItemBase * itemBase, bool wantDebug, QStringList & keys);
 
+signals:
+    void clickObsoleteSignal();
+
 protected slots:
 	void setContent();
 	void setInstanceTitle();
@@ -100,6 +108,11 @@ protected slots:
 	void instanceTitleLeave();
 	void instanceTitleEditable(bool editable);
 	void changeLock(bool);
+	void changeSticky(bool);
+    void clickObsolete(const QString &);
+    void xyEntry();
+    void unitsClicked();
+    void rotEntry();
 
 protected:
 	void appendStuff(ItemBase* item, bool swappingEnabled); //finds out if it's a wire or something else
@@ -112,21 +125,25 @@ protected:
 	void setCurrentItem(ItemBase *);
 	void setNullContent();
 	void setUpTitle(ItemBase *);
-	void setUpIcons(ModelPart *);
+	void setUpIcons(ItemBase *, bool swappingEnabled);
 	void addTags(ModelPart * modelPart);
-	void partTitle(const QString & title, const QString & version, const QString & url);
+	void partTitle(const QString & title, const QString & version, const QString & url, bool obsolete);
 	void displayProps(ModelPart * modelPart, ItemBase * itemBase, bool swappingEnabled);
 	void clearPropThingPlugin(PropThing * propThing);
 	void clearPropThingPlugin(PropThing * propThing, QWidget * plugin);
-	QPixmap * getPixmap(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier);
 	void viewConnectorItemInfo(QGraphicsSceneHoverEvent * event, ConnectorItem* item);
+    void showLayers(bool show, ItemBase *, const QString & family, const QString & value, bool swappingEnabled);
+    void makeLockFrame();
+    void makeLocationFrame();
+    void makeRotationFrame();
+    void setLocation(ItemBase *);
+    void setRotation(ItemBase *);
 
 protected:
 	QPointer<ItemBase> m_currentItem;
 	bool m_currentSwappingEnabled;					// previous item (possibly hovered over)
 
 	QTimer m_setContentTimer;
-	QPointer<class InfoGraphicsView> m_infoGraphicsView;
 	QPointer<ItemBase> m_lastItemBase;
 	bool m_lastSwappingEnabled;						// previous item (selected)
 	class FLineEdit * m_titleEdit;
@@ -140,17 +157,34 @@ protected:
 	QLabel * m_connDescr;
 	QLabel * m_connName;
 	QLabel * m_connType;
-	QLabel * m_proplabel;
-	QLabel * m_taglabel;
+	QLabel * m_propLabel;
+	QLabel * m_placementLabel;
+	QLabel * m_tagLabel;
 	QLabel * m_connLabel;
+    QLabel * m_layerLabel;
+    QLabel * m_lockLabel;
+    QLabel * m_locationLabel;
+    QLabel * m_rotationLabel;
 	QFrame * m_connFrame;
 	QFrame * m_propFrame;
+	QFrame * m_placementFrame;
+    QFrame * m_layerFrame;
+    QFrame * m_lockFrame;
+    QFrame * m_locationFrame;
+    QFrame * m_rotationFrame;
 	QCheckBox * m_lockCheckbox;
+	QCheckBox * m_stickyCheckbox;
 	QGridLayout * m_propLayout;
+	QGridLayout * m_placementLayout;
+    QVBoxLayout * m_layerLayout;
 	QList <PropThing *> m_propThings;
-    QPointer<class InfoGraphicsView> m_pendingInfoGraphicsView;
 	QPointer<ItemBase> m_pendingItemBase;
 	bool m_pendingSwappingEnabled;
+    QWidget * m_layerWidget;
+    QDoubleSpinBox * m_xEdit;
+    QDoubleSpinBox * m_yEdit;
+    QLabel * m_unitsLabel;
+    QDoubleSpinBox * m_rotEdit;
 
 	// note: these m_last items should only be checked for equality and not otherwise accessed
 	ItemBase * m_lastTitleItemBase;
@@ -159,14 +193,11 @@ protected:
 	ModelPart * m_lastTagsModelPart;
 	int m_lastConnectorItemCount;
 	ConnectorItem * m_lastConnectorItem;
-	ModelPart * m_lastIconModelPart;
+	ItemBase * m_lastIconItemBase;
 	ModelPart * m_lastPropsModelPart;
 	ItemBase * m_lastPropsItemBase;
 	bool m_lastPropsSwappingEnabled;
-	// end note
-
-protected:
-	static QHash<QString, QPixmap *> m_pixmaps;
+    bool m_tinyMode;
 };
 
 #endif
