@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6509 $:
+$Revision: 6984 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-04 07:19:37 +0200 (Thu, 04 Oct 2012) $
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -36,6 +36,7 @@ $Date: 2012-10-04 07:19:37 +0200 (Thu, 04 Oct 2012) $
 #include "moduleidnames.h"
 #include "../layerattributes.h"
 #include "../debugdialog.h"
+#include "../svg/gerbergenerator.h"
 
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -52,8 +53,12 @@ $Date: 2012-10-04 07:19:37 +0200 (Thu, 04 Oct 2012) $
 QList<QString> PaperSizeNames;
 QList<QSizeF> PaperSizeDimensions;
 
-QHash<QString, QString> BoardLayerTemplates;
-QHash<QString, QString> SilkscreenLayerTemplates;
+static QHash<QString, QString> NamesToXmlNames;
+static QHash<QString, QString> XmlNamesToNames;
+
+static QHash<QString, QString> BoardLayerTemplates;
+static QHash<QString, QString> SilkscreenLayerTemplates;
+static QHash<QString, QString> Silkscreen0LayerTemplates;
 static const int LineThickness = 8;
 static const QRegExp HeightExpr("height=\\'\\d*px");
 static QString StandardCustomBoardExplanation;
@@ -68,8 +73,8 @@ static const double JND = (double) 0.01;
 QString Board::OneLayerTranslated;
 QString Board::TwoLayersTranslated;
 
-Board::Board( ModelPart * modelPart, ViewLayer::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+Board::Board( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: PaletteItem(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
     m_svgOnly = true;
 	m_fileNameComboBox = NULL;
@@ -79,15 +84,25 @@ Board::Board( ModelPart * modelPart, ViewLayer::ViewIdentifier viewIdentifier, c
         }
         if (itemType() == ModelPart::Board && !modelPart->properties().keys().contains("filename")) {
             // deal with old style custom boards
-            modelPart->modelPartShared()->setProperty("filename", "");
+            modelPart->modelPartShared()->setProperty("filename", "", false);
         }
     }
 
     if (StandardCustomBoardExplanation.isEmpty()) {
-        StandardCustomBoardExplanation = tr("\n\nA custom board svg typically has one silkscreen layer and one board layer.\n") +
+        StandardCustomBoardExplanation = tr("\n\nA custom board svg typically has one or two silkscreen layers and one board layer.\n") +
                                             tr("Have a look at the circle_pcb.svg file in your Fritzing installation folder at parts/svg/core/pcb/.\n\n");
     }
 
+    if (NamesToXmlNames.count() == 0) {
+        NamesToXmlNames.insert("copper bottom", "copper0");
+        NamesToXmlNames.insert("copper top", "copper1");
+        NamesToXmlNames.insert("silkscreen bottom", "silkscreen0");
+        NamesToXmlNames.insert("silkscreen top", "silkscreen");
+        foreach (QString key, NamesToXmlNames.keys()) {
+            XmlNamesToNames.insert(NamesToXmlNames.value(key), key);
+        }
+    }
+
 }
 
 Board::~Board() {
@@ -140,14 +155,14 @@ QStringList Board::collectValues(const QString & family, const QString & prop, Q
 
 }
 
-bool Board::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget) 
+bool Board::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide) 
 {
 	if (prop.compare("filename", Qt::CaseInsensitive) == 0 && isBoard(this)) {
         setupLoadImage(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
 		return true;
 	}
 
-	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
 bool Board::rotation45Allowed() {
@@ -323,7 +338,8 @@ bool Board::checkImage(const QString & filename) {
     QList<QDomElement> elements;
     TextUtils::findElementsWithAttribute(root, "id", elements);
     int layers = 0;
-    int boardLayers = 0;
+    QList<QDomElement> boardElements;
+    int silk0Layers = 0;
     int silk1Layers = 0;
     bool boardHasChildren = false;
     foreach (QDomElement element, elements) {
@@ -332,25 +348,31 @@ bool Board::checkImage(const QString & filename) {
         if (viewLayerID != ViewLayer::UnknownLayer) {
             layers++;
             if (viewLayerID == ViewLayer::Board) {
-                boardLayers++;
+                boardElements << element;
                 if (element.childNodes().count() > 0) {
                     boardHasChildren = true;
                 }
             }
-            if (viewLayerID == ViewLayer::Silkscreen1) {
+            else if (viewLayerID == ViewLayer::Silkscreen1) {
                 silk1Layers++;
             }
+            else if (viewLayerID == ViewLayer::Silkscreen0) {
+                silk0Layers++;
+            }
         }
     }
 
-    if (boardLayers == 1 && !boardHasChildren) {
+    if (boardElements.count() == 1 && !boardHasChildren) {
         unableToLoad(filename, tr("the <board> element contains no shape elements") + StandardCustomBoardExplanation);
         return false;
     }
 
-    if ((boardLayers == 1) && (silk1Layers == 1)) return true;
+    if ((boardElements.count() == 1) && ((silk1Layers == 1) || (silk0Layers == 1))) {
+        moreCheckImage(filename);
+        return true;
+    }
 
-    if (boardLayers > 1) {
+    if (boardElements.count() > 1) {
         unableToLoad(filename, tr("because there are multiple <board> layers") + StandardCustomBoardExplanation);
         return false;
     }
@@ -360,7 +382,12 @@ bool Board::checkImage(const QString & filename) {
         return false;
     }
 
-    if (layers > 0 && boardLayers == 0) {
+    if (silk0Layers > 1) {
+        unableToLoad(filename, tr("because there are multiple <silkscreen0> layers") + StandardCustomBoardExplanation);
+        return false;
+    }
+
+    if (layers > 0 && boardElements.count() == 0) {
         unableToLoad(filename, tr("because there is no <board> layer") + StandardCustomBoardExplanation);
         return false;
     }
@@ -370,14 +397,125 @@ bool Board::checkImage(const QString & filename) {
         return false;
     }
 
-    if (layers == 0 || (boardLayers == 1 && silk1Layers == 0)) {
-        return canLoad(filename, tr("but the pcb itself will have no silkscreen layer") + StandardCustomBoardExplanation);
+    if (layers == 0 || (boardElements.count() == 1 && silk1Layers == 0 && silk1Layers == 0)) {
+        bool result = canLoad(filename, tr("but the pcb itself will have no silkscreen layer") + StandardCustomBoardExplanation);
+        if (result) moreCheckImage(filename);
+        return result;
     }
 
     unableToLoad(filename, tr("the svg doesn't fit the custom board format") + StandardCustomBoardExplanation);
     return false;
 }
 
+void Board::moreCheckImage(const QString & filename) {
+    QFile file(filename);
+    file.open(QFile::ReadOnly);
+    QString svg = file.readAll();
+    file.close();
+
+    QString nsvg = setBoardOutline(svg);
+
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	QDomDocument domDocument;
+	domDocument.setContent(nsvg, &errorStr, &errorLine, &errorColumn);
+    QDomElement element = TextUtils::findElementWithAttribute(domDocument.documentElement(), "id", GerberGenerator::MagicBoardOutlineID);
+
+    int subpaths = 1;
+    int mCount = 0;
+    if (element.tagName() == "path") {
+        QString originalPath = element.attribute("d", "").trimmed();
+        if (GerberGenerator::MultipleZs.indexIn(originalPath) >= 0) {
+            QStringList ds = element.attribute("d").split("z", QString::SkipEmptyParts);
+            subpaths = ds.count();
+            foreach (QString d, ds) {
+                if (d.trimmed().startsWith("m", Qt::CaseInsensitive)) mCount++;
+            }
+        }
+    }
+
+    QString msg = tr("<b>The custom shape has been loaded, and you will see the new board shortly.</b><br/><br/>");
+    msg += tr("Before actual PCB production we recommend that you test your custom shape by using the 'File > Export for Production > Extended Gerber' option. ");
+    msg += tr("Check the resulting contour file with a Gerber-viewer application to make sure the shape came out as expected.<br/><br/>");
+
+    msg += tr("The rest of this message concerns 'cutouts'. ");
+    msg += tr("These are circular or irregularly-shaped holes that you can optionally incorporate into a custom PCB shape.<br/><br/>");
+    if (subpaths == 1) {
+        msg += tr("<b>The custom shape has no cutouts.</b>");
+    }
+    else {
+        msg += tr("<b>The custom shape has %n cutouts.</b>", "", subpaths - 1);
+        if (subpaths != mCount) {
+            msg += tr("<br/>However, the cutouts may not be formatted correctly.");
+        }
+    }
+    msg +=  tr("<br/><br/>If you intended your custom shape to have cutouts and you did not get the expected result, ");
+    msg += tr("it is because Fritzing requires that you make cutouts using a shape 'subtraction' or 'difference' operation in your vector graphics editor.");
+    QMessageBox::information(NULL, "Custom Shape", msg);
+}
+
+QString Board::setBoardOutline(const QString & svg) {
+    QString errorStr;
+	int errorLine;
+	int errorColumn;
+
+	QDomDocument domDocument;
+	if (!domDocument.setContent(svg, true, &errorStr, &errorLine, &errorColumn)) {
+		return svg;
+	}
+
+    QDomElement root = domDocument.documentElement();
+	if (root.tagName() != "svg") {
+		return svg;
+	}
+
+    QDomElement board = TextUtils::findElementWithAttribute(root, "id", "board");
+    if (board.isNull()) {
+        board = root;
+    }
+
+    QList<QDomElement> leaves;
+    TextUtils::collectLeaves(board, leaves);
+
+    if (leaves.count() == 1) {
+        QDomElement leaf = leaves.at(0);
+        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
+        return TextUtils::removeXMLEntities(domDocument.toString());
+    }
+
+    int ix = 0;
+    QStringList ids;
+    foreach (QDomElement leaf, leaves) {
+        ids.append(leaf.attribute("id", ""));
+        leaf.setAttribute("id", QString("____%1____").arg(ix++));
+    }
+
+    QSvgRenderer renderer(domDocument.toByteArray());
+
+    ix = 0;
+    foreach (QDomElement leaf, leaves) {    
+        leaf.setAttribute("id", ids.at(ix++));
+    }
+
+    double maxArea = 0;
+    int maxIndex = -1;
+    for (int i = 0; i < leaves.count(); i++) {
+        QRectF r = renderer.boundsOnElement(QString("____%1____").arg(i));
+        if (r.width() * r.height() > maxArea) {
+            maxArea = r.width() * r.height();
+            maxIndex = i;
+        }
+    }
+
+    if (maxIndex >= 0) {
+        QDomElement leaf = leaves.at(maxIndex);
+        leaf.setAttribute("id", GerberGenerator::MagicBoardOutlineID);
+        return TextUtils::removeXMLEntities(domDocument.toString());
+    }
+
+    return svg;
+}
 void Board::unableToLoad(const QString & fileName, const QString & reason) {
 	QMessageBox::information(
 		NULL,
@@ -405,7 +543,7 @@ void Board::prepLoadImageAux(const QString & fileName, bool addName)
 	}
 }
 
-ViewLayer::ViewIdentifier Board::useViewIdentifierForPixmap(ViewLayer::ViewIdentifier vid, bool) 
+ViewLayer::ViewID Board::useViewIDForPixmap(ViewLayer::ViewID vid, bool) 
 {
     if (vid == ViewLayer::PCBView) {
         return ViewLayer::IconView;
@@ -414,10 +552,26 @@ ViewLayer::ViewIdentifier Board::useViewIdentifierForPixmap(ViewLayer::ViewIdent
     return ViewLayer::UnknownView;
 }
 
+QString Board::convertToXmlName(const QString & name) {
+    foreach (QString key, ItemBase::TranslatedPropertyNames.keys()) {
+        if (name.compare(ItemBase::TranslatedPropertyNames.value(key), Qt::CaseInsensitive) == 0) {
+            return NamesToXmlNames.value(key);
+        }
+    }
+
+    return name;
+}
+
+QString Board::convertFromXmlName(const QString & xmlName) {
+    QString result = ItemBase::TranslatedPropertyNames.value(XmlNamesToNames.value(xmlName));
+    if (result.isEmpty()) return xmlName;
+    return result;
+}
+
 ///////////////////////////////////////////////////////////
 
-ResizableBoard::ResizableBoard( ModelPart * modelPart, ViewLayer::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
-	: Board(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
+ResizableBoard::ResizableBoard( ModelPart * modelPart, ViewLayer::ViewID viewID, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
+	: Board(modelPart, viewID, viewGeometry, id, itemMenu, doLabel)
 {
 	fixWH();
 
@@ -457,17 +611,17 @@ void ResizableBoard::loadTemplates() {
 
     BoardLayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Board));
     SilkscreenLayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Silkscreen1));
+    Silkscreen0LayerTemplates.insert(moduleID(), getShapeForRenderer(svg, ViewLayer::Silkscreen0));
 }
 
 double ResizableBoard::minWidth() {
-	return 0.5 * GraphicsUtils::SVGDPI;
+	return 0.25 * GraphicsUtils::SVGDPI;
 }
 
 double ResizableBoard::minHeight() {
-	return 0.5 * GraphicsUtils::SVGDPI;
+	return 0.25 * GraphicsUtils::SVGDPI;
 }
 
-
 void ResizableBoard::mousePressEvent(QGraphicsSceneMouseEvent * event) 
 {
 	m_corner = ResizableBoard::NO_CORNER;
@@ -532,20 +686,28 @@ void ResizableBoard::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 			size.setWidth(size.width() - ds.x());
 			size.setHeight(size.height() - ds.y());
 			break;
-                default:
-                        break;
+        default:
+            break;
 	}
 
-	if (size.width() < minWidth()) size.setWidth(minWidth());
-	if (size.height() < minHeight()) size.setHeight(minHeight());
+	if (size.width() < minWidth()) {
+        DebugDialog::debug("to min width");
+        size.setWidth(minWidth());
+    }
+	if (size.height() < minHeight()) {
+        DebugDialog::debug("to min height");
+        size.setHeight(minHeight());
+    }
 
 	if (m_keepAspectRatio) {
 		double cw = size.height() * m_aspectRatio.width() / m_aspectRatio.height();
 		double ch = size.width() * m_aspectRatio.height() / m_aspectRatio.width();
 		if (ch < minHeight()) {
+            DebugDialog::debug("from min height");
 			size.setWidth(cw);
 		}
 		else if (cw < minWidth()) {
+            DebugDialog::debug("from min width");
 			size.setHeight(ch);
 		}
 		else {
@@ -573,9 +735,8 @@ void ResizableBoard::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 		if (changeTransform) {
 			QTransform oldT = transform();
 
-			DebugDialog::debug(QString("t old m:%1,%2,%3,%4 d:%5,%6 p:%7,%8 sz:%9,%10")
-				.arg(oldT.m11()).arg(oldT.m12()).arg(oldT.m21()).arg(oldT.m22())
-				.arg(oldT.toAffine().dx()).arg(oldT.toAffine().dy())
+			DebugDialog::debug(QString("t old m:%1 p:%2,%3 sz:%4,%5")
+                .arg(TextUtils::svgMatrix(oldT))
 				.arg(pos().x()).arg(pos().y())
 				.arg(oldSize.width()).arg(oldSize.height()));
 
@@ -596,9 +757,8 @@ void ResizableBoard::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 			}
 			
 			QTransform t = transform();
-			DebugDialog::debug(QString("t new m:%1,%2,%3,%4 d:%5,%6 p:%7,%8 sz:%9,%10")
-				.arg(t.m11()).arg(t.m12()).arg(t.m21()).arg(t.m22())
-				.arg(t.toAffine().dx()).arg(t.toAffine().dy())
+			DebugDialog::debug(QString("t new m:%1 p:%2,%3 sz:%4,%5")
+				.arg(TextUtils::svgMatrix(t))
 				.arg(pos().x()).arg(pos().y())
 				.arg(size.width()).arg(size.height()));
 		}
@@ -642,14 +802,7 @@ void ResizableBoard::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {
 	}
 }
 
-bool ResizableBoard::setUpImage(ModelPart * modelPart, ViewLayer::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
-{
-	bool result = Board::setUpImage(modelPart, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
-
-	return result;
-}
-
-ViewLayer::ViewIdentifier ResizableBoard::theViewIdentifier() {
+ViewLayer::ViewID ResizableBoard::theViewID() {
 	return ViewLayer::PCBView;
 }
 
@@ -659,9 +812,12 @@ void ResizableBoard::resizePixels(double w, double h, const LayerHash & viewLaye
 
 bool ResizableBoard::resizeMM(double mmW, double mmH, const LayerHash & viewLayers) {
 	if (mmW == 0 || mmH == 0) {
-		QString error;
 		LayerAttributes layerAttributes;
-		setUpImage(modelPart(), m_viewIdentifier, viewLayers, m_viewLayerID, m_viewLayerSpec, true, layerAttributes, error);
+        layerAttributes.viewID = m_viewID;
+        layerAttributes.viewLayerID = m_viewLayerID;
+        layerAttributes.viewLayerPlacement = m_viewLayerPlacement;
+        layerAttributes.createShape = layerAttributes.doConnectors = true;
+		setUpImage(modelPart(), viewLayers, layerAttributes);
 		modelPart()->setLocalProp("height", QVariant());
 		modelPart()->setLocalProp("width", QVariant());
 		// do the layerkin
@@ -706,16 +862,15 @@ void ResizableBoard::resizeMMAux(double mmW, double mmH)
 				itemBase->modelPart()->setLocalProp("width", mmW);
 				itemBase->modelPart()->setLocalProp("height", mmH);
 			}
-			break;
 		}
 	}
 
 }
 
-void ResizableBoard::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerSpec viewLayerSpec) {
+void ResizableBoard::loadLayerKin( const LayerHash & viewLayers, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 
 	loadTemplates();				
-	Board::loadLayerKin(viewLayers, viewLayerSpec);
+	Board::loadLayerKin(viewLayers, viewLayerPlacement);
 	double w =  m_modelPart->localProp("width").toDouble();
 	if (w != 0) {
 		resizeMM(w, m_modelPart->localProp("height").toDouble(), viewLayers);
@@ -732,18 +887,18 @@ void ResizableBoard::setInitialSize() {
 	}
 }
 
-QString ResizableBoard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
+QString ResizableBoard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	double w = m_modelPart->localProp("width").toDouble();
 	if (w != 0) {
 		double h = m_modelPart->localProp("height").toDouble();
 		QString xml = makeLayerSvg(viewLayerID, w, h, GraphicsUtils::mm2mils(w), GraphicsUtils::mm2mils(h));
 		if (!xml.isEmpty()) {
-			return PaletteItemBase::normalizeSvg(xml, viewLayerID, blackOnly, dpi);
+			return PaletteItemBase::normalizeSvg(xml, viewLayerID, blackOnly, dpi, factor);
 		}
 	}
 
-	return Board::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
+	return Board::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi, factor);
 }
 
 QSizeF ResizableBoard::getSizeMM() {
@@ -758,7 +913,8 @@ QString ResizableBoard::makeLayerSvg(ViewLayer::ViewLayerID viewLayerID, double
 		case ViewLayer::Board:
 			return makeBoardSvg(mmW, mmH, milsW, milsH);
 		case ViewLayer::Silkscreen1:
-			return makeSilkscreenSvg(mmW, mmH, milsW, milsH);
+		case ViewLayer::Silkscreen0:
+			return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
 			break;
 		default:
 			return "";
@@ -767,7 +923,8 @@ QString ResizableBoard::makeLayerSvg(ViewLayer::ViewLayerID viewLayerID, double
 
 QString ResizableBoard::makeNextLayerSvg(ViewLayer::ViewLayerID viewLayerID, double mmW, double mmH, double milsW, double milsH) {
 
-	if (viewLayerID == ViewLayer::Silkscreen1) return makeSilkscreenSvg(mmW, mmH, milsW, milsH);
+	if (viewLayerID == ViewLayer::Silkscreen1) return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
+	if (viewLayerID == ViewLayer::Silkscreen0) return makeSilkscreenSvg(viewLayerID, mmW, mmH, milsW, milsH);
 
 	return "";
 }
@@ -782,9 +939,10 @@ QString ResizableBoard::makeBoardSvg(double mmW, double mmH, double milsW, doubl
     return makeSvg(mmW, mmH, BoardLayerTemplates.value(moduleID()));
 }
 
-QString ResizableBoard::makeSilkscreenSvg(double mmW, double mmH, double milsW, double milsH) {
+QString ResizableBoard::makeSilkscreenSvg(ViewLayer::ViewLayerID viewLayerID, double mmW, double mmH, double milsW, double milsH) {
     Q_UNUSED(milsW);
     Q_UNUSED(milsH);
+    if (viewLayerID == ViewLayer::Silkscreen0) return makeSvg(mmW, mmH, Silkscreen0LayerTemplates.value(moduleID()));
     return makeSvg(mmW, mmH, SilkscreenLayerTemplates.value(moduleID()));
 }
 
@@ -837,12 +995,12 @@ void ResizableBoard::getParams(QPointF & p, QSizeF & s) {
 }
 
 bool ResizableBoard::hasCustomSVG() {
-	return theViewIdentifier() == m_viewIdentifier;
+	return theViewID() == m_viewID;
 }
 
-bool ResizableBoard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+bool ResizableBoard::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
-	bool result = Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+	bool result = Board::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 
 	if (prop.compare("shape", Qt::CaseInsensitive) == 0) {
 
@@ -945,7 +1103,7 @@ bool ResizableBoard::hasPartNumberProperty()
 
 void ResizableBoard::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-	if (m_hidden) return;
+	if (m_hidden || m_layerHidden) return;
 
 	Board::paintSelected(painter, option, widget);
 
@@ -1012,6 +1170,15 @@ bool ResizableBoard::inResize() {
 	return m_corner != ResizableBoard::NO_CORNER;
 }
 
+void ResizableBoard::figureHover() {
+    setAcceptHoverEvents(true);
+    setAcceptedMouseButtons(ALLMOUSEBUTTONS);
+	foreach(ItemBase * lkpi, m_layerKin) {
+		lkpi->setAcceptHoverEvents(false);
+		lkpi->setAcceptedMouseButtons(Qt::NoButton);
+	}
+}
+
 void ResizableBoard::hoverEnterEvent( QGraphicsSceneHoverEvent * event ) {
 	Board::hoverEnterEvent(event);
 }
@@ -1048,6 +1215,7 @@ ResizableBoard::Corner ResizableBoard::findCorner(QPointF scenePos, Qt::Keyboard
 	Q_UNUSED(modifiers);
 		
 	if (!this->isSelected()) return ResizableBoard::NO_CORNER;
+    if (this->moveLock()) return ResizableBoard::NO_CORNER;
 
 	double d = CornerHandleSize / m_currentScale;
 	double d2 = d * d;
@@ -1323,7 +1491,7 @@ void ResizableBoard::updatePaperSizes(double w, double h) {
 void ResizableBoard::initPaperSizes() {
     if (PaperSizeNames.count() == 0) {
         PaperSizeNames << tr("A0 (1030x1456)") << tr("A1 (728x1030)") << tr("A2 (515x728)") << tr("A3 (364x515)") << tr("A4 (257x364)") << tr("A5 (182x257)") << tr("A6 (128x182)") 
-            << tr("Letter (8.5x11)") << tr("Legal (8.5x14)") << tr("Ledger (17x11)") << tr("Tabloid (11x17");
+            << tr("Letter (8.5x11)") << tr("Legal (8.5x14)") << tr("Ledger (17x11)") << tr("Tabloid (11x17)");
         PaperSizeDimensions << QSizeF(1030,1456) << QSizeF(728,1030) << QSizeF(515,728) << QSizeF(364,515) << QSizeF(257,364) << QSizeF(182,257) << QSizeF(128,182) 
             << QSizeF(215.9,279.4) << QSizeF(215.9,355.6) << QSizeF(432,279) << QSizeF(279,432);
     }
