@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6563 $:
+$Revision: 6999 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-13 10:32:03 +0200 (Sat, 13 Oct 2012) $
+$Date: 2013-04-28 14:14:07 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -38,6 +38,7 @@ $Date: 2012-10-13 10:32:03 +0200 (Sat, 13 Oct 2012) $
 #include <QProcess>
 #include <QDockWidget>
 #include <QXmlStreamWriter>
+#include <QRegExp>
 
 #include "fritzingwindow.h"
 #include "sketchareawidget.h"
@@ -107,6 +108,29 @@ protected:
     GridSizeThing * m_gridSizeThing;
 };
 
+
+struct TraceMenuThing {
+    int boardCount;
+    int boardSelectedCount;
+	bool jiEnabled;
+	bool exEnabled;
+	bool viaEnabled;
+	bool exChecked;
+	bool gfrEnabled;
+    bool gfsEnabled;
+
+    TraceMenuThing() {
+        boardCount = 0;
+        boardSelectedCount = 0;
+        gfsEnabled = false;
+        jiEnabled = false;
+	    exEnabled = false;
+	    exChecked = true;
+	    viaEnabled = false;
+	    gfrEnabled = false;
+    }
+};
+
 class MainWindow : public FritzingWindow
 {
     Q_OBJECT
@@ -143,7 +167,7 @@ public:
 	// fritzing, then this functions may help with the bundle tasks
 	// on the complex entities: sketches, bins, modules (?)
 	void saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave, bool askForFilename, const QString & destFolderPath, bool saveModel, bool deleteLeftovers);
-	void loadBundledNonAtomicEntity(const QString &filename, Bundler *bundler, bool addToBin, bool dontAsk);
+	bool loadBundledNonAtomicEntity(const QString &filename, Bundler *bundler, bool addToBin, bool dontAsk);
 	void saveAsShareable(const QString & path, bool saveModel);
 
 	
@@ -153,7 +177,7 @@ public:
 	void setCloseSilently(bool);
 	class PCBSketchWidget * pcbView();
 	void noBackup();
-	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID, bool useViewLayerSpec, ViewLayer::ViewLayerSpec);
+	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID, bool useViewLayerPlacement, ViewLayer::ViewLayerPlacement, QMap<QString, QString> & propsMap);
     void swapLayers(ItemBase * itemBase, int layers, const QString & msg, int delay);
 	bool saveAsAux(const QString & fileName);
 	void swapObsolete(bool displayFeedback);
@@ -165,11 +189,15 @@ public:
 	void copperFill(ViewLayer::ViewLayerID);
     bool hasAnyAlien();
     void exportSvg(double res, bool selectedItems, bool flatten, const QString & filename);
-    void setCurrentView(ViewLayer::ViewIdentifier);
+    void setCurrentView(ViewLayer::ViewID);
     bool usesPart(const QString & moduleID);
+    bool anyUsePart(const QString & moduleID);
     bool updateParts(const QString & moduleID, QUndoCommand * parentCommand);
 	void updatePartsBin(const QString & moduleID);
 	bool hasCustomBoardShape();
+    void selectPartsWithModuleID(ModelPart *);
+    void addToSketch(QList<ModelPart *> &);
+	QStringList newDesignRulesCheck(bool showOkMessage);
 
 public:
 	static void initNames();
@@ -204,6 +232,9 @@ public slots:
 	void updateTraceMenu();
 	virtual void updateFileMenu();
     void showStatusMessage(const QString &);
+    void orderFabHoverEnter();
+    void orderFabHoverLeave();
+    void setGroundFillKeepout();
 
 protected slots:
 	void mainLoad();
@@ -220,7 +251,7 @@ protected slots:
 	void pasteInPlace();
     void duplicate();
     void doDelete();
-    void doDeletePlus();
+    void doDeleteMinus();
     void selectAll();
     void deselect();
     void zoomIn();
@@ -234,6 +265,7 @@ protected slots:
     void setBackgroundColor();
     void showBreadboardView();
     void showSchematicView();
+    void showProgramView();
 	void showPartsBinIconView();
 	void showPartsBinListView();
     void updateEditMenu();
@@ -263,6 +295,12 @@ protected slots:
 	void sendBackward();
 	void bringForward();
 	void bringToFront();
+    void alignLeft();
+    void alignRight();
+    void alignVerticalCenter();
+    void alignTop();
+    void alignHorizontalCenter();
+    void alignBottom();
 	void rotate90cw();
 	void rotate90ccw();
 	void rotate180();
@@ -293,7 +331,7 @@ protected slots:
 	void setInfoViewOnHover(bool infoViewOnHover);
 	void updateItemMenu();
 
-	void autoroute();
+	void newAutoroute();
 	void orderFab();
 	void activeLayerTop();
 	void activeLayerBottom();
@@ -333,6 +371,7 @@ protected slots:
 	void startSaveInstancesSlot(const QString & fileName, ModelPart *, QXmlStreamWriter &);
 	void loadedViewsSlot(class ModelBase *, QDomElement & views);
 	void loadedRootSlot(const QString & filename, ModelBase *, QDomElement & views);
+    void obsoleteSMDOrientationSlot();
 	void exportNormalizedSVG();
 	void exportNormalizedFlattenedSVG();
 
@@ -348,9 +387,8 @@ protected slots:
 
 	void openProgramWindow();
 	void linkToProgramFile(const QString & filename, const QString & language, const QString & programmer, bool addLink, bool strong);
-	void newDesignRulesCheck();
-	void designRulesKeepout();
-	void subSwapSlot(SketchWidget *, ItemBase *, const QString & newModuleID, ViewLayer::ViewLayerSpec, long & newID, QUndoCommand * parentCommand);
+	QStringList newDesignRulesCheck();
+	void subSwapSlot(SketchWidget *, ItemBase *, const QString & newModuleID, ViewLayer::ViewLayerPlacement, long & newID, QUndoCommand * parentCommand);
 	void updateLayerMenuSlot();
 	bool save();
 	bool saveAs();
@@ -361,7 +399,6 @@ protected slots:
 	void changeTraceLayer();
 	void routingStatusLabelMousePress(QMouseEvent*);
 	void routingStatusLabelMouseRelease(QMouseEvent*);
-	void updateNet();
 	void selectMoveLock();
 	void moveLock();
 	void setSticky();
@@ -384,9 +421,17 @@ protected slots:
     void keepMargins();
 	void dockChangeActivation(bool activate, QWidget * originator);
     void addToMyParts(ModelPart *);
+    void hidePartSilkscreen();
+    void fabQuote();
+    void findPartInSketch();
+    void fireQuote();
+    void setViewFromBelowToggle();
+    void setViewFromBelow();
+    void setViewFromAbove();
 
 protected:
 	void initSketchWidget(SketchWidget *);
+    virtual void initProgrammingWidget();
 
     virtual void createActions();
     virtual void createFileMenuActions();
@@ -412,7 +457,7 @@ protected:
 	void closeEvent(QCloseEvent * event);
 	void saveAsAuxAux(const QString & fileName);
 	void printAux(QPrinter &printer, bool removeBackground, bool paginate);
-	void exportAux(QString fileName, QImage::Format format, bool removeBackground);
+	void exportAux(QString fileName, QImage::Format format, int quality, bool removeBackground);
 	void notYetImplemented(QString action);
 	bool eventFilter(QObject *obj, QEvent *event);
 	void setActionsIcons(int index, QList<QAction *> &);
@@ -420,11 +465,12 @@ protected:
 	void exportToGerber();
     void exportBOM();
     void exportNetlist();
+    void exportSpiceNetlist();
     void exportSvg(double res, bool selectedItems, bool flatten);
 	void exportSvgWatermark(QString & svg, double res);
 	void exportEtchable(bool wantPDF, bool wantSVG);
 
-	virtual QList<QWidget*> getButtonsForView(ViewLayer::ViewIdentifier viewId);
+	virtual QList<QWidget*> getButtonsForView(ViewLayer::ViewID viewId);
 	const QString untitledFileName();
 	int &untitledFileCount();
 	const QString fileExtension();
@@ -435,10 +481,12 @@ protected:
 
 	void createTraceMenuActions();
 	void hideShowTraceMenu();
+    void hideShowProgramMenu();
+    void updatePCBTraceMenu(QGraphicsItem *, TraceMenuThing &);
 
-	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
-	void copyToSvgFolder(const QFileInfo& file, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
-	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder, bool importingSinglePart);
+	QString copyToSvgFolder(const QFileInfo& file, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
 
 	void closeIfEmptySketch(MainWindow* mw);
 	bool whatToDoWithAlienFiles();
@@ -462,6 +510,7 @@ protected:
 	SketchToolButton *createAutorouteButton(SketchAreaWidget *parent);
 	SketchToolButton *createOrderFabButton(SketchAreaWidget *parent);
 	QWidget *createActiveLayerButton(SketchAreaWidget *parent);
+	QWidget *createViewFromButton(SketchAreaWidget *parent);
 	class ExpandingLabel * createRoutingStatusLabel(SketchAreaWidget *);
 	SketchToolButton *createExportEtchableButton(SketchAreaWidget *parent);
 	SketchToolButton *createNoteButton(SketchAreaWidget *parent);
@@ -478,7 +527,7 @@ protected:
 
 	bool alreadyOpen(const QString & fileName);
 	void svgMissingLayer(const QString & layername, const QString & path);
-	long swapSelectedAuxAux(ItemBase * itemBase, const QString & moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, QUndoCommand * parentCommand);
+	long swapSelectedAuxAux(ItemBase * itemBase, const QString & moduleID, ViewLayer::ViewLayerPlacement viewLayerPlacement, QMap<QString, QString> & propsMap, QUndoCommand * parentCommand);
 	bool swapSpecial(const QString & prop, QMap<QString, QString> & currPropsMap);
 
 	void enableAddBendpointAct(QGraphicsItem *);
@@ -500,12 +549,12 @@ protected:
 	QString getBomProps(ItemBase *);
 	ModelPart * findReplacedby(ModelPart * originalModelPart);
 	void groundFillAux(bool fillGroundTraces, ViewLayer::ViewLayerID viewLayerID);
+    void groundFillAux2(bool fillGroundTraces);
 	void connectStartSave(bool connect);
 	void loadBundledSketch(const QString &fileName, bool addToRecent, bool setAsLastOpened);
     void dropEvent(QDropEvent *event);
     void dragEnterEvent(QDragEnterEvent *event);
     void mainLoadAux(const QString & fileName);
-    bool updateExportActions();
 	QWidget * createGridSizeForm(GridSizeThing *);
     void massageOutput(QString & svg, bool doMask, bool doSilk, bool doPaste, QString & maskTop, QString & maskBottom, const QString & fileName, ItemBase * board, int dpi, const LayerList &);
     virtual void initLockedFiles(bool lockFiles);
@@ -542,7 +591,7 @@ protected:
 	virtual void setCurrentTabIndex(int);
 	virtual QWidget * currentTabWidget();
 	virtual bool activeLayerWidgetAlwaysOn();
-
+    bool copySvg(const QString & path, QFileInfoList & svgEntryInfoList);
 
 protected:
 	static void removeActionsStartingAt(QMenu *menu, int start=0);
@@ -622,6 +671,14 @@ protected:
 	class WireAction *m_sendBackwardWireAct;
 	class WireAction *m_sendToBackWireAct;
 
+    QMenu *m_alignMenu;
+    QAction * m_alignVerticalCenterAct;
+    QAction * m_alignHorizontalCenterAct;
+    QAction * m_alignTopAct;
+    QAction * m_alignLeftAct;
+    QAction * m_alignBottomAct;
+    QAction * m_alignRightAct;
+
 	// Export Menu
 	QMenu *m_exportMenu;
 	QAction *m_exportJpgAct;
@@ -634,6 +691,7 @@ protected:
 	QAction *m_exportEtchableSvgAct;
 	QAction *m_exportBomAct;
 	QAction *m_exportNetlistAct;
+	QAction *m_exportSpiceNetlistAct;
 	QAction *m_exportSvgAct;
 
     // Edit Menu
@@ -646,9 +704,9 @@ protected:
     QAction *m_pasteInPlaceAct;
     QAction *m_duplicateAct;
     QAction *m_deleteAct;
-    QAction *m_deletePlusAct;
+    QAction *m_deleteMinusAct;
     class WireAction *m_deleteWireAct;
-    class WireAction *m_deleteWirePlusAct;
+    class WireAction *m_deleteWireMinusAct;
     QAction *m_selectAllAct;
     QAction *m_deselectAct;
 	QAction *m_addNoteAct;
@@ -679,6 +737,7 @@ protected:
 	QAction *m_disconnectAllAct;
 	QAction *m_selectAllObsoleteAct;
 	QAction *m_swapObsoleteAct;
+    QAction *m_findPartInSketchAct;
 	QAction * m_openProgramWindowAct;
 
 	QAction *m_addBendpointAct;
@@ -689,6 +748,8 @@ protected:
     QAction *m_showAllLayersAct;
 	QAction *m_hideAllLayersAct;
 
+    QAction *m_hidePartSilkscreenAct;
+
     // View Menu
     QMenu *m_viewMenu;
     QAction *m_zoomInAct;
@@ -702,6 +763,7 @@ protected:
     QAction *m_setBackgroundColorAct;
     QAction *m_showBreadboardAct;
     QAction *m_showSchematicAct;
+    QAction *m_showProgramAct;
     QAction *m_showPCBAct;
 	QAction *m_showPartsBinIconViewAct;
 	QAction *m_showPartsBinListViewAct;
@@ -722,15 +784,19 @@ protected:
 	QMenu *m_pcbTraceMenu;
 	QMenu *m_schematicTraceMenu;
 	QMenu *m_breadboardTraceMenu;
-	QAction *m_autorouteAct;
+	QAction *m_newAutorouteAct;
 	QAction *m_orderFabAct;
 	QAction *m_activeLayerTopAct;
 	QAction *m_activeLayerBottomAct;
 	QAction *m_activeLayerBothAct;
+	QAction *m_viewFromBelowToggleAct;
+	QAction *m_viewFromBelowAct;
+	QAction *m_viewFromAboveAct;
 	class WireAction *m_createTraceWireAct;
 	class WireAction *m_createWireWireAct;
 	QAction *m_createJumperAct;
 	QAction *m_changeTraceLayerAct;
+	class WireAction *m_changeTraceLayerWireAct;
 	QAction *m_excludeFromAutorouteAct;
 	class WireAction *m_excludeFromAutorouteWireAct;
     QAction * m_showUnroutedAct;
@@ -748,9 +814,10 @@ protected:
 	class ConnectorItemAction *m_setOneGroundFillSeedAct;
 	QAction *m_setGroundFillSeedsAct;
 	QAction *m_clearGroundFillSeedsAct;
+    QAction *m_setGroundFillKeepoutAct;
 	QAction *m_newDesignRulesCheckAct;
-	QAction *m_newDesignRulesKeepoutAct;
 	QAction *m_autorouterSettingsAct;
+	QAction *m_fabQuoteAct;
 	QAction *m_tidyWiresAct;
 	QAction *m_checkLoadedTracesAct;
 
@@ -770,7 +837,8 @@ protected:
 	QAction *m_partsEditorHelpAct;
 
 	// Wire Color Menu
-	QMenu * m_wireColorMenu;
+	QMenu * m_breadboardWireColorMenu;
+	QMenu * m_schematicWireColorMenu;
 
     // Dot icons
     QIcon m_dotIcon;
@@ -779,6 +847,7 @@ protected:
 	QList<SketchToolButton*> m_rotateButtons;
 	QList<SketchToolButton*> m_flipButtons;
 	QStackedWidget * m_activeLayerButtonWidget;
+	QStackedWidget * m_viewFromButtonWidget;
 
     bool m_comboboxChanged;
     bool m_restarting;
@@ -809,6 +878,7 @@ protected:
 
 	QPointer<class LayerPalette> m_layerPalette;
 	QPointer<class ProgramWindow> m_programWindow;
+	QPointer<class ProgramWindow> m_programView;
 	QList<LinkedFile *>  m_linkedProgramFiles;
 	QString m_backupFileNameAndPath;
 	QTimer m_autosaveTimer;
@@ -833,7 +903,8 @@ protected:
 	QString m_oldTopDockStyle;
 	QString m_oldBottomDockStyle;
 	bool m_dontKeepMargins;
-
+    QPointer<QDialog> m_rolloverQuoteDialog;
+    bool m_obsoleteSMDOrientation;
 
 public:
 	static int AutosaveTimeoutMinutes;
@@ -848,6 +919,7 @@ protected:
 	static int UntitledSketchIndex;
 	static int CascadeFactorX;
 	static int CascadeFactorY;
+    static QRegExp GuidMatcher;
 };
 
 
