@@ -1,7 +1,7 @@
 /*******************************************************************
 skw
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6567 $:
+$Revision: 6984 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-15 15:13:22 +0200 (Mon, 15 Oct 2012) $
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -33,7 +33,6 @@ $Date: 2012-10-15 15:13:22 +0200 (Mon, 15 Oct 2012) $
 #include "../debugdialog.h"
 #include "../waitpushundostack.h"
 #include "../help/aboutbox.h"
-#include "../autoroute/cmrouter/cmrouter.h"
 #include "../autoroute/autorouteprogressdialog.h"
 #include "../items/virtualwire.h"
 #include "../items/jumperitem.h"
@@ -76,6 +75,7 @@ static QString pngActionType = ".png";
 static QString svgActionType = ".svg";
 static QString bomActionType = ".html";
 static QString netlistActionType = ".xml";
+static QString spiceNetlistActionType = ".cir";
 
 static QHash<QString, QPrinter::OutputFormat> filePrintFormats;
 static QHash<QString, QImage::Format> fileExportFormats;
@@ -118,7 +118,7 @@ bool sortPartList(ItemBase * b1, ItemBase * b2) {
 
 void MainWindow::initNames()
 {
-	OtherKnownExtensions << jpgActionType << psActionType << pdfActionType << pngActionType << svgActionType << bomActionType << netlistActionType;
+	OtherKnownExtensions << jpgActionType << psActionType << pdfActionType << pngActionType << svgActionType << bomActionType << netlistActionType << spiceNetlistActionType;
 
 	filePrintFormats[pdfActionType] = QPrinter::PdfFormat;
 	filePrintFormats[psActionType] = QPrinter::PostScriptFormat;
@@ -224,6 +224,7 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG)
 		fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_paste_mask_top%1", suffix));
 	}
 	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_silk_top%1", suffix));
+	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_silk_bottom%1", suffix));
 
 	QString maskTop, maskBottom;
 	QList<ItemBase *> copperLogoItems, holes;
@@ -253,6 +254,10 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG)
 			viewLayerIDs << ViewLayer::Silkscreen1 << ViewLayer::Silkscreen1Label;
 			doSilk = true;
 		}
+		else if (fileName.contains("silk_bottom")) {
+			viewLayerIDs << ViewLayer::Silkscreen0 << ViewLayer::Silkscreen0Label;
+			doSilk = true;
+		}
 
 		if (doMask) {
 			m_pcbGraphicsView->hideCopperLogoItems(copperLogoItems);
@@ -262,9 +267,13 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG)
 		}
 
 		if (wantSVG) {
-			bool empty;
-		    QRectF imageRect;
-			QString svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageRect, board, GraphicsUtils::IllustratorDPI, false, false, empty);
+			RenderThing renderThing;
+            renderThing.printerScale = GraphicsUtils::SVGDPI;
+            renderThing.blackOnly = true;
+            renderThing.dpi = GraphicsUtils::IllustratorDPI;
+            renderThing.hideTerminalPoints = true;
+            renderThing.selectedItems = renderThing.renderBlocker = false;
+			QString svg = m_pcbGraphicsView->renderToSVG(renderThing, board, viewLayerIDs);
 			massageOutput(svg, doMask, doSilk, doPaste, maskTop, maskBottom, fileName, board, GraphicsUtils::IllustratorDPI, viewLayerIDs);		
 			QString merged = mergeBoardSvg(svg, board, GraphicsUtils::IllustratorDPI, false, viewLayerIDs);
             TextUtils::writeUtf8(fileName.arg(""), merged);
@@ -283,9 +292,13 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG)
 			    int res = printer.resolution();
 
                 if (svg.isEmpty()) {
-			        bool empty;
-                    QRectF imageRect;
-			        svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageRect, board, res, false, false, empty);
+			        RenderThing renderThing;
+                    renderThing.printerScale = GraphicsUtils::SVGDPI;
+                    renderThing.blackOnly = true;
+                    renderThing.dpi = res;
+                    renderThing.hideTerminalPoints = true;
+                    renderThing.selectedItems = renderThing.renderBlocker = false;
+			        svg = m_pcbGraphicsView->renderToSVG(renderThing, board, viewLayerIDs);
 			        massageOutput(svg, doMask, doSilk, doPaste, maskTop, maskBottom, fileName, board, res, viewLayerIDs);
                 }
 			
@@ -395,10 +408,13 @@ QString MainWindow::mergeBoardSvg(QString & svg, ItemBase * board, int res, bool
 	QString boardSvg = getBoardSvg(board, res, viewLayerIDs);
 
     LayerList outlineLayerIDs = ViewLayer::outlineLayers();
-	QRectF imageRect;
-	bool empty;
-
-	QString outlineSvg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, outlineLayerIDs, true, imageRect, board, res, false, false, empty);
+	RenderThing renderThing;
+    renderThing.printerScale = GraphicsUtils::SVGDPI;
+    renderThing.blackOnly = true;
+    renderThing.dpi = res;
+    renderThing.hideTerminalPoints = true;
+    renderThing.selectedItems = renderThing.renderBlocker = false;
+	QString outlineSvg = m_pcbGraphicsView->renderToSVG(renderThing, board, outlineLayerIDs);
 	outlineSvg = GerberGenerator::cleanOutline(outlineSvg);
     outlineSvg = TextUtils::slamStrokeAndFill(outlineSvg, "black", "0.5", "none");
 
@@ -440,9 +456,13 @@ QString MainWindow::getBoardSvg(ItemBase * board, int res,  LayerList & viewLaye
 	}
 	board->setSelected(true);
 
-	bool empty;
-    QRectF imageRect;
-	QString svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageRect, board, res, true, false, empty);
+	RenderThing renderThing;
+    renderThing.printerScale = GraphicsUtils::SVGDPI;
+    renderThing.blackOnly = true;
+    renderThing.dpi = res;
+    renderThing.selectedItems = renderThing.hideTerminalPoints = true;
+    renderThing.renderBlocker = false;
+	QString svg = m_pcbGraphicsView->renderToSVG(renderThing, board, viewLayerIDs);
 	board->setSelected(false);
 	foreach (QGraphicsItem * item, items) {
 		item->setSelected(true);
@@ -481,6 +501,11 @@ void MainWindow::doExport() {
 		return;
 	}
 
+	if (actionType.compare(spiceNetlistActionType) == 0) {
+		exportSpiceNetlist();
+		return;
+	}
+
 	if (actionType.compare(svgActionType) == 0) {
 		exportSvg(GraphicsUtils::IllustratorDPI, false, false);
 		return;
@@ -515,7 +540,8 @@ void MainWindow::doExport() {
 				m_statusBar->showMessage(tr("Sketch exported"), 2000);
 			} else { // PNG...
 				DebugDialog::debug(QString("format: %1 %2").arg(fileExt).arg(fileExportFormats[actionType]));
-				exportAux(fileName,fileExportFormats[actionType], true);
+                int quality = (actionType == pngActionType ? 1 : 100);
+				exportAux(fileName,fileExportFormats[actionType], quality, true);
 			}
 			delete fileProgressDialog;
 
@@ -523,7 +549,7 @@ void MainWindow::doExport() {
 	#endif
 }
 
-void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeBackground) 
+void MainWindow::exportAux(QString fileName, QImage::Format format, int quality, bool removeBackground) 
 {
     double resMultiplier = 3;
 
@@ -572,6 +598,8 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 	m_currentGraphicsView->scene()->render(&painter, target, source, Qt::KeepAspectRatio);
 	painter.end();
 
+    //image.save(FolderUtils::getUserDataStorePath("") + "/export.png");
+
 	if (removeBackground) {
 		m_currentGraphicsView->setBackground(color);
 	}
@@ -584,7 +612,7 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 	if (imageWriter.supportsOption(QImageIOHandler::Description)) {
 		imageWriter.setText("", TextUtils::CreatedWithFritzingString);
 	}
-    imageWriter.setQuality(100);
+    imageWriter.setQuality(quality);
 	bool result = imageWriter.write(image);
 	if (!result) {
 		QMessageBox::warning(this, tr("Fritzing"), tr("Unable to save %1").arg(fileName) );
@@ -825,7 +853,7 @@ void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &ex
 	QString dirToRemove;
 	if (destFolderPath.isEmpty()) {
 		destFolder = QDir::temp();
-		FolderUtils::createFolderAnCdIntoIt(destFolder, FolderUtils::getRandText());
+		FolderUtils::createFolderAnCdIntoIt(destFolder, TextUtils::getRandText());
 		dirToRemove = destFolder.path();
 	}
 	else {
@@ -939,6 +967,11 @@ void MainWindow::createExportActions() {
     m_exportNetlistAct->setStatusTip(tr("Save a netlist in XML format"));
     connect(m_exportNetlistAct, SIGNAL(triggered()), this, SLOT(doExport()));
 
+    m_exportSpiceNetlistAct = new QAction(tr("SPICE Netlist..."), this);
+    m_exportSpiceNetlistAct->setData(spiceNetlistActionType);
+    m_exportSpiceNetlistAct->setStatusTip(tr("Save a netlist in SPICE format"));
+    connect(m_exportSpiceNetlistAct, SIGNAL(triggered()), this, SLOT(doExport()));
+
 	m_exportEagleAct = new QAction(tr("Eagle..."), this);
 	m_exportEagleAct->setData(eagleActionType);
 	m_exportEagleAct->setStatusTip(tr("Export the current sketch to Eagle CAD"));
@@ -1060,9 +1093,14 @@ void MainWindow::exportSvg(double res, bool selectedItems, bool flatten, const Q
 		viewLayerIDs << viewLayer->viewLayerID();
 	}
 
-	QRectF imageRect;
-	bool empty;
-	QString svg = m_currentGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, false, imageRect, NULL, res, selectedItems, false, empty);
+	RenderThing renderThing;
+    renderThing.printerScale = GraphicsUtils::SVGDPI;
+    renderThing.blackOnly = false;
+    renderThing.dpi = res;
+    renderThing.selectedItems = selectedItems;
+    renderThing.hideTerminalPoints = true;
+    renderThing.renderBlocker = false;
+	QString svg = m_currentGraphicsView->renderToSVG(renderThing, NULL, viewLayerIDs);
 	if (svg.isEmpty()) {
 		// tell the user something reasonable
 		return;
@@ -1093,7 +1131,8 @@ void MainWindow::exportSvgWatermark(QString & svg, double res)
 	bool result = splitter.splitString(watermarkSvg, "watermark");
 	if (!result) return;
 
-	result = splitter.normalize(res, "watermark", false);
+    double factor;
+	result = splitter.normalize(res, "watermark", false, factor);
 	if (!result) return;
 
 	QString transWatermark = splitter.shift((svgSize.width() - watermarkSize.width()) * res, svgSize.height() * res, "watermark", true);
@@ -1130,8 +1169,8 @@ void MainWindow::exportBOM() {
 	}
 
     QList <ItemBase*> partList;
-    QMap<QString, int> shoppingList;
-	QHash<QString, ItemBase *> descrs;
+    QList<QString> descrList;
+	QMultiHash<QString, ItemBase *> descrs;
 
 	m_currentGraphicsView->collectParts(partList);
 
@@ -1140,18 +1179,13 @@ void MainWindow::exportBOM() {
 	foreach (ItemBase * itemBase, partList) {
 		if (itemBase->itemType() != ModelPart::Part) continue;
 
-        QString desc = getBomProps(itemBase);
-
-        if(!shoppingList.contains(desc)) {
-            shoppingList.insert(desc, 1);
-			descrs.insert(desc, itemBase);
-        }
-        else {
-            shoppingList[desc]++;
+        QString desc = itemBase->title() + "%%%%%" + getBomProps(itemBase);  // keeps different parts separate if there are no properties
+        descrs.insert(desc, itemBase);
+        if (!descrList.contains(desc)) {
+            descrList.append(desc);
         }
     }
 
-
 	QString assemblyString;
 	foreach (ItemBase * itemBase, partList) {
 		if (itemBase->itemType() != ModelPart::Part) continue;
@@ -1160,13 +1194,10 @@ void MainWindow::exportBOM() {
     }
 
 	QString shoppingListString;
-    QMapIterator<QString, int> it(shoppingList);
-    while (it.hasNext()) {
-        it.next();
-		ItemBase * itemBase = descrs.value(it.key());
-
-		// it.value() = count; it.key() = bomProps
-		shoppingListString += bomRowTemplate.arg(it.value()).arg(itemBase->title()).arg(it.key());
+    foreach (QString descr, descrList) {
+        QList<ItemBase *> itemBases = descrs.values(descr);
+        QStringList split = descr.split("%%%%%");
+		shoppingListString += bomRowTemplate.arg(itemBases.count()).arg(split.at(0)).arg(split.at(1));
     }
 
 	QString bom = bomTemplate
@@ -1219,6 +1250,170 @@ void MainWindow::exportBOM() {
 	delete fileProgressDialog;
 }
 
+void MainWindow::exportSpiceNetlist() {
+    if (m_schematicGraphicsView == NULL) return;
+
+    // examples: 
+    // http://www.allaboutcircuits.com/vol_5/chpt_7/8.html
+    // http://cutler.eecs.berkeley.edu/classes/icbook/spice/UserGuide/elements_fr.html
+    // http://www.csd.uoc.gr/~hy422/2011s/datasheets/ngspice-user-manual.pdf
+
+    QString path = defaultSaveFolder();
+    QString fileExt;
+    QString extFmt = fileExtFormats.value(spiceNetlistActionType);
+    QString fname = path + "/" + constructFileName("spice", spiceNetlistActionType);
+    //DebugDialog::debug(QString("fname %1\n%2").arg(fname).arg(extFmt));
+    QString fileName = FolderUtils::getSaveFileName(this,
+            tr("Export SPICE Netlist..."),
+            fname,
+            extFmt,
+            &fileExt
+    );
+
+    if (fileName.isEmpty()) {
+		return; //Cancel pressed
+    }
+
+    static QRegExp curlies("\\{([^\\}]*)\\}");
+
+    QFileInfo fileInfo(m_fwFilename);
+    QString output = fileInfo.completeBaseName();
+    output += "\n";
+
+	QHash<ConnectorItem *, int> indexer;
+	QList< QList<ConnectorItem *>* > netList;
+	this->m_schematicGraphicsView->collectAllNets(indexer, netList, true, false);
+
+
+    //DebugDialog::debug("_______________");
+    QSet<ItemBase *> itemBases;
+    QList<ConnectorItem *> * ground = NULL;
+	foreach (QList<ConnectorItem *> * net, netList) {
+        if (net->count() < 2) continue;
+
+        foreach (ConnectorItem * ci, *net) {
+            //ci->debugInfo("net");
+            if (ci->isGrounded()) {
+                ground = net;
+            }
+            itemBases.insert(ci->attachedTo());
+        }
+        //DebugDialog::debug("_______________");
+    }
+
+    if (ground) {
+        // make sure ground is index zero
+        netList.removeOne(ground);
+        netList.prepend(ground);
+    }
+
+    //DebugDialog::debug("_______________");
+    //DebugDialog::debug("_______________");
+    DebugDialog::debug("_______________");
+
+	foreach (QList<ConnectorItem *> * net, netList) {
+        if (net->count() < 2) continue;
+
+        foreach (ConnectorItem * ci, *net) {
+            ci->debugInfo("net");
+        }
+
+        DebugDialog::debug("_______________");
+    }
+
+    //DebugDialog::debug("_______________");
+    //DebugDialog::debug("_______________");
+
+    foreach (ItemBase * itemBase, itemBases) {
+        QString spice = itemBase->spice();
+        if (spice.isEmpty()) continue;
+
+        while (true) {
+            int ix = curlies.indexIn(spice);
+            if (ix < 0) break;
+
+            QString token = curlies.cap(1).toLower();
+            QString replacement;
+            if (token == "instancetitle") {
+                replacement = itemBase->instanceTitle();
+                if (ix > 0 && replacement.at(0).toLower() == spice.at(ix - 1).toLower()) {
+                    // if the type letter is repeated
+                    replacement = replacement.mid(1);
+                }
+                replacement.replace(" ", "_");
+            }
+            else if (token.startsWith("net ")) {
+                QString cname = token.mid(4).trimmed();
+                foreach (ConnectorItem * ci, itemBase->cachedConnectorItems()) {
+                    if (ci->connectorSharedID().toLower() == cname) {
+                        int ix = -1;
+	                    foreach (QList<ConnectorItem *> * net, netList) {
+                            ix++;
+                            if (net->contains(ci)) break;
+                        }
+
+                        replacement = QString::number(ix);
+                        break;
+                    }
+                }
+            }
+            else {
+                QVariant variant = itemBase->modelPart()->localProp(token);
+                if (variant.isNull()) {
+                    replacement = itemBase->modelPart()->properties().value(token, "");
+                }
+                else {
+                    replacement = variant.toString();
+                }
+            }
+
+            spice.replace(ix, curlies.cap(0).count(), replacement);
+            DebugDialog::debug("spice " + spice);
+        }
+
+        output += spice;  
+    }
+
+    output += "\n";
+
+    // remove redundant models
+    QStringList models;
+    foreach (ItemBase * itemBase, itemBases) {
+        QString spiceModel = itemBase->spiceModel();
+        if (spiceModel.isEmpty()) continue;
+        if (models.contains(spiceModel, Qt::CaseInsensitive)) continue;
+
+        models.append(spiceModel);
+    }
+
+    foreach (QString model, models) {
+        output += model;
+        output += "\n";
+    }
+
+
+    output += ".TRAN 1ms 100ms\n";
+    output += "* .AC DEC 100 100 1MEG\n";
+    output += ".END\n";
+
+	foreach (QList<ConnectorItem *> * net, netList) {
+		delete net;
+	}
+	netList.clear();
+
+	QClipboard *clipboard = QApplication::clipboard();
+	if (clipboard != NULL) {
+		clipboard->setText(output);
+	}
+
+    //DebugDialog::debug(fileExt + " selected to export");
+    if(!alreadyHasExtension(fileName, spiceNetlistActionType)) {
+		fileName += spiceNetlistActionType;
+    }
+
+    TextUtils::writeUtf8(fileName, output);
+}
+
 void MainWindow::exportNetlist() {
 	QHash<ConnectorItem *, int> indexer;
 	QList< QList<ConnectorItem *>* > netList;
@@ -1323,8 +1518,6 @@ void MainWindow::exportNetlist() {
 		clipboard->setText(doc.toByteArray());
 	}
 	delete fileProgressDialog;
-
-
 }
 
 FileProgressDialog * MainWindow::exportProgress() {
@@ -1354,7 +1547,8 @@ QString MainWindow::getBomProps(ItemBase * itemBase)
 		QWidget widget;
 		QWidget * resultWidget = NULL;
 		QString resultKey, resultValue;
-		itemBase->collectExtraInfo(&widget, properties.value("family"), key, value, false, resultKey, resultValue, resultWidget);
+        bool hide;
+		itemBase->collectExtraInfo(&widget, properties.value("family"), key, value, false, resultKey, resultValue, resultWidget, hide);
 		if (resultValue.isEmpty()) continue;
 
 		pString += ItemBase::translatePropertyName(resultKey) + " " + resultValue + "; ";
@@ -1403,6 +1597,7 @@ void MainWindow::exportToGerber() {
         prefix += QString("_%1_%2").arg(board->instanceTitle()).arg(board->id());
     }
 	GerberGenerator::exportToGerber(prefix, exportDir, board, m_pcbGraphicsView, true);
+
 	m_pcbGraphicsView->restoreLayerVisibility();
 	m_statusBar->showMessage(tr("Sketch exported to Gerber"), 2000);
 
@@ -1443,4 +1638,3 @@ void MainWindow::massageOutput(QString & svg, bool doMask, bool doSilk, bool doP
 		svg = TextUtils::expandAndFill(svg, "black", GerberGenerator::MaskClearanceMils * 2 * dpi / 1000);
 	}
 }
-
