@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6567 $:
+$Revision: 6963 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-15 15:13:22 +0200 (Mon, 15 Oct 2012) $
+$Date: 2013-04-14 10:13:52 +0200 (So, 14. Apr 2013) $
 
 ********************************************************************/
 
@@ -112,6 +112,7 @@ bool ModelBase::loadFromFile(const QString & fileName, ModelBase * referenceMode
 	bool checkForRats = true;
 	bool checkForTraces = true;
     bool checkForMysteryParts = true;
+    bool checkForObsoleteSMDOrientation = true;
 	m_fritzingVersion = root.attribute("fritzingVersion");
 	if (m_fritzingVersion.length() > 0) {
 		// with version 0.4.3 ratsnests in fz files are obsolete
@@ -131,6 +132,10 @@ bool ModelBase::loadFromFile(const QString & fileName, ModelBase * referenceMode
 		versionThingRats.minorVersion = 7;
 		versionThingRats.minorSubVersion = 5;
 		checkForMysteryParts = !Version::greaterThan(versionThingRats, versionThingFz);
+        // with version 0.8.0 flipSMD is horizontal
+	    versionThingRats.minorVersion = 7;
+	    versionThingRats.minorSubVersion = 13;
+	    checkForObsoleteSMDOrientation = !Version::greaterThan(versionThingRats, versionThingFz);
 	}
 
 	ModelPartSharedRoot * modelPartSharedRoot = this->rootModelPartShared();
@@ -142,6 +147,8 @@ bool ModelBase::loadFromFile(const QString & fileName, ModelBase * referenceMode
 		}
 	}
 
+    // ensures changeBinIcon() is not available
+    // this may be a bug?
 	QString iconFilename = root.attribute("icon");
 	if (iconFilename.isEmpty()) {
 		iconFilename = title.text() + ".png";
@@ -204,6 +211,17 @@ bool ModelBase::loadFromFile(const QString & fileName, ModelBase * referenceMode
 		}
 	}
 
+    if (checkForObsoleteSMDOrientation) {
+		QDomElement instance = instances.firstChildElement("instance");
+   		while (!instance.isNull()) {
+			if (checkObsoleteOrientation(instance)) {
+                emit obsoleteSMDOrientationSignal();
+                break;
+            }
+			instance = instance.nextSiblingElement("instance");
+		}
+    }
+
 	bool result = loadInstances(domDocument, instances, modelParts, checkViews);
 	emit loadedInstances(this, instances);
 	return result;
@@ -237,10 +255,14 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 
    		// for now assume all parts are in the palette
    		QString moduleIDRef = instance.attribute("moduleIdRef");
+
+        //DebugDialog::debug("loading " + moduleIDRef);
 		if (moduleIDRef.compare(ModuleIDNames::SpacerModuleIDName) == 0) {
 			ModelPart * mp = new ModelPart(ModelPart::Space);
 			mp->setInstanceText(instance.attribute("path"));
 			mp->setParent(m_root);
+            mp->modelPartShared()->setModuleID(ModuleIDNames::SpacerModuleIDName);
+            mp->modelPartShared()->setPath(instance.attribute("path"));
 			modelParts.append(mp);
 			instance = instance.nextSiblingElement("instance");
 			continue;
@@ -248,7 +270,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 
 
    		modelPart = m_referenceModel->retrieveModelPart(moduleIDRef);
-   		if (modelPart == NULL) {
+        if (modelPart == NULL) {
 			DebugDialog::debug(QString("module id %1 not found in database").arg(moduleIDRef));
 			modelPart = fixObsoleteModuleID(domDocument, instance, moduleIDRef);
 			if (modelPart == NULL) {
@@ -265,6 +287,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 			}
    		}
 
+        modelPart->setInBin(true);
    		modelPart = addModelPart(m_root, modelPart);
    		modelPart->setInstanceDomElement(instance);
 		modelParts.append(modelPart);
@@ -272,7 +295,7 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
    		// TODO Mariano: i think this is not the way
    		QString instanceTitle = instance.firstChildElement("title").text();
    		if(!instanceTitle.isNull() && !instanceTitle.isEmpty()) {
-   			modelPart->setInstanceTitle(instanceTitle);
+   			modelPart->setInstanceTitle(instanceTitle, false);
    		}
 
 		QDomElement localConnectors = instance.firstChildElement("localConnectors");
@@ -342,6 +365,11 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
 }
 
 ModelPart * ModelBase::addModelPart(ModelPart * parent, ModelPart * copyChild) {
+
+    //if (copyChild->moduleID() == "df9d072afa2b594ac67b60b4153ff57b_29") {
+    //    DebugDialog::debug("alive in here");
+    //}
+
 	ModelPart * modelPart = new ModelPart();
 	modelPart->copyNew(copyChild);
 	modelPart->setParent(parent);
@@ -489,6 +517,11 @@ void ModelBase::renewModelIndexes(QDomElement & parentElement, const QString & c
 		if (!views.isNull()) {
 			QDomElement view = views.firstChildElement();
 			while (!view.isNull()) {
+                bool ok;
+                int superpart = view.attribute("superpart").toLong(&ok);
+                if (ok) {
+                    view.setAttribute("superpart", QString::number(oldToNew.value(superpart)));
+                }
 				QDomElement connectors = view.firstChildElement("connectors");
 				if (!connectors.isNull()) {
 					QDomElement connector = connectors.firstChildElement("connector");
@@ -568,6 +601,15 @@ bool ModelBase::isRatsnest(QDomElement & instance) {
 	return false;
 }
 
+bool ModelBase::checkObsoleteOrientation(QDomElement & instance)
+{
+	QString flippedSMD = instance.attribute("flippedSMD", "");
+	if (flippedSMD != "true") return false;
+
+	QDomElement views = instance.firstChildElement("views");
+	QDomElement pcbView = views.firstChildElement("pcbView");
+    return (pcbView.attribute("layer", "") == "copper0");
+}
 
 void ModelBase::checkTraces(QDomElement & instance) {
 	QString moduleIDRef = instance.attribute("moduleIdRef");
