@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6565 $:
+$Revision: 7004 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-15 12:10:48 +0200 (Mon, 15 Oct 2012) $
+$Date: 2013-04-29 13:10:59 +0200 (Mo, 29. Apr 2013) $
 
 ********************************************************************/
 
@@ -38,19 +38,13 @@ $Date: 2012-10-15 12:10:48 +0200 (Mon, 15 Oct 2012) $
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../items/moduleidnames.h"
+#include "../items/partfactory.h"
 
-bool PaletteModel::CreateAllPartsBinFile = false;  // now generating the all parts bin in advance using a python script:  [fritzing]/part-gen-scripts/misc_scripts/genAllParts.py
-
-bool PaletteModel::CreateNonCorePartsBinFile = false;
-bool PaletteModel::CreateContribPartsBinFile = true;
 bool PaletteModel::CreateTempPartsBinFile = true;
 
 static bool JustAppendAllPartsInstances = false;
 static bool FirstTime = true;
-static bool FirstTimeWrite = true;
 
-QString PaletteModel::AllPartsBinFilePath;
-QString PaletteModel::NonCorePartsBinFilePath;
 QString PaletteModel::ContribPartsBinFilePath;
 
 static QString FritzingContribPath;
@@ -65,12 +59,12 @@ const static QString InstanceTemplate(
         		"\t\t</instance>\n");
 
 
-void setFlip(ViewLayer::ViewIdentifier viewIdentifier, QXmlStreamReader & xml, QHash<ViewLayer::ViewIdentifier, ViewImage *> & viewImages)
+void setFlip(ViewLayer::ViewID viewID, QXmlStreamReader & xml, QHash<ViewLayer::ViewID, ViewImage *> & viewImages)
 {
     bool fv = xml.attributes().value("flipvertical", "").toString().compare("true") == 0;
     bool fh = xml.attributes().value("fliphorizontal", "").toString().compare("true") == 0;
-    viewImages.value(viewIdentifier)->canFlipHorizontal = fh;
-    viewImages.value(viewIdentifier)->canFlipVertical = fv;
+    viewImages.value(viewID)->canFlipHorizontal = fh;
+    viewImages.value(viewID)->canFlipVertical = fv;
 }
 
 /////////////////////////////////////
@@ -87,7 +81,7 @@ PaletteModel::PaletteModel(bool makeRoot, bool doInit) : ModelBase( makeRoot ) {
     m_fullLoad = false;
 
 	if (doInit) {
-		initParts();
+		initParts(false);
 	}
 }
 
@@ -98,7 +92,7 @@ PaletteModel::~PaletteModel()
     }
 }
 
-void PaletteModel::initParts() {
+void PaletteModel::initParts(bool dbExists) {
 	QDir * dir = FolderUtils::getApplicationSubFolder("parts");
 	if (dir == NULL) {
 	    QMessageBox::information(NULL, QObject::tr("Fritzing"),
@@ -109,7 +103,7 @@ void PaletteModel::initParts() {
 	FritzingContribPath = dir->absoluteFilePath("contrib");
 	delete dir;
 
-	loadParts();
+	loadParts(dbExists);
 	if (m_root == NULL) {
 	    QMessageBox::information(NULL, QObject::tr("Fritzing"),
 	                             QObject::tr("No parts found.") );
@@ -117,8 +111,6 @@ void PaletteModel::initParts() {
 }
 
 void PaletteModel::initNames() {
-	AllPartsBinFilePath = FolderUtils::getApplicationSubFolderPath("bins")+"/allParts.dbg" + FritzingBinExtension;
-	NonCorePartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/nonCoreParts" + FritzingBinExtension;
 	ContribPartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/contribParts" + FritzingBinExtension;
 }
 
@@ -137,7 +129,7 @@ bool PaletteModel::containsModelPart(const QString & moduleID) {
 	return m_partHash.contains(moduleID);
 }
 
-void PaletteModel::loadParts() {
+void PaletteModel::loadParts(bool dbExists) {
 	QStringList nameFilters;
 	nameFilters << "*" + FritzingPartExtension;
 
@@ -155,13 +147,11 @@ void PaletteModel::loadParts() {
 
 
 	if (FirstTime) {
-		writeCommonBinsHeader();
 	}
 
 	int totalPartCount = 0;
 	emit loadedPart(0, totalPartCount);
 
-
 	QDir * dir1 = FolderUtils::getApplicationSubFolder("parts");
 	if (dir1 != NULL) {
 		countParts(*dir1, nameFilters, totalPartCount);
@@ -170,7 +160,7 @@ void PaletteModel::loadParts() {
 	countParts(dir2, nameFilters, totalPartCount);
 
 	QDir dir3(":/resources/parts");
-    if (m_fullLoad) {
+    if (m_fullLoad || !dbExists) {
 	    countParts(dir3, nameFilters, totalPartCount);
     }
 
@@ -182,13 +172,12 @@ void PaletteModel::loadParts() {
 
 	loadPartsAux(dir2, nameFilters, loadingPart, totalPartCount);
 
-    if (m_fullLoad) {
+    if (m_fullLoad || !dbExists) {
         // otherwise these will already be in the database
 	    loadPartsAux(dir3, nameFilters, loadingPart, totalPartCount);  
     }
 
 	if (FirstTime) {
-		writeCommonBinsFooter();
 	}
 	
 	JustAppendAllPartsInstances = false;   
@@ -202,103 +191,6 @@ void PaletteModel::loadParts() {
 
 }
 
-void PaletteModel::writeCommonBinsHeader() {
-	writeCommonBinsHeaderAux(CreateAllPartsBinFile, AllPartsBinFilePath, "All Parts");
-	writeCommonBinsHeaderAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath, "All my parts");
-	writeCommonBinsHeaderAux(CreateContribPartsBinFile, ContribPartsBinFilePath, "Contributed Parts");
-}
-
-void PaletteModel::writeCommonBinsHeaderAux(bool doIt, const QString &filename, const QString &binName) {
-	QString header =
-		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + 
-		TextUtils::CreatedWithFritzingXmlComment +
-		QString("<module fritzingVersion='%1' icon='%2.png'>\n").arg(Version::versionString()).arg(binName) +
-		QString("\t<title>%1</title>\n").arg(binName) +
-		"\t<instances>\n";
-	writeCommonBinAux(header, QFile::WriteOnly, doIt, filename);
-}
-
-void PaletteModel::writeCommonBinsFooter() {
-	writeCommonBinsFooterAux(CreateAllPartsBinFile, AllPartsBinFilePath);
-	writeCommonBinsFooterAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath);
-	writeCommonBinsFooterAux(CreateContribPartsBinFile, ContribPartsBinFilePath);
-}
-
-void PaletteModel::writeCommonBinsFooterAux(bool doIt, const QString &filename) {
-	QString footer = "\t</instances>\n</module>\n";
-	writeCommonBinAux(footer, QFile::Append, doIt, filename);
-}
-
-void PaletteModel::writeCommonBinInstance(const QString &moduleID, const QString &path, bool doIt, const QString &filename) {
-	if (!doIt) return;
-	
-	QString pathAux = path;
-	pathAux.remove(FolderUtils::getApplicationSubFolderPath("")+"/");
-
-	if (JustAppendAllPartsInstances) {
-		QString instance = InstanceTemplate.arg(moduleID).arg("");
-		writeCommonBinAux(instance, QFile::Append, doIt, filename);
-	}
-	else {
-		QString errorStr;
-		int errorLine;
-		int errorColumn;
-		QDomDocument domDocument;
-
-		QFile file(filename);
-		if (!domDocument.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
-			return;
-		}
-
-		QDomElement root = domDocument.documentElement();
-   		if (root.isNull()) {
-   			return;
-		}
-
-		if (root.tagName() != "module") {
-			return;
-		}
-
-		QDomElement instances = root.firstChildElement("instances");
-		if (instances.isNull()) return;
-
-		QDomElement instance = domDocument.createElement("instance");
-		instances.appendChild(instance);
-		instance.setAttribute("moduleIdRef", moduleID);
-		instance.setAttribute("path", "");
-		QDomElement views = domDocument.createElement("views");
-		instance.appendChild(views);
-		QDomElement iconView = domDocument.createElement("iconView");
-		views.appendChild(iconView);
-		iconView.setAttribute("layer", "icon");
-		QDomElement geometry = domDocument.createElement("geometry");
-		iconView.appendChild(geometry);
-		geometry.setAttribute("x", "-1");
-		geometry.setAttribute("y", "-1");
-		geometry.setAttribute("z", "-1");
-		writeCommonBinAux(domDocument.toString(), QFile::WriteOnly, doIt, filename);
-	}
-}
-
-void PaletteModel::writeCommonBinAux(const QString &textToWrite, QIODevice::OpenMode openMode, bool doIt, const QString &filename) {
-	if(!doIt) return;
-
-	if (FirstTimeWrite) {
-		FirstTimeWrite = false;
-		QFileInfo info(filename);
-		QDir dir = info.absoluteDir();
-		dir.mkpath(info.absolutePath());
-	}
-
-	QFile file(filename);
-	if (file.open(openMode | QFile::Text)) {
-		QTextStream out(&file);
-		out.setCodec("UTF-8");
-		out << textToWrite;
-		file.close();
-	}
-}
-
 void PaletteModel::countParts(QDir & dir, QStringList & nameFilters, int & partCount) {
     QStringList list = dir.entryList(nameFilters, QDir::Files | QDir::NoSymLinks);
 	partCount += list.size();
@@ -314,7 +206,7 @@ void PaletteModel::countParts(QDir & dir, QStringList & nameFilters, int & partC
 }
 
 void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loadingPart, int totalPartCount) {
-    QString temp = dir.absolutePath();
+    //QString temp = dir.absolutePath();
     QFileInfoList list = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
     for (int i = 0; i < list.size(); ++i) {
         QFileInfo fileInfo = list.at(i);
@@ -330,7 +222,7 @@ void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loa
     	QString temp2 = dirs[i];
        	dir.cd(temp2);
 
-		m_loadingContrib = temp2=="contrib";
+		m_loadingContrib = (temp2 == "contrib");
 
     	loadPartsAux(dir, nameFilters, loadingPart, totalPartCount);
     	dir.cdUp();
@@ -393,6 +285,8 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
 	QDomElement t = root.firstChildElement("title");
 	TextUtils::findText(t, title);
 
+    //DebugDialog::debug("module ID " + moduleID);
+
 	// FIXME: properties is nested right now
 	if (moduleID.compare(ModuleIDNames::WireModuleIDName) == 0) {
 		type = ModelPart::Wire;
@@ -412,10 +306,10 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
 	else if (moduleID.compare(ModuleIDNames::NoteModuleIDName) == 0) {
 		type = ModelPart::Note;
 	}
-	else if (moduleID.compare(ModuleIDNames::JustPowerModuleIDName) == 0) {
-		type = ModelPart::Symbol;
+	else if (moduleID.endsWith(ModuleIDNames::TwoPowerModuleIDName)) {
+		type = ModelPart::Part;
 	}
-	else if (moduleID.compare(ModuleIDNames::PowerModuleIDName) == 0) {
+	else if (moduleID.endsWith(ModuleIDNames::PowerModuleIDName)) {
 		type = ModelPart::Symbol;
 	}
 	else if (moduleID.compare(ModuleIDNames::GroundModuleIDName) == 0) {
@@ -424,6 +318,9 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
 	else if (moduleID.endsWith(ModuleIDNames::NetLabelModuleIDName)) {
 		type = ModelPart::Symbol;
 	}
+	else if (moduleID.compare(ModuleIDNames::PowerLabelModuleIDName) == 0) {
+		type = ModelPart::Symbol;
+	}
 	else if (moduleID.compare(ModuleIDNames::RulerModuleIDName) == 0) {
 		type = ModelPart::Ruler;
 	}
@@ -439,6 +336,9 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
 	else if (moduleID.endsWith(ModuleIDNames::StripboardModuleIDName)) {
 		type = ModelPart::Breadboard;
 	}
+	else if (moduleID.endsWith(ModuleIDNames::Stripboard2ModuleIDName)) {
+		type = ModelPart::Breadboard;
+	}
 	else if (propertiesText.contains("breadboard", Qt::CaseInsensitive)) {
 		type = ModelPart::Breadboard;
 	}
@@ -464,6 +364,14 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
 
 	modelPart->setContrib(m_loadingContrib);
 
+    QDomElement subparts = root.firstChildElement("schematic-subparts");
+    QDomElement subpart = subparts.firstChildElement("subpart");
+    while (!subpart.isNull()) {
+        ModelPart * subModelPart = makeSubpart(modelPart, subpart);
+        m_partHash.insert(subModelPart->moduleID(), subModelPart);
+        subpart = subpart.nextSiblingElement("subpart");
+    }
+
 	if (m_partHash.value(moduleID, NULL)) {
 		if(!update) {
 			QMessageBox::warning(NULL, QObject::tr("Fritzing"),
@@ -486,29 +394,9 @@ ModelPart * PaletteModel::loadPart(const QString & path, bool update) {
     	modelPart->setParent(m_root);
    	}
 
-    if (!m_fullLoad) {
-	    if (FirstTime) {
-		    // make sure saving to the common bin takes place after fastLoad
-		    //DebugDialog::debug(QString("all parts %1").arg(JustAppendAllPartsInstances));
-		    writeCommonBinInstance(moduleID,path,CreateAllPartsBinFile,AllPartsBinFilePath);
-
-		    if (modelPart->isContrib()) {
-			    //if (path.startsWith(FritzingContribPath, Qt::CaseInsensitive)) {
-				    writeCommonBinInstance(moduleID,path,CreateContribPartsBinFile,ContribPartsBinFilePath);
-			    //}
-		    }
-
-		    //DebugDialog::debug(QString("non core parts %1").arg(JustAppendAllPartsInstances));
-		    if (!modelPart->isCore() && !modelPart->isObsolete()) {
-			    writeCommonBinInstance(moduleID,path,CreateNonCorePartsBinFile,NonCorePartsBinFilePath);
-		    }
-	    }
-    }
-
     return modelPart;
 }
 
-
 bool PaletteModel::loadFromFile(const QString & fileName, ModelBase * referenceModel, bool checkViews) {
 	QList<ModelPart *> modelParts;
 	bool result = ModelBase::loadFromFile(fileName, referenceModel, modelParts, checkViews);
@@ -607,9 +495,7 @@ void PaletteModel::setOrdererChildren(QList<QObject*> children) {
 QList<ModelPart *> PaletteModel::search(const QString & searchText, bool allowObsolete) {
 	QList<ModelPart *> modelParts;
 
-	QStringList strings = searchText.split(":");
-	if (strings.count() > 2) return modelParts;
-
+	QStringList strings = searchText.split(" ");
     search(m_root, strings, modelParts, allowObsolete);
     return modelParts;
 }
@@ -619,51 +505,59 @@ void PaletteModel::search(ModelPart * modelPart, const QStringList & searchStrin
 	// or use lucene
 	// or google search api
 
+    int count = 0;
+    foreach (QString searchString, searchStrings) {
+        bool gotOne = false;
+        if (modelPart->title().contains(searchString, Qt::CaseInsensitive)) {
+            gotOne = true;
+	    }
+        else if (modelPart->description().contains(searchString, Qt::CaseInsensitive)) {
+            gotOne = true;
+	    }
+        else if (modelPart->url().contains(searchString, Qt::CaseInsensitive)) {
+            gotOne = true;
+	    }
+        else if (modelPart->author().contains(searchString, Qt::CaseInsensitive)) {
+            gotOne = true;
+	    }
+        else if (modelPart->moduleID().contains(searchString, Qt::CaseInsensitive)) {
+            gotOne = true;
+	    }
+        else {
+		    foreach (QString string, modelPart->tags()) {
+			    if (string.contains(searchString, Qt::CaseInsensitive)) {
+                    gotOne = true;
+				    break;
+			    }
+		    }
+	    }
+        if (!gotOne) {
+		    foreach (QString string, modelPart->properties().values()) {
+			    if (string.contains(searchString, Qt::CaseInsensitive)) {
+                    gotOne = true;
+				    break;
+			    }
+		    }
+	    }
+        if (!gotOne) {
+		    foreach (QString string, modelPart->properties().keys()) {
+			    if (string.contains(searchString, Qt::CaseInsensitive)) {
+                    gotOne = true;
+				    break;
+			    }
+		    }
+	    }
+        if (!gotOne) break;
 
-    ModelPart * candidate = NULL;
-    if (!candidate && modelPart->title().contains(searchStrings[0], Qt::CaseInsensitive)) {
-        candidate = modelPart;
-	}
-    if (!candidate && modelPart->description().contains(searchStrings[0], Qt::CaseInsensitive)) {
-        candidate = modelPart;
-	}
-    if (!candidate && modelPart->url().contains(searchStrings[0], Qt::CaseInsensitive)) {
-        candidate = modelPart;
-	}
-    if (!candidate && modelPart->author().contains(searchStrings[0], Qt::CaseInsensitive)) {
-        candidate = modelPart;
-	}
-    if (!candidate) {
-		foreach (QString string, modelPart->tags()) {
-			if (string.contains(searchStrings[0], Qt::CaseInsensitive)) {
-                candidate = modelPart;
-				break;
-			}
-		}
-	}
-    if (!candidate) {
-		foreach (QString string, modelPart->properties().values()) {
-			if (string.contains(searchStrings[0], Qt::CaseInsensitive)) {
-                candidate = modelPart;
-				break;
-			}
-		}
-	}
-    if (!candidate) {
-		foreach (QString string, modelPart->properties().keys()) {
-			if (string.contains(searchStrings[0], Qt::CaseInsensitive)) {
-                candidate = modelPart;
-				break;
-			}
-		}
-	}
+        count++;
+    }
 
-    if (candidate && !modelParts.contains(candidate)) {
-        if (!allowObsolete && candidate->isObsolete()) {
+    if ((count == searchStrings.count()) && !modelParts.contains(modelPart)) {
+        if (!allowObsolete && modelPart->isObsolete()) {
         }
         else
         {
-            modelParts.append(candidate);
+            modelParts.append(modelPart);
             emit addSearchMaximum(1);
         }
     }
@@ -679,3 +573,107 @@ void PaletteModel::search(ModelPart * modelPart, const QStringList & searchStrin
 	}
 }
 
+QList<ModelPart *> PaletteModel::findContribNoBin() {
+    QList<ModelPart *> modelParts;
+    foreach (ModelPart * modelPart, m_partHash.values()) {
+        if (modelPart->isContrib()) {
+            if (!modelPart->isInBin()) {
+                modelParts << modelPart;
+            }
+        }
+    }
+    return modelParts;
+}
+
+ModelPart * PaletteModel::makeSubpart(ModelPart * originalModelPart, const QDomElement & originalSubpart) {
+    QString newLabel = originalSubpart.attribute("label");
+    QString newID = originalSubpart.attribute("id");
+    QDomElement originalRoot = originalSubpart.ownerDocument().documentElement();
+    QString moduleID = originalRoot.attribute("moduleId") + "_" + newID;
+    ModelPart * modelPart = retrieveModelPart(moduleID);
+    if (modelPart) {
+        return modelPart;
+    }
+
+    QDomDocument subdoc = originalSubpart.ownerDocument().cloneNode(true).toDocument();
+    QDomElement root = subdoc.documentElement();
+    QDomElement subparts = root.firstChildElement("schematic-subparts");
+    root.removeChild(subparts);
+    root.setAttribute("moduleId", moduleID);
+    QDomElement label = root.firstChildElement("label");
+    if (!label.isNull()) {
+        TextUtils::replaceChildText(label, newLabel);
+    }
+    QDomElement views = root.firstChildElement("views");
+    QDomElement schematicView = views.firstChildElement(ViewLayer::viewIDXmlName(ViewLayer::SchematicView));
+    QDomElement schematicLayers = schematicView.firstChildElement("layers");
+    QString image = schematicLayers.attribute("image");
+    image.replace(".svg", "_" + newID + ".svg");
+    schematicLayers.setAttribute("image", image);
+    views.removeChild(schematicView);
+    QDomElement view = views.firstChildElement();
+    while (!view.isNull()) {
+        QDomElement layers = view.firstChildElement("layers");
+        view.removeChild(layers);
+        layers = schematicLayers.cloneNode(true).toElement();
+        view.appendChild(layers);
+        view = view.nextSiblingElement();
+    }
+    views.appendChild(schematicView);
+
+    QDomElement connectors = root.firstChildElement("connectors");
+    QDomElement connector = connectors.firstChildElement("connector");
+    QHash<QString, QDomElement> connectorHash;
+    while (!connector.isNull()) {
+        QDomElement next = connector.nextSiblingElement("connector");
+        connectors.removeChild(connector);
+        connectorHash.insert(connector.attribute("id"), connector);
+        connector = next;
+    }
+    QDomElement originalConnectors = originalSubpart.firstChildElement("connectors");
+    QDomElement originalConnector = originalConnectors.firstChildElement("connector");
+    QString schematicLayerName = ViewLayer::viewLayerXmlNameFromID(ViewLayer::Schematic);
+    while (!originalConnector.isNull()) {
+        QString id = originalConnector.attribute("id");
+        QDomElement connector = connectorHash.value(id);
+        connectors.appendChild(connector);
+        QDomElement cviews = connector.firstChildElement("views");
+        QDomElement view = cviews.firstChildElement();
+        while (!view.isNull()) {
+            QDomElement p = view.firstChildElement("p");
+            bool firstTime = true;
+            while (!p.isNull()) {
+                QDomElement next = p.nextSiblingElement("p");
+                if (firstTime) {
+                    p.setAttribute("layer", schematicLayerName);
+                    firstTime = false;
+                }
+                else {
+                    view.removeChild(p);
+                }
+                p = next;
+            }
+            view = view.nextSiblingElement();
+        }
+
+        originalConnector = originalConnector.nextSiblingElement("connector");
+    }
+
+    QString path = PartFactory::fzpPath() + moduleID + ".fzp";
+    QString fzp = subdoc.toString(4);
+    TextUtils::writeUtf8(path, fzp);
+
+    modelPart = new ModelPart(subdoc, path, ModelPart::SchematicSubpart);
+    modelPart->setSubpartID(newID);
+    originalModelPart->modelPartShared()->addSubpart(modelPart->modelPartShared());
+    m_partHash.insert(moduleID, modelPart);
+    return modelPart;
+}
+
+QList<ModelPart *> PaletteModel::allParts() {
+    QList<ModelPart *> modelParts;
+    foreach (ModelPart * modelPart, m_partHash.values()) {
+        if (!modelPart->isObsolete()) modelParts.append(modelPart);
+    }
+    return modelParts;
+}
