@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6569 $:
+$Revision: 6962 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-16 05:51:14 +0200 (Tue, 16 Oct 2012) $
+$Date: 2013-04-14 00:08:36 +0200 (So, 14. Apr 2013) $
 
 ********************************************************************/
 
@@ -44,6 +44,7 @@ $Date: 2012-10-16 05:51:14 +0200 (Tue, 16 Oct 2012) $
 #include "../../waitpushundostack.h"
 #include "../../debugdialog.h"
 #include "../../utils/folderutils.h"
+#include "../../utils/textutils.h"
 #include "../../utils/fileprogressdialog.h"
 #include "../../referencemodel/referencemodel.h"
 #include "../partsbinpalettewidget.h"
@@ -95,7 +96,6 @@ QString BinManager::MyPartsBinTemplateLocation;
 QString BinManager::SearchBinLocation;
 QString BinManager::SearchBinTemplateLocation;
 QString BinManager::ContribPartsBinLocation;
-QString BinManager::ContribPartsBinTemplateLocation;
 QString BinManager::TempPartsBinTemplateLocation;
 QString BinManager::CorePartsBinLocation;
 
@@ -145,6 +145,9 @@ void BinManager::initStandardBins()
 	//DebugDialog::debug("init bin manager");
 	QList<BinLocation *> actualLocations;
 	findAllBins(actualLocations);
+
+    hackLocalContrib(actualLocations);
+
 	restoreStateAndGeometry(actualLocations);
 	foreach (BinLocation * location, actualLocations) {
 		PartsBinPaletteWidget* bin = newBin();
@@ -154,6 +157,8 @@ void BinManager::initStandardBins()
 		registerBin(bin);
 		delete location;
 	}
+    actualLocations.clear();
+
 	//DebugDialog::debug("open core bin");
 	openCoreBinIn();
 		
@@ -315,14 +320,6 @@ void BinManager::addToMyParts(ModelPart *modelPart) {
 	}
 }
 
-void BinManager::addToContrib(ModelPart *modelPart) {
-	PartsBinPaletteWidget *bin = getOrOpenBin(ContribPartsBinLocation, ContribPartsBinTemplateLocation);
-	if (bin) {
-		addPartToBinAux(bin,modelPart);
-		setAsCurrentTab(bin);
-	}
-}
-
 void BinManager::addToTempPartsBin(ModelPart *modelPart) {
 	PartsBinPaletteWidget *bin = getOrOpenBin(m_tempPartsBinLocation, TempPartsBinTemplateLocation);
 	if (bin) {
@@ -491,6 +488,12 @@ void BinManager::setAsCurrentBin(PartsBinPaletteWidget* bin) {
 		bin->focusSearch();
 	}
 
+    /*
+
+    // jrc 3-july-2013 commented out this stylesheet change:
+    //      it causes the tab bar to lose its scroll position
+    //      the stylesheet change is commented out in the qss file, so visually it's a no-op
+
 	QString style = m_mainWindow->styleSheet();
 	if(m_currentBin && m_stackTabWidget) {
 		StackTabBar *currTabBar = m_stackTabWidget->stackTabBar();
@@ -505,6 +508,7 @@ void BinManager::setAsCurrentBin(PartsBinPaletteWidget* bin) {
 		currTabBar->setStyleSheet("");
 		currTabBar->setStyleSheet(style);
 	}
+    */
 }
 
 void BinManager::closeBinIn(int index) {
@@ -588,8 +592,8 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 	actualLocations.clear();
 
 	foreach (BinLocation * tLocation, theoreticalLocations) {
-                tLocation->marked = false;
-                bool gotOne = false;
+        tLocation->marked = false;
+        bool gotOne = false;
 
 		for (int ix = 0; ix < tempLocations.count(); ix++) {
 			BinLocation  * aLocation = tempLocations[ix];
@@ -597,6 +601,7 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 				gotOne = true;
 				actualLocations.append(aLocation);
 				tempLocations.removeAt(ix);
+                //DebugDialog::debug("adding loc 1 " + aLocation->path);
 				break;
 			}
 		}
@@ -609,7 +614,8 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 					gotOne = true;
 					actualLocations.append(aLocation);
 					tempLocations.removeAt(ix);
-					break;
+                    //DebugDialog::debug("adding loc 2 " + aLocation->path);
+                    break;
 				}
 			}
 		}
@@ -618,8 +624,9 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 		if (!tLocation->path.isEmpty()) {
 			QFileInfo info(tLocation->path);
 			if (info.exists()) {
+                //DebugDialog::debug("adding loc 3 " + tLocation->path);
 				actualLocations.append(tLocation);
-                                tLocation->marked = true;
+                tLocation->marked = true;
 			}		
 		}
 	}
@@ -632,6 +639,10 @@ void BinManager::restoreStateAndGeometry(QList<BinLocation *> & actualLocations)
 
 	// catch the leftovers
 	actualLocations.append(tempLocations);
+    //foreach (BinLocation * aLocation, tempLocations) {
+        //DebugDialog::debug("adding loc 4 " + aLocation->path);
+    //}
+
 
 }
 
@@ -658,6 +669,93 @@ void BinManager::readTheoreticalLocations(QList<BinLocation *> & theoreticalLoca
 	}
 }
 
+void BinManager::hackLocalContrib(QList<BinLocation *> & locations) 
+{
+    // with release 0.7.12, there is no more local contrib bin
+    // so clear out existing local contrib bins by copying parts to mine bin
+
+    BinLocation * localContrib = NULL;
+    BinLocation * myParts = NULL;
+    foreach (BinLocation * location, locations) {
+        if (location->location == BinLocation::User) {
+            if (location->title == "Contributed Parts") {
+                localContrib = location;
+            }
+            else if (location->title == "My Parts") {
+                myParts = location;
+            }
+        }
+    }
+
+    if (localContrib == NULL) return;
+
+    if (myParts == NULL) {
+        createIfBinNotExists(MyPartsBinLocation, MyPartsBinTemplateLocation);
+	    myParts = new BinLocation;
+	    myParts->location = BinLocation::User;
+	    myParts->path = MyPartsBinLocation;
+	    QString icon;
+	    getBinTitle(myParts->path, myParts->title, icon);
+	    locations.append(myParts);
+    }
+
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+
+    QFile contribFile(localContrib->path);
+    QDomDocument contribDoc;
+    bool result = contribDoc.setContent(&contribFile, true, &errorStr, &errorLine, &errorColumn);
+    locations.removeOne(localContrib);
+    contribFile.close();
+    bool removed = contribFile.remove();
+    if (!removed) {
+        DebugDialog::debug("failed to remove contrib bin");
+    }
+
+    if (!result) return;
+
+    QFile myPartsFile(myParts->path);
+    QDomDocument myPartsDoc;
+    if (!myPartsDoc.setContent(&myPartsFile, true, &errorStr, &errorLine, &errorColumn)) {
+        return;
+    }
+
+    QDomElement myPartsRoot = myPartsDoc.documentElement();
+    QDomElement myPartsInstances = myPartsRoot.firstChildElement("instances");
+
+    bool changed = false;
+
+    QDomElement contribRoot = contribDoc.documentElement();
+    QDomElement contribInstances = contribRoot.firstChildElement("instances");
+    QDomElement contribInstance = contribInstances.firstChildElement("instance");
+    while (!contribInstance.isNull()) {
+        QString moduleIDRef = contribInstance.attribute("moduleIdRef");
+        QDomElement myPartsInstance = myPartsInstances.firstChildElement("instance");
+        bool already = false;
+        while (!myPartsInstance.isNull()) {
+            QString midr = myPartsInstance.attribute("moduleIdRef");
+            if (midr == moduleIDRef) {
+                already = true;
+                break;
+            }
+            myPartsInstance = myPartsInstance.nextSiblingElement("instance");
+        }
+
+        if (!already) {
+            QDomNode node = contribInstance.cloneNode(true);
+            myPartsInstances.appendChild(node);
+            changed = true;
+        }
+
+        contribInstance = contribInstance.nextSiblingElement("instance");
+    }
+
+    if (changed) {
+        TextUtils::writeUtf8(myParts->path, myPartsDoc.toString(0));
+    }
+}
+
 void BinManager::findAllBins(QList<BinLocation *> & locations) 
 {
 	BinLocation * location = new BinLocation;
@@ -667,6 +765,12 @@ void BinManager::findAllBins(QList<BinLocation *> & locations)
 	getBinTitle(location->path, location->title, icon);
 	locations.append(location);
 
+	location = new BinLocation;
+	location->location = BinLocation::App;
+	location->path = ContribPartsBinLocation;
+	getBinTitle(location->path, location->title, icon);
+	locations.append(location);
+
 	QDir userBinsDir(FolderUtils::getUserDataStorePath("bins"));
 	findBins(userBinsDir, locations, BinLocation::User);
 
@@ -773,7 +877,7 @@ void BinManager::initNames() {
     BinManager::MyPartsBinTemplateLocation =":/resources/bins/my_parts.fzb";
     BinManager::SearchBinLocation = FolderUtils::getUserDataStorePath("bins")+"/search.fzb";
     BinManager::SearchBinTemplateLocation =":/resources/bins/search.fzb";
-	BinManager::ContribPartsBinLocation = FolderUtils::getUserDataStorePath("bins")+"/contribParts.fzb";
+	BinManager::ContribPartsBinLocation = FolderUtils::getApplicationSubFolderPath("bins")+"/contribParts.fzb";
     BinManager::CorePartsBinLocation = FolderUtils::getApplicationSubFolderPath("bins")+"/core.fzb";
     BinManager::TempPartsBinTemplateLocation =":/resources/bins/temp.fzb";
 
@@ -798,7 +902,9 @@ void BinManager::search(const QString & searchText) {
     searchBin->removeParts();
     foreach (ModelPart * modelPart, modelParts) {
         //DebugDialog::debug(modelPart->title());
-        this->addPartTo(searchBin, modelPart, false);
+        if (modelPart->itemType() != ModelPart::SchematicSubpart) {
+            this->addPartTo(searchBin, modelPart, false);
+        }
         progress.incValue();
     }
  
@@ -843,8 +949,14 @@ void BinManager::updateBinCombinedMenu(PartsBinPaletteWidget * bin) {
 	ItemBase *itemBase = bin->selectedItemBase();
 	bool enabled = (itemBase != NULL);
 	m_editPartNewAction->setEnabled(enabled && itemBase->canEditPart());
-	m_exportPartAction->setEnabled(enabled && !itemBase->modelPart()->isCore());
+
+    bool enableAnyway = false;
+#ifndef QT_NO_DEBUG
+    enableAnyway = true;
+#endif
+	m_exportPartAction->setEnabled(enabled && (!itemBase->modelPart()->isCore() || enableAnyway));
 	m_removePartAction->setEnabled(enabled && bin->allowsChanges());
+	m_findPartAction->setEnabled(enabled);
 }
 
 void BinManager::createCombinedMenu() 
@@ -883,6 +995,14 @@ void BinManager::createCombinedMenu()
 	m_renameBinAction->setStatusTip(tr("Rename parts bin..."));
 	connect(m_renameBinAction, SIGNAL(triggered()),this, SLOT(renameBin()));
 
+	m_copyToSketchAction = new QAction(tr("Copy to Sketch"), this);
+	m_copyToSketchAction->setStatusTip(tr("Copy all the parts in the bin to a sketch"));
+	connect(m_copyToSketchAction, SIGNAL(triggered()),this, SLOT(copyToSketch()));
+
+	m_copyAllToSketchAction = new QAction(tr("Copy all to Sketch"), this);
+	m_copyAllToSketchAction->setStatusTip(tr("Copy all loaded parts to the sketch"));
+	connect(m_copyAllToSketchAction, SIGNAL(triggered()),this, SLOT(copyAllToSketch()));
+
 	m_showListViewAction = new QAction(tr("Show Bin in List View"), this);
 	m_showListViewAction->setCheckable(true);
 	m_showListViewAction->setStatusTip(tr("Display parts as a list"));
@@ -903,6 +1023,10 @@ void BinManager::createCombinedMenu()
 	m_combinedMenu->addAction(m_saveBinAsAction);
 	m_combinedMenu->addAction(m_saveBinAsBundledAction);
 	m_combinedMenu->addAction(m_renameBinAction);
+#ifndef QT_NO_DEBUG
+	m_combinedMenu->addAction(m_copyToSketchAction);
+	m_combinedMenu->addAction(m_copyAllToSketchAction);
+#endif
 	m_combinedMenu->addSeparator();
 	m_combinedMenu->addAction(m_showIconViewAction);
 	m_combinedMenu->addAction(m_showListViewAction);
@@ -910,10 +1034,12 @@ void BinManager::createCombinedMenu()
 	m_editPartNewAction = new QAction(tr("Edit Part (new parts editor)..."),this);
 	m_exportPartAction = new QAction(tr("Export Part..."),this);
 	m_removePartAction = new QAction(tr("Remove Part"),this);
+	m_findPartAction = new QAction(tr("Find Part in Sketch"),this);
 
 	connect(m_editPartNewAction, SIGNAL(triggered()),this, SLOT(editSelectedNew()));
 	connect(m_exportPartAction, SIGNAL(triggered()),this, SLOT(exportSelected()));
 	connect(m_removePartAction, SIGNAL(triggered()),this, SLOT(removeSelected()));
+	connect(m_findPartAction, SIGNAL(triggered()),this, SLOT(findSelected()));
 
 	connect(m_combinedMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinCombinedMenuCurrent()));
 
@@ -921,6 +1047,7 @@ void BinManager::createCombinedMenu()
 	m_combinedMenu->addAction(m_editPartNewAction);
 	m_combinedMenu->addAction(m_exportPartAction);
 	m_combinedMenu->addAction(m_removePartAction);
+	m_combinedMenu->addAction(m_findPartAction);
 
 }
 
@@ -937,6 +1064,7 @@ void BinManager::createContextMenus() {
 	m_partContextMenu->addAction(m_editPartNewAction);
 	m_partContextMenu->addAction(m_exportPartAction);
 	m_partContextMenu->addAction(m_removePartAction);
+	m_partContextMenu->addAction(m_findPartAction);
 }
 
 void BinManager::closeBin() {
@@ -962,12 +1090,13 @@ void BinManager::deleteBin() {
 	QFile::remove(filename);
 }
 
-void BinManager::importPartToContribBin(const QString & filename) {
+void BinManager::importPartToMineBin(const QString & filename) {
 
 	if (!filename.isEmpty() && !filename.isNull()) {
-        PartsBinPaletteWidget * bin = getOrOpenBin(ContribPartsBinLocation, ContribPartsBinTemplateLocation);
+        PartsBinPaletteWidget * bin = getOrOpenBin(MyPartsBinLocation, MyPartsBinTemplateLocation);
         if (bin == NULL) return;
 
+        setAsCurrentTab(bin);
         importPart(filename, bin);
     }
 }
@@ -1076,24 +1205,40 @@ bool BinManager::removeSelected() {
 	ModelPart * mp = bin->selectedModelPart();
 	if (mp == NULL) return false;
 
+    if (m_mainWindow->anyUsePart(mp->moduleID())) {
+        QMessageBox::warning(this, tr("Remove from Bin"), tr("Unable to remove part '%1'--it is in use in a sketch").arg(mp->title()));
+        return false;
+    }
+
 	QMessageBox::StandardButton answer = QMessageBox::question(
 		this,
 		tr("Remove from bin"),
-		tr("Do you really want to remove '%1' from the bin?").arg(mp->title()),
+		tr("Do you really want to remove '%1' from the bin? This operation cannot be undone.").arg(mp->title()),
 		QMessageBox::Yes | QMessageBox::No,
-		QMessageBox::Yes
+		QMessageBox::No
 	);
 	// TODO: make button texts translatable
 	if(answer != QMessageBox::Yes) return false;
 
 
 	m_undoStack->push(new QUndoCommand("Parts bin: part removed"));
-	bin->removePart(mp->moduleID());
+	bin->removePart(mp->moduleID(), mp->path());
 	bin->setDirty();
 
 	return true;
 }
 
+void BinManager::findSelected() {
+	PartsBinPaletteWidget * bin = currentBin();
+	if (bin == NULL) return;
+
+	ModelPart * mp = bin->selectedModelPart();
+	if (mp == NULL) return;
+
+    m_mainWindow->selectPartsWithModuleID(mp);
+}
+
+
 void BinManager::exportSelected() {
 	PartsBinPaletteWidget * bin = currentBin();
 	if (bin == NULL) return;
@@ -1140,10 +1285,10 @@ void BinManager::copyFilesToContrib(ModelPart * mp, QWidget * originator) {
 	dir.cd("svg");
 	dir.cd("contrib");
 
-	QList<ViewLayer::ViewIdentifier> viewIdentifiers;
-	viewIdentifiers << ViewLayer::IconView << ViewLayer::BreadboardView << ViewLayer::SchematicView << ViewLayer::PCBView;
-	foreach (ViewLayer::ViewIdentifier viewIdentifier, viewIdentifiers) {
-		QString fn = mp->hasBaseNameFor(viewIdentifier);
+	QList<ViewLayer::ViewID> viewIDs;
+	viewIDs << ViewLayer::IconView << ViewLayer::BreadboardView << ViewLayer::SchematicView << ViewLayer::PCBView;
+	foreach (ViewLayer::ViewID viewID, viewIDs) {
+		QString fn = mp->hasBaseNameFor(viewID);
 		if (!fn.isEmpty()) {
 			QFile svg(dir.absoluteFilePath(fn));
 			svg.copy(prefix + fn);
@@ -1199,3 +1344,24 @@ void BinManager::reloadPart(const QString & moduleID) {
 		bin->reloadPart(moduleID);
 	}
 }
+
+void BinManager::copyToSketch() {
+	PartsBinPaletteWidget * bin = currentBin();
+	if (bin == NULL) return;
+
+    QList<ModelPart *> modelParts = bin->getAllParts();
+    if (modelParts.count() == 0) return;
+
+    if (m_mainWindow) {
+        m_mainWindow->addToSketch(modelParts);
+    }
+}
+
+void BinManager::copyAllToSketch() {
+    QList<ModelPart *> modelParts;
+
+    if (m_mainWindow) {
+        m_mainWindow->addToSketch(modelParts);
+    }
+}
+
