@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6569 $:
+$Revision: 6904 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-16 05:51:14 +0200 (Tue, 16 Oct 2012) $
+$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
 
 ********************************************************************/
 
@@ -260,6 +260,14 @@ bool PartsBinPaletteWidget::saveAsAux(const QString &filename) {
 	}
 	emit saved(hasAlienParts());
 
+    foreach (QString path, m_removed) {
+        bool result = QFile::remove(path);
+        if (!result) {
+            DebugDialog::debug("unable to delete '" + path + "' from bin");
+        }
+    }
+    m_removed.clear();
+
 	return true;
 }
 
@@ -325,11 +333,20 @@ void PartsBinPaletteWidget::grabTitle(const QString & title, QString & iconFilen
 		m_icon = new QIcon(QPixmap::fromImage(image));
 	}
 	else {
-		QString path = ":resources/bins/icons/" + iconFilename;
-		QFile file(path);
-		if (file.exists()) {
-			m_icon = new QIcon(path);
-		}
+        QFileInfo info(m_fileName);
+        QDir dir = info.absoluteDir();
+        QString path = dir.absoluteFilePath(iconFilename);
+        QFile file1(path);
+        if (file1.exists()) {
+            m_icon = new QIcon(path);
+        }
+        else {
+            path = ":resources/bins/icons/" + iconFilename;
+		    QFile file2(path);
+		    if (file2.exists()) {
+			    m_icon = new QIcon(path);
+		    }
+        }
 	}
 }
 
@@ -444,7 +461,7 @@ bool PartsBinPaletteWidget::open(QString fileName, QWidget * progressTarget, boo
     	load(fileName, progressTarget, fastLoad);
     	m_isDirty = false;
     } else if(fileName.endsWith(FritzingBundledBinExtension)) {
-    	m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false, false);
+    	return m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false, false);
     }
 
     return true;
@@ -475,7 +492,7 @@ void PartsBinPaletteWidget::load(const QString &filename, QWidget * progressTarg
 	    //DebugDialog::debug("after palette model");
 
 	    QString name = m_title;
-	    if (name.isEmpty()) name = QFileInfo(filename).baseName();
+	    if (name.isEmpty()) name = QFileInfo(filename).completeBaseName();
 
 	    bool deleteWhenDone = false;
         if (progressTarget != NULL) {
@@ -609,14 +626,16 @@ void PartsBinPaletteWidget::addPart(const QString& moduleID, int position) {
 	addPart(modelPart, position);
 }
 
-void PartsBinPaletteWidget::removePart(const QString& moduleID) {
+void PartsBinPaletteWidget::removePart(const QString & moduleID, const QString & path) {
 	m_iconView->removePart(moduleID);
 	m_listView->removePart(moduleID);
 
 	// remove the model part from the model last, as this deletes it,
 	// and the removePart calls above still need the modelpart
 	m_model->removePart(moduleID);
-
+    if (path.contains(FolderUtils::getUserDataStorePath())) {
+        m_removed << path;
+    }
 }
 
 
@@ -631,7 +650,7 @@ void PartsBinPaletteWidget::removeParts() {
 
 void PartsBinPaletteWidget::removeAlienParts() {
 	foreach(QString moduleID, m_alienParts) {
-		removePart(moduleID);
+		removePart(moduleID, "");
 	}
 	m_alienParts.clear();
 }
@@ -730,6 +749,8 @@ QAction *PartsBinPaletteWidget::addPartToMeAction() {
 void PartsBinPaletteWidget::addSketchPartToMe() {
     m_manager->openBinIn(this->m_fileName, false);
 	QString moduleID = m_manager->getSelectedModuleIDFromSketch();
+    if (moduleID.isEmpty()) return;
+
 	bool wasAlreadyIn = contains(moduleID);
 	addPart(moduleID, -1);
 	if(!wasAlreadyIn) {
@@ -832,7 +853,7 @@ QMenu * PartsBinPaletteWidget::binContextMenu()
 
 	// TODO: need to enable/disable actions based on this bin
 
-	ModelPartSharedRoot * root = m_model->rootModelPartShared();
+	ModelPartSharedRoot * root = (m_model == NULL) ? NULL : m_model->rootModelPartShared();
 	if (root) {
 		QString iconFilename = root->icon();
 		if (iconFilename.compare(CustomIconName) == 0 || isCustomSvg(iconFilename)) {
@@ -862,8 +883,14 @@ void PartsBinPaletteWidget::changeIconColor() {
 		}
 	}
 
+#ifndef QT_NO_DEBUG
+    //QFileInfo info(m_fileName);
+    //image.save(FolderUtils::getUserDataStorePath("") + "/" + info.completeBaseName() + ".png");
+#endif
+
 	delete m_icon;
 	m_icon = new QIcon(QPixmap::fromImage(image));
+
 	m_manager->setTabIcon(this, m_icon);
 	setDirty();
 
@@ -930,3 +957,14 @@ void PartsBinPaletteWidget::reloadPart(const QString & moduleID) {
 	m_iconView->reloadPart(moduleID);
 	m_listView->reloadPart(moduleID);
 }
+
+QList<ModelPart *> PartsBinPaletteWidget::getAllParts() {
+    QList<ModelPart*> empty;
+    if (m_model == NULL) return empty;
+
+    if (m_model->root() == NULL) return empty;
+
+    return m_model->root()->getAllParts();
+}
+
+
