@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6546 $:
+$Revision: 6976 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-10 20:47:53 +0200 (Wed, 10 Oct 2012) $
+$Date: 2013-04-21 09:50:09 +0200 (So, 21. Apr 2013) $
 
 ********************************************************************/
 
@@ -37,6 +37,8 @@ $Date: 2012-10-10 20:47:53 +0200 (Wed, 10 Oct 2012) $
 
 #include <limits>
 
+QSizeF SchematicSketchWidget::m_jumperItemSize = QSizeF(0, 0);
+
 static QString SchematicTraceColor = "black";
 static const double TraceHoverStrokeFactor = 3;
 
@@ -50,8 +52,10 @@ bool sameGround(ConnectorItem * c1, ConnectorItem * c2)
 	return (c1Grounded == c2Grounded);
 }
 
-SchematicSketchWidget::SchematicSketchWidget(ViewLayer::ViewIdentifier viewIdentifier, QWidget *parent)
-    : PCBSketchWidget(viewIdentifier, parent)
+///////////////////////////////////////////////////
+
+SchematicSketchWidget::SchematicSketchWidget(ViewLayer::ViewID viewID, QWidget *parent)
+    : PCBSketchWidget(viewID, parent)
 {
 	m_shortName = QObject::tr("schem");
 	m_viewName = QObject::tr("Schematic View");
@@ -65,14 +69,15 @@ SchematicSketchWidget::SchematicSketchWidget(ViewLayer::ViewIdentifier viewIdent
 }
 
 void SchematicSketchWidget::addViewLayers() {
-	addSchematicViewLayers();
+	setViewLayerIDs(ViewLayer::Schematic, ViewLayer::SchematicTrace, ViewLayer::Schematic, ViewLayer::SchematicRuler, ViewLayer::SchematicNote);
+	addViewLayersAux(ViewLayer::layersForView(ViewLayer::SchematicView), ViewLayer::layersForViewFromBelow(ViewLayer::SchematicView));
 }
 
 ViewLayer::ViewLayerID SchematicSketchWidget::getDragWireViewLayerID(ConnectorItem *) {
 	return ViewLayer::SchematicTrace;
 }
 
-ViewLayer::ViewLayerID SchematicSketchWidget::getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec viewLayerSpec) {
+ViewLayer::ViewLayerID SchematicSketchWidget::getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	if (viewGeometry.getAnyTrace()) {
 		return ViewLayer::SchematicTrace;
 	}
@@ -81,7 +86,7 @@ ViewLayer::ViewLayerID SchematicSketchWidget::getWireViewLayerID(const ViewGeome
 		return ViewLayer::SchematicWire;
 	}
 
-	return SketchWidget::getWireViewLayerID(viewGeometry, viewLayerSpec);
+	return SketchWidget::getWireViewLayerID(viewGeometry, viewLayerPlacement);
 }
 
 void SchematicSketchWidget::initWire(Wire * wire, int penWidth) {
@@ -125,10 +130,6 @@ double SchematicSketchWidget::getRatsnestWidth() {
 	return 0.7;
 }
 
-bool SchematicSketchWidget::usesJumperItem() {
-	return false;
-}
-
 void SchematicSketchWidget::setClipEnds(ClipableWire * vw, bool) {
 	vw->setClipEnds(false);
 }
@@ -141,7 +142,7 @@ void SchematicSketchWidget::getBendpointWidths(Wire * wire, double width, double
 	negativeOffsetRect = true;
 }
 
-void SchematicSketchWidget::getLabelFont(QFont & font, QColor & color, ViewLayer::ViewLayerSpec) {
+void SchematicSketchWidget::getLabelFont(QFont & font, QColor & color, ItemBase *) {
 	font.setFamily("Droid Sans");
 	font.setPointSize(9);
 	font.setBold(false);
@@ -170,6 +171,7 @@ void SchematicSketchWidget::setNewPartVisible(ItemBase * itemBase) {
 				itemBase->setEverVisible(false);
 				return;
 			}
+
 			break;
 	}
 }
@@ -189,6 +191,7 @@ bool SchematicSketchWidget::canDropModelPart(ModelPart * modelPart) {
 		case ModelPart::Hole:
 			return false;
 		case ModelPart::Symbol:
+		case ModelPart::SchematicSubpart:
 			return true;
 		default:
 			break;
@@ -223,23 +226,21 @@ void SchematicSketchWidget::updateBigDots()
 		//DebugDialog::debug(QString("update big dot %1 %2").arg(traceWire->id()).arg(connectorItem->connectorSharedID()));
 
 		connectorItems.append(connectorItem);
-		connectorItem->setMarked(false);
 	}
 
+    QList<ConnectorItem *> visited;
 	foreach (ConnectorItem * connectorItem, connectorItems) {
-		if (!connectorItem->marked()) {
-			connectorItem->restoreColor(false, 0, false);
-		}
+	    connectorItem->restoreColor(visited);
 	}
 }
 
 void SchematicSketchWidget::changeConnection(long fromID, const QString & fromConnectorID,
 									long toID, const QString & toConnectorID,
-									ViewLayer::ViewLayerSpec viewLayerSpec,
+									ViewLayer::ViewLayerPlacement viewLayerPlacement,
 									bool connect, bool doEmit, bool updateConnections)
 {
 	m_updateDotsTimer.stop();
-	SketchWidget::changeConnection(fromID, fromConnectorID, toID, toConnectorID, viewLayerSpec, connect,  doEmit,  updateConnections);
+	SketchWidget::changeConnection(fromID, fromConnectorID, toID, toConnectorID, viewLayerPlacement, connect,  doEmit,  updateConnections);
 	m_updateDotsTimer.start();
 }
 
@@ -248,7 +249,7 @@ void SchematicSketchWidget::setInstanceTitle(long itemID, const QString & oldTex
 
     if (isUndoable) {
 	    SymbolPaletteItem * sitem = qobject_cast<SymbolPaletteItem *>(findItem(itemID));
-	    if (sitem && sitem->isNetLabel()) {
+	    if (sitem && sitem->isOnlyNetLabel()) {
             setProp(sitem, "label", ItemBase::TranslatedPropertyNames.value("label"), oldText, newText, true);
             return;
         }
@@ -261,7 +262,7 @@ void SchematicSketchWidget::setProp(ItemBase * itemBase, const QString & prop, c
 {
     if (prop =="label") {
         SymbolPaletteItem * sitem = qobject_cast<SymbolPaletteItem *>(itemBase);
-        if (sitem != NULL && sitem->isNetLabel()) {
+        if (sitem != NULL && sitem->isOnlyNetLabel()) {
             if (sitem->getLabel() == newValue) {
                 return;
             }
@@ -270,6 +271,7 @@ void SchematicSketchWidget::setProp(ItemBase * itemBase, const QString & prop, c
 	        parentCommand->setText(tr("Change label from %1 to %2").arg(oldValue).arg(newValue));
 
 	        new CleanUpWiresCommand(this, CleanUpWiresCommand::UndoOnly, parentCommand);
+	        new CleanUpRatsnestsCommand(this, CleanUpWiresCommand::UndoOnly, parentCommand);
 
 	        QList<Wire *> done;
 	        foreach (ConnectorItem * toConnectorItem, sitem->connector0()->connectedToItems()) {
@@ -284,6 +286,7 @@ void SchematicSketchWidget::setProp(ItemBase * itemBase, const QString & prop, c
 	        new SetPropCommand(this, itemBase->id(), "label", oldValue, newValue, true, parentCommand);
             new ChangeLabelTextCommand(this, itemBase->id(), oldValue, newValue, parentCommand);
 
+	        new CleanUpRatsnestsCommand(this, CleanUpWiresCommand::RedoOnly, parentCommand);
 	        new CleanUpWiresCommand(this, CleanUpWiresCommand::RedoOnly, parentCommand);
 
 	        m_undoStack->waitPush(parentCommand, PropChangeDelay);
@@ -313,6 +316,7 @@ void SchematicSketchWidget::setVoltage(double v, bool doEmit)
 	parentCommand->setText(tr("Change voltage from %1 to %2").arg(sitem->voltage()).arg(v));
 
 	new CleanUpWiresCommand(this, CleanUpWiresCommand::UndoOnly, parentCommand);
+	new CleanUpRatsnestsCommand(this, CleanUpWiresCommand::UndoOnly, parentCommand);
 
 	QList<Wire *> done;
 	foreach (ConnectorItem * toConnectorItem, sitem->connector0()->connectedToItems()) {
@@ -326,6 +330,7 @@ void SchematicSketchWidget::setVoltage(double v, bool doEmit)
 
 	new SetPropCommand(this, item->id(), "voltage", QString::number(sitem->voltage()), QString::number(v), true, parentCommand);
 
+	new CleanUpRatsnestsCommand(this, CleanUpWiresCommand::RedoOnly, parentCommand);
 	new CleanUpWiresCommand(this, CleanUpWiresCommand::RedoOnly, parentCommand);
 
 	m_undoStack->waitPush(parentCommand, PropChangeDelay);
@@ -335,7 +340,7 @@ double SchematicSketchWidget::defaultGridSizeInches() {
 	return GraphicsUtils::StandardSchematicSeparationMils / 1000;
 }
 
-ViewLayer::ViewLayerID SchematicSketchWidget::getLabelViewLayerID(ViewLayer::ViewLayerSpec) {
+ViewLayer::ViewLayerID SchematicSketchWidget::getLabelViewLayerID(ItemBase *) {
 	return ViewLayer::SchematicLabel;
 }
 
@@ -345,13 +350,13 @@ const QString & SchematicSketchWidget::traceColor(ConnectorItem *) {
 	else return m_lastColorSelected;
 }
 
-const QString & SchematicSketchWidget::traceColor(ViewLayer::ViewLayerSpec) {
+const QString & SchematicSketchWidget::traceColor(ViewLayer::ViewLayerPlacement) {
 	return SchematicTraceColor;
 }
 
-bool SchematicSketchWidget::isInLayers(ConnectorItem * connectorItem, ViewLayer::ViewLayerSpec viewLayerSpec) {
+bool SchematicSketchWidget::isInLayers(ConnectorItem * connectorItem, ViewLayer::ViewLayerPlacement viewLayerPlacement) {
 	Q_UNUSED(connectorItem);
-	Q_UNUSED(viewLayerSpec);
+	Q_UNUSED(viewLayerPlacement);
 	return true;
 }
 
@@ -373,7 +378,7 @@ bool SchematicSketchWidget::sameElectricalLayer2(ViewLayer::ViewLayerID, ViewLay
 }
 
 double SchematicSketchWidget::getKeepout() {
-	return 0.1 * GraphicsUtils::SVGDPI;
+	return 0.1 * GraphicsUtils::SVGDPI;  // in pixels
 }
 
 bool SchematicSketchWidget::acceptsTrace(const ViewGeometry & viewGeometry) {
@@ -428,8 +433,8 @@ QString SchematicSketchWidget::generateCopperFillUnit(ItemBase * itemBase, QPoin
 	return "";
 }
 
-ViewLayer::ViewLayerSpec SchematicSketchWidget::createWireViewLayerSpec(ConnectorItem * from, ConnectorItem * to) {
-	return SketchWidget::createWireViewLayerSpec(from, to);
+ViewLayer::ViewLayerPlacement SchematicSketchWidget::createWireViewLayerPlacement(ConnectorItem * from, ConnectorItem * to) {
+	return SketchWidget::createWireViewLayerPlacement(from, to);
 }
 
 double SchematicSketchWidget::getWireStrokeWidth(Wire *, double wireWidth)
@@ -437,7 +442,7 @@ double SchematicSketchWidget::getWireStrokeWidth(Wire *, double wireWidth)
 	return wireWidth * TraceHoverStrokeFactor;
 }
 
-Wire * SchematicSketchWidget::createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec spec) 
+Wire * SchematicSketchWidget::createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement spec) 
 {
 	viewGeometry.setSchematicTrace(true);
 	Wire * wire =  SketchWidget::createTempWireForDragging(fromWire, wireModel, connectorItem, viewGeometry, spec);
@@ -474,3 +479,52 @@ bool SchematicSketchWidget::canConnect(Wire *, ItemBase *) {
 QString SchematicSketchWidget::checkDroppedModuleID(const QString & moduleID) {
     return moduleID;
 }
+
+LayerList SchematicSketchWidget::routingLayers(ViewLayer::ViewLayerPlacement) {
+    LayerList layerList;
+    layerList << ViewLayer::Schematic;
+    return layerList;
+}
+
+bool SchematicSketchWidget::attachedToBottomLayer(ConnectorItem * connectorItem) {
+    return (connectorItem->attachedToViewLayerID() == ViewLayer::Schematic) ||
+           (connectorItem->attachedToViewLayerID() == ViewLayer::SchematicTrace);
+}
+
+bool SchematicSketchWidget::attachedToTopLayer(ConnectorItem *) {
+    return false;
+}
+
+QSizeF SchematicSketchWidget::jumperItemSize() {
+    if (SchematicSketchWidget::m_jumperItemSize.width() == 0) {
+	    long newID = ItemBase::getNextID();
+	    ViewGeometry viewGeometry;
+	    viewGeometry.setLoc(QPointF(0, 0));
+	    ItemBase * itemBase = addItem(referenceModel()->retrieveModelPart(ModuleIDNames::NetLabelModuleIDName), defaultViewLayerPlacement(NULL), BaseCommand::SingleView, viewGeometry, newID, -1, NULL);
+	    if (itemBase) {
+		    SymbolPaletteItem * netLabel = qobject_cast<SymbolPaletteItem *>(itemBase);
+            netLabel->setLabel("00");
+            SchematicSketchWidget::m_jumperItemSize = netLabel->boundingRect().size();
+            deleteItem(itemBase, true, false, false);
+        }
+    }
+
+	return SchematicSketchWidget::m_jumperItemSize;
+}
+
+QHash<QString, QString> SchematicSketchWidget::getAutorouterSettings() {
+    return SketchWidget::getAutorouterSettings();
+}
+
+void SchematicSketchWidget::setAutorouterSettings(QHash<QString, QString> & autorouterSettings) {
+    SketchWidget::setAutorouterSettings(autorouterSettings);
+}
+
+void SchematicSketchWidget::getDroppedItemViewLayerPlacement(ModelPart * modelPart, ViewLayer::ViewLayerPlacement & viewLayerPlacement) {
+    SketchWidget::getDroppedItemViewLayerPlacement(modelPart, viewLayerPlacement);
+}
+
+ViewLayer::ViewLayerPlacement SchematicSketchWidget::getViewLayerPlacement(ModelPart * modelPart, QDomElement & instance, QDomElement & view, ViewGeometry & viewGeometry) 
+{
+    return SketchWidget::getViewLayerPlacement(modelPart, instance, view, viewGeometry);
+}
