@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6572 $:
+$Revision: 6999 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-16 13:08:45 +0200 (Tue, 16 Oct 2012) $
+$Date: 2013-04-28 14:14:07 +0200 (So, 28. Apr 2013) $
 
 ********************************************************************/
 
@@ -35,6 +35,7 @@ $Date: 2012-10-16 13:08:45 +0200 (Tue, 16 Oct 2012) $
 #include <QGraphicsEllipseItem>
 #include <QSet>
 #include <QHash>
+#include <QMap>
 #include <QTimer>
 
 #include "../items/paletteitem.h"
@@ -66,9 +67,29 @@ struct SwapThing {
     ItemBase * itemBase;
     long newModelIndex;
     QString newModuleID;
-    ViewLayer::ViewLayerSpec viewLayerSpec;
+    ViewLayer::ViewLayerPlacement viewLayerPlacement;
     QList<Wire *> wiresToDelete;
     QUndoCommand * parentCommand;
+    QHash<ConnectorItem *, ChangeConnectionCommand *> reconnections;
+    QHash<ConnectorItem *, Connector *> byWire;
+    QHash<ConnectorItem *, ConnectorItem *> toConnectorItems;
+    QHash<ConnectorItem *, Connector *> swappedGender;
+    SketchWidget * bbView;
+    QMap<QString, QString> propsMap;
+};
+
+struct RenderThing {
+    bool selectedItems;
+    double printerScale;
+    bool blackOnly;
+    QRectF imageRect;
+    QRectF offsetRect;
+    double dpi;
+	bool renderBlocker;
+    QRectF itemsBoundingRect;
+    QGraphicsItem * board;
+	bool empty;
+    bool hideTerminalPoints;
 };
 
 class SizeItem : public QObject, public QGraphicsLineItem
@@ -85,13 +106,13 @@ class SketchWidget : public InfoGraphicsView
 	Q_OBJECT
 
 public:
-    SketchWidget(ViewLayer::ViewIdentifier, QWidget *parent=0, int size=400, int minSize=300);
+    SketchWidget(ViewLayer::ViewID, QWidget *parent=0, int size=400, int minSize=300);
 	~SketchWidget();
 
-	void pushCommand(QUndoCommand *);
+	void pushCommand(QUndoCommand *, QObject * signalTarget);
     class WaitPushUndoStack * undoStack();
-    ItemBase * addItem(ModelPart *, ViewLayer::ViewLayerSpec, BaseCommand::CrossViewType, const ViewGeometry &, long id, long modelIndex, AddDeleteItemCommand * originatingCommand);
-	ItemBase * addItem(const QString & moduleID, ViewLayer::ViewLayerSpec, BaseCommand::CrossViewType, const ViewGeometry &, long id, long modelIndex, AddDeleteItemCommand * originatingCommand);
+    ItemBase * addItem(ModelPart *, ViewLayer::ViewLayerPlacement, BaseCommand::CrossViewType, const ViewGeometry &, long id, long modelIndex, AddDeleteItemCommand * originatingCommand);
+	ItemBase * addItem(const QString & moduleID, ViewLayer::ViewLayerPlacement, BaseCommand::CrossViewType, const ViewGeometry &, long id, long modelIndex, AddDeleteItemCommand * originatingCommand);
     void deleteItem(long id, bool deleteModelPart, bool doEmit, bool later);
     virtual void deleteItem(ItemBase *, bool deleteModelPart, bool doEmit, bool later);
     void simpleMoveItem(long id, QPointF);
@@ -105,6 +126,8 @@ public:
     void selectItem(long id, bool state, bool updateInfoView, bool doEmit);
     void selectItem(ItemBase * itemBase);
     void selectItems(QList<ItemBase *>);
+    void selectItemsWithModuleID(ModelPart *);
+    void addToSketch(QList<ModelPart *> &);
     void selectDeselectAllCommand(bool state);
     void changeWire(long fromID, QLineF line, QPointF pos, bool updateConnections, bool updateRatsnest);   
     void changeLeg(long fromID, const QString & connectorID, const QPolygonF &, bool relative, const QString & why);   
@@ -124,13 +147,14 @@ public:
 	void sendBackward();
 	void bringForward();
 	void bringToFront();
+    void alignItems(Qt::Alignment);
 	double fitInWindow();
-	void rotateX(double degrees, bool rubberBandLegEnabled);
+	void rotateX(double degrees, bool rubberBandLegEnabled, ItemBase * originatingItem);
 	void flipX(Qt::Orientations orientation, bool rubberBandLegEnabled);
 	void addBendpoint(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
 	void flattenCurve(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
 
-	void deleteSelected(Wire *, bool plus);
+	void deleteSelected(Wire *, bool minus);
 	PaletteItem *getSelectedPart();
 
     void addViewLayer(ViewLayer *);
@@ -141,7 +165,6 @@ public:
 	void setLayerActive(ViewLayer::ViewLayerID viewLayerID, bool active);
     bool layerIsVisible(ViewLayer::ViewLayerID);
     bool layerIsActive(ViewLayer::ViewLayerID);
-	void sortSelectedByZ(QList<ItemBase *> & bases);
 	void sortAnyByZ(const QList<QGraphicsItem *> & items, QList<ItemBase *> & bases);
  	void mousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
  	void setBackground(QColor);
@@ -153,13 +176,13 @@ public:
 	virtual void changeConnection(long fromID,
 						  const QString & fromConnectorID,
 						  long toID, const QString & toConnectorID,
-						  ViewLayer::ViewLayerSpec,
+						  ViewLayer::ViewLayerPlacement,
 						  bool connect, bool doEmit, 
 						  bool updateConnections);
 
  	ItemCount calcItemCount();
 
-	ViewLayer::ViewIdentifier viewIdentifier();
+	ViewLayer::ViewID viewID();
 	void setViewLayerIDs(ViewLayer::ViewLayerID part, ViewLayer::ViewLayerID wire, ViewLayer::ViewLayerID connector, ViewLayer::ViewLayerID ruler, ViewLayer::ViewLayerID note);
 	void stickem(long stickTargetID, long stickSourceID, bool stick);
 	void stickyScoop(ItemBase * stickyOne, bool checkCurrent, CheckStickyCommand *);
@@ -174,16 +197,12 @@ public:
 	void noteChanged(ItemBase *, const QString & oldText, const QString &newtext, QSizeF oldSize, QSizeF newSize);
 
 	void setInfoViewOnHover(bool infoViewOnHover);
-	virtual ItemBase * addItemAux(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, bool doConnectors, ViewLayer::ViewIdentifier, bool temporary);
-	ItemBase * addItemAuxTemp(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, bool doConnectors, ViewLayer::ViewIdentifier, bool temporary);
+	virtual ItemBase * addItemAux(ModelPart *, ViewLayer::ViewLayerPlacement, const ViewGeometry &, long id, bool doConnectors, ViewLayer::ViewID, bool temporary);
+	ItemBase * addItemAuxTemp(ModelPart *, ViewLayer::ViewLayerPlacement, const ViewGeometry &, long id, bool doConnectors, ViewLayer::ViewID, bool temporary);
 
     bool swappingEnabled(ItemBase *);
 
 	virtual void addViewLayers();
-	void addPcbViewLayers();
-	void addSchematicViewLayers();
-	void addBreadboardViewLayers();
-	void addIconViewLayers();
 
 	void changeWireColor(long wireId, const QString& color, double opacity);
 	void changeWireWidth(long wireId, double width);
@@ -218,23 +237,14 @@ public:
 	void noteSizeChanged(ItemBase * itemBase, const QSizeF & oldSize, const QSizeF & newSize);
 	void resizeNote(long itemID, const QSizeF & );
 	class SelectItemCommand* stackSelectionState(bool pushIt, QUndoCommand * parentCommand);
-	QString renderToSVG(double printerScale, const LayerList &, 
-						bool blackOnly, QRectF & imageRect, ItemBase * board, double dpi, 
-						bool selectedItems, bool renderBlocker, bool & empty);
-	QString renderToSVG(double printerScale, const LayerList &, 
-						bool blackOnly, QRectF & imageRect, ItemBase * board, QRectF & offsetRect, double dpi, 
-						bool selectedItems, bool renderBlocker, bool & empty);
-	QString renderToSVG(double printerScale, bool blackOnly, QRectF & imageRect, QRectF & offsetRect, double dpi, 
-								  bool renderBlocker, 
-								  QList<QGraphicsItem *> & itemsAndLabels, QRectF itemsBoundingRect, bool & empty);
+	QString renderToSVG(RenderThing &, QGraphicsItem * board, const LayerList &);
 
 	bool spaceBarIsPressed();
 	virtual long setUpSwap(SwapThing &, bool master);
 	ConnectorItem * lastHoverEnterConnectorItem();
 	ItemBase * lastHoverEnterItem();
 	LayerHash & viewLayers();
-	virtual void createTrace(Wire*);
-	virtual void updateNet(Wire*);
+	virtual void createTrace(Wire*, bool useLastWireColor);
 	virtual void selectAllWires(ViewGeometry::WireFlag);
 	virtual void tidyWires();
 	const QString & getShortName();
@@ -244,7 +254,7 @@ public:
 	void disconnectAll();
 	virtual bool canDisconnectAll();
 	virtual bool ignoreFemale();
-	virtual ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	virtual ViewLayer::ViewLayerID getWireViewLayerID(const ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	ItemBase * findItem(long id);
 	long createWire(ConnectorItem * from, ConnectorItem * to, ViewGeometry::WireFlags, bool dontUpdate, BaseCommand::CrossViewType, QUndoCommand * parentCommand);
 	int selectAllObsolete();
@@ -268,28 +278,28 @@ public:
 	void initGrid();
 	virtual double defaultGridSizeInches();
 	void clearPasteOffset();
-	ViewLayer::ViewLayerSpec defaultViewLayerSpec();
+	virtual ViewLayer::ViewLayerPlacement defaultViewLayerPlacement(ModelPart *);
 	void addFixedToCenterItem2(class SketchMainHelp *item);
 	void collectAllNets(QHash<class ConnectorItem *, int> & indexer, QList< QList<class ConnectorItem *>* > & allPartConnectorItems, bool includeSingletons, bool bothSides);
 	virtual bool routeBothSides();
 	virtual void changeLayer(long id, double z, ViewLayer::ViewLayerID viewLayerID);
 	void ratsnestConnect(ConnectorItem * connectorItem, bool connect);
 	void ratsnestConnect(ItemBase *, bool connect);
-	void ratsnestConnect(ConnectorItem * c1, ConnectorItem * c2, bool connect);
+	void ratsnestConnect(ConnectorItem * c1, ConnectorItem * c2, bool connect, bool wait);
 	virtual void addDefaultParts();
 	float getTopZ();
 	QGraphicsItem * addWatermark(const QString & filename);
 	void copyHeart(QList<ItemBase *> & bases, bool saveBoundingRects, QByteArray & itemData, QList<long> & modelIndexes);
 	void pasteHeart(QByteArray & itemData, bool seekOutsideConnections);
 	ViewGeometry::WireFlag getTraceFlag();
-	void changeBus(ItemBase *, bool connec, const QString & oldBus, const QString & newBus, QList<ConnectorItem *> &, const QString & message);
+	void changeBus(ItemBase *, bool connec, const QString & oldBus, const QString & newBus, QList<ConnectorItem *> &, const QString & message, const QString & oldLayout, const QString & newLayout);
 	const QString & filenameIf();
 	void setItemDropOffset(long id, QPointF offset);
 	void prepLegBendpointMove(ConnectorItem * from, int index, QPointF oldPos, QPointF newPos, ConnectorItem * to, bool changeConnections);
 	void prepLegCurveChange(ConnectorItem * from, int index, const class Bezier * oldB, const class Bezier * newB, bool triggerFirstTime);
 	void prepLegBendpointChange(ConnectorItem * from, int oldCount, int newCount, int index, QPointF pos, const class Bezier *, const class Bezier *, const class Bezier *, bool triggerFirstTime);
 	void prepLegSelection(ItemBase *);
-	void changeWireCurve(long id, const Bezier *);
+	void changeWireCurve(long id, const Bezier *, bool autoroutable);
 	void changeLegCurve(long id, const QString & connectorID, int index, const Bezier *);
 	void addLegBendpoint(long id, const QString & connectorID, int index, QPointF, const class Bezier *, const class Bezier *);
 	void removeLegBendpoint(long id, const QString & connectorID, int index, const class Bezier *);
@@ -316,21 +326,30 @@ public:
     void showUnrouted();
 	QPointF alignOneToGrid(ItemBase * itemBase);
 	void showEvent(QShowEvent * event);
-
+    void blockUI(bool);
+    void viewItemInfo(ItemBase * item);
+    virtual QHash<QString, QString> getAutorouterSettings();
+    virtual void setAutorouterSettings(QHash<QString, QString> &);
+    void hidePartLayer(long id, ViewLayer::ViewLayerID, bool hide);
+    void hidePartLayer(ItemBase *, ViewLayer::ViewLayerID, bool hide);
+    void moveItem(ItemBase *, double x, double y);
 
 protected:
-    void dragEnterEvent(QDragEnterEvent *event);
-	bool dragEnterEventAux(QDragEnterEvent *event);
-	virtual bool canDropModelPart(ModelPart * modelPart);
+    void dragEnterEvent(QDragEnterEvent *);
+	bool dragEnterEventAux(QDragEnterEvent *);
+	virtual bool canDropModelPart(ModelPart *);
 
-    void dragLeaveEvent(QDragLeaveEvent *event);
-    void dragMoveEvent(QDragMoveEvent *event);
-    void dropEvent(QDropEvent *event);
-	virtual void mousePressEvent(QMouseEvent *event);
-	void mouseMoveEvent(QMouseEvent *event);
-	void mouseReleaseEvent(QMouseEvent *event);
+    void dragLeaveEvent(QDragLeaveEvent *);
+    void dragMoveEvent(QDragMoveEvent *);
+    void dropEvent(QDropEvent *);
+	virtual void mousePressEvent(QMouseEvent *);
+	void mouseMoveEvent(QMouseEvent *);
+	void mouseReleaseEvent(QMouseEvent *);
+    void mouseDoubleClickEvent (QMouseEvent *);
+    void contextMenuEvent(QContextMenuEvent *);
+    bool viewportEvent(QEvent *);
     void paintEvent(QPaintEvent *);
-    PaletteItem* addPartItem(ModelPart * modelPart, ViewLayer::ViewLayerSpec, PaletteItem * paletteItem, bool doConnectors, bool & ok, ViewLayer::ViewIdentifier, bool temporary);
+    virtual PaletteItem* addPartItem(ModelPart * , ViewLayer::ViewLayerPlacement, PaletteItem *, bool doConnectors, bool & ok, ViewLayer::ViewID, bool temporary);
 	void clearHoldingSelectItem();
 	bool startZChange(QList<ItemBase *> & bases);
 	void continueZChange(QList<ItemBase *> & bases, int start, int end, bool (*test)(int current, int start), int inc, const QString & text);
@@ -340,29 +359,28 @@ protected:
 	ViewLayer::ViewLayerID getPartViewLayerID();
 	ViewLayer::ViewLayerID getRulerViewLayerID();
 	ViewLayer::ViewLayerID getConnectorViewLayerID();
-	virtual ViewLayer::ViewLayerID getLabelViewLayerID(ViewLayer::ViewLayerSpec);
+	virtual ViewLayer::ViewLayerID getLabelViewLayerID(ItemBase *);
 	ViewLayer::ViewLayerID getNoteViewLayerID();
 	void dragMoveHighlightConnector(QPoint eventPos);
 
 	void addToScene(ItemBase * item, ViewLayer::ViewLayerID viewLayerID);
-	ConnectorItem * findConnectorItem(ItemBase * item, const QString & connectorID, ViewLayer::ViewLayerSpec);
-	bool checkMoved();
+	ConnectorItem * findConnectorItem(ItemBase * item, const QString & connectorID, ViewLayer::ViewLayerPlacement);
+	bool checkMoved(bool wait);
 
 	void changeConnectionAux(long fromID, const QString & fromConnectorID,
 						  long toID, const QString & toConnectorID,
-						  ViewLayer::ViewLayerSpec,
+						  ViewLayer::ViewLayerPlacement,
 						  bool connect, bool updateConnections);
 
-
-	void cutDeleteAux(QString undoStackMessage, bool plus, Wire * wire);
+	void cutDeleteAux(QString undoStackMessage, bool minus, Wire * wire);
 	void deleteAux(QSet<ItemBase *> & deletedItems, QUndoCommand * parentCommand, bool doPush);
 
 	ChangeConnectionCommand * extendChangeConnectionCommand(BaseCommand::CrossViewType, long fromID, const QString & fromConnectorID,
 									   long toID, const QString & toConnectorID,
-									   ViewLayer::ViewLayerSpec,
+									   ViewLayer::ViewLayerPlacement,
 									   bool connect, QUndoCommand * parent);
 	ChangeConnectionCommand * extendChangeConnectionCommand(BaseCommand::CrossViewType, ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem,
-										ViewLayer::ViewLayerSpec,
+										ViewLayer::ViewLayerPlacement,
 										bool connect, QUndoCommand * parentCommand);
 
 
@@ -372,7 +390,7 @@ protected:
 	void dragWireChanged(class Wire* wire, ConnectorItem * from, ConnectorItem * to);
 	void dragRatsnestChanged();
 	void killDroppingItem();
-	ViewLayer::ViewLayerID getViewLayerID(ModelPart *, ViewLayer::ViewIdentifier, ViewLayer::ViewLayerSpec);
+	ViewLayer::ViewLayerID getViewLayerID(ModelPart *, ViewLayer::ViewID, ViewLayer::ViewLayerPlacement);
 	ItemBase * overSticky(ItemBase *);
 	virtual void setNewPartVisible(ItemBase *);
 	virtual bool collectFemaleConnectees(ItemBase *, QSet<ItemBase *> &);
@@ -382,12 +400,12 @@ protected:
 	bool currentlyInfoviewed(ItemBase *item);
 	void resizeEvent(QResizeEvent *);
 
-	void addViewLayersAux(const LayerList &layers, float startZ = 1.5);
+	void addViewLayersAux(const LayerList &, const LayerList & layersFromBelow, float startZ = 1.5);
 	virtual bool disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash &, bool doCommand, bool rubberBandLegEnabled, QUndoCommand * parentCommand);
 	void clearDragWireTempCommand();
 	bool draggingWireEnd();
 	void moveItems(QPoint globalPos, bool checkAutoScroll, bool rubberBandLegEnabled);
-	virtual ViewLayer::ViewLayerID multiLayerGetViewLayerID(ModelPart * modelPart, ViewLayer::ViewIdentifier, ViewLayer::ViewLayerSpec, LayerList &);
+	virtual ViewLayer::ViewLayerID multiLayerGetViewLayerID(ModelPart * modelPart, ViewLayer::ViewID, ViewLayer::ViewLayerPlacement, LayerList &);
 	virtual BaseCommand::CrossViewType wireSplitCrossView();
 	virtual bool canChainMultiple();
 	virtual bool canChainWire(Wire *);
@@ -410,6 +428,7 @@ protected:
 	void handleConnect(QDomElement & connect, ModelPart *, const QString & fromConnectorID, ViewLayer::ViewLayerID, QStringList & alreadyConnected, 
 						QHash<long, ItemBase *> & newItems, QUndoCommand * parentCommand, bool seekOutsideConnections);
 	void setUpSwapReconnect(SwapThing &, ItemBase * itemBase, long newID, bool master);
+    void makeSwapWire(SketchWidget *, ItemBase *, long newID, ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, Connector * newConnector, QUndoCommand * parentCommand);
 	bool swappedGender(ConnectorItem * originalConnectorItem, Connector * newConnector);
 	void setLastPaletteItemSelected(PaletteItem * paletteItem);
 	void setLastPaletteItemSelectedIf(ItemBase * itemBase);
@@ -418,15 +437,15 @@ protected:
 	void clickBackground(QMouseEvent *);
 	void categorizeDragWires(QSet<Wire *> & wires, QList<ItemBase *> & freeWires);
 	void categorizeDragLegs(bool rubberBandLegEnabled);
-	void prepMove(ItemBase * originatingItem, bool rubberBandLegEnabled);
+	void prepMove(ItemBase * originatingItem, bool rubberBandLegEnabled, bool includeRatsnest);
 	void initBackgroundColor();
 	QPointF calcNewLoc(ItemBase * moveBase, ItemBase * detachFrom);
 	long findPartOrWire(long itemID);
-	virtual AddItemCommand * newAddItemCommand(BaseCommand::CrossViewType crossViewType, 
-											   QString moduleID, ViewLayer::ViewLayerSpec, ViewGeometry & viewGeometry, qint64 id, 
-											   bool updateInfoView, long modelIndex, QUndoCommand *parent);
+	AddItemCommand * newAddItemCommand(BaseCommand::CrossViewType crossViewType, ModelPart *,
+											   QString moduleID, ViewLayer::ViewLayerPlacement, ViewGeometry & viewGeometry, qint64 id, 
+											   bool updateInfoView, long modelIndex, bool addSubparts, QUndoCommand *parent);
 	int selectAllItems(QSet<ItemBase *> & itemBases, const QString & msg);
-	bool moveByArrow(int dx, int dy, QKeyEvent * );
+	bool moveByArrow(double dx, double dy, QKeyEvent * );
 	double gridSizeInches();
 	virtual bool canAlignToTopLeft(ItemBase *);
 	virtual bool canAlignToCenter(ItemBase *);
@@ -437,25 +456,27 @@ protected:
 	void dropItemEvent(QDropEvent *event);
     virtual QString checkDroppedModuleID(const QString & moduleID);
 	QString makeWireSVG(Wire * wire, QPointF offset, double dpi, double printerscale, bool blackOnly);
-	QString makeWireSVGAux(Wire * wire, double width, const QString & color, QPointF offset, double dpi, double printerScale, bool blackOnly);
+	QString makeWireSVGAux(Wire * wire, double width, const QString & color, QPointF offset, double dpi, double printerScale, bool blackOnly, bool dashed);
 
 	QString makeMoveSVG(double printerScale, double dpi, QPointF & offset); 
-	void prepDeleteProps(ItemBase * itemBase, long id, const QString & newModuleID, QUndoCommand * parentCommand);
-	void prepDeleteOtherProps(ItemBase * itemBase, long id, const QString & newModuleID, QUndoCommand * parentCommand);
-	virtual ViewLayer::ViewLayerSpec getViewLayerSpec(ModelPart * modelPart, QDomElement & instance, QDomElement & view, ViewGeometry &);
-	virtual ViewLayer::ViewLayerSpec wireViewLayerSpec(ConnectorItem *);
+	void prepDeleteProps(ItemBase * itemBase, long id, const QString & newModuleID, QMap<QString, QString> & propsMap, QUndoCommand * parentCommand);
+	void prepDeleteOtherProps(ItemBase * itemBase, long id, const QString & newModuleID, QMap<QString, QString> & propsMap, QUndoCommand * parentCommand);
+	virtual ViewLayer::ViewLayerPlacement getViewLayerPlacement(ModelPart *, QDomElement & instance, QDomElement & view, ViewGeometry &);
+	virtual ViewLayer::ViewLayerPlacement wireViewLayerPlacement(ConnectorItem *);
 
-	virtual bool resizingJumperItemPress(QGraphicsItem * item);
+	virtual bool resizingJumperItemPress(ItemBase *);
 	virtual bool resizingJumperItemRelease();
-	virtual bool resizingBoardPress(QGraphicsItem * item);
-	virtual bool resizingBoardRelease();
+	bool resizingBoardPress(ItemBase *);
+	bool resizingBoardRelease();
+	void resizeBoard();
+    void resizeWithHandle(ItemBase * itemBase, double mmW, double mmH);
 	virtual QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	virtual bool acceptsTrace(const ViewGeometry &);
-	virtual ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
+	virtual ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerPlacement, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
 	void showPartLabelsAux(bool show, QList<ItemBase *> & itemBases);
 	virtual void extraRenderSvgStep(ItemBase *, QPointF offset, double dpi, double printerScale, QString & outputSvg);
-	virtual ViewLayer::ViewLayerSpec createWireViewLayerSpec(ConnectorItem * from, ConnectorItem * to);
-	virtual Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	virtual ViewLayer::ViewLayerPlacement createWireViewLayerPlacement(ConnectorItem * from, ConnectorItem * to);
+	virtual Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerPlacement);
 	virtual void prereleaseTempWireForDragging(Wire*);
 	void checkFit(ModelPart * newModelPart, ItemBase * itemBase, long newID,
 								QHash<ConnectorItem *, Connector *> & found, QList<ConnectorItem *> & notFound,
@@ -472,29 +493,38 @@ protected:
 	bool checkUpdateRatsnest(QList<ConnectorItem *> & connectorItems);
 	void makeRatsnestViewGeometry(ViewGeometry & viewGeometry, ConnectorItem * source, ConnectorItem * dest); 
 	virtual double getTraceWidth();
-	virtual const QString & traceColor(ViewLayer::ViewLayerSpec);
-	void createTrace(Wire * fromWire, const QString & commandString, ViewGeometry::WireFlag);
-	bool createOneTrace(Wire * wire, ViewGeometry::WireFlag flag, bool allowAny, QList<Wire *> & done, QUndoCommand * parentCommand);
+	virtual const QString & traceColor(ViewLayer::ViewLayerPlacement);
+	void createTrace(Wire * fromWire, const QString & commandString, ViewGeometry::WireFlag, bool useLastWireColor);
+	bool createOneTrace(Wire * wire, ViewGeometry::WireFlag flag, bool allowAny, QList<Wire *> & done, bool useLastWireColor, QUndoCommand * parentCommand);
 	void removeWire(Wire * w, QList<ConnectorItem *> & ends, QList<Wire *> & done, QUndoCommand * parentCommand);
     void selectAllWiresFrom(ViewGeometry::WireFlag flag, QList<QGraphicsItem *> & items);
     bool canConnect(ItemBase * from, ItemBase * to);
     virtual bool canConnect(Wire * from, ItemBase * to);
     void removeDragWire();
+    QGraphicsItem * getClickedItem(QList<QGraphicsItem *> & items);
+    void cleanupRatsnests(QList< QPointer<ConnectorItem> > & connectorItems, bool connect);
+    void rotateWire(Wire *, QTransform & rotation, QPointF center, bool undoOnly, QUndoCommand * parentCommand);
+	QString renderToSVG(RenderThing &, const LayerList &);
+	QString renderToSVG(RenderThing &, QList<QGraphicsItem *> & itemsAndLabels);
+    QList<ItemBase *> collectSuperSubs(ItemBase *);
+    void squashShapes(QPointF scenePos);
+    void unsquashShapes();
 
 protected:
 	static bool lessThan(int a, int b);
 	static bool greaterThan(int a, int b);
 
 signals:
-	void itemAddedSignal(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, SketchWidget * dropOrigin);
+	void itemAddedSignal(ModelPart *, ItemBase *, ViewLayer::ViewLayerPlacement, const ViewGeometry &, long id, SketchWidget * dropOrigin);
 	void itemDeletedSignal(long id);
 	void clearSelectionSignal();
 	void itemSelectedSignal(long id, bool state);
+    void itemMovedSignal(ItemBase *);
 	void wireDisconnectedSignal(long fromID, QString fromConnectorID);
 	void wireConnectedSignal(long fromID,  QString fromConnectorID, long toID, QString toConnectorID);
 	void changeConnectionSignal(long fromID, QString fromConnectorID,
 								long toID, QString toConnectorID,
-								ViewLayer::ViewLayerSpec,
+								ViewLayer::ViewLayerPlacement,
 								bool connect, bool updateConnections);
 	void copyBoundingRectsSignal(QHash<QString, QRectF> &);
 	void cleanUpWiresSignal(CleanUpWiresCommand *);
@@ -529,9 +559,15 @@ signals:
     void canConnectSignal(Wire * from, ItemBase * to, bool & connect);
     void swapStartSignal(SwapThing & swapThing, bool master);
 	void showing(SketchWidget *);
+    void clickedItemCandidateSignal(QGraphicsItem *, bool & ok);
+    void resizedSignal(ItemBase *);
+    void cleanupRatsnestsSignal(bool doEmit);
+    void addSubpartSignal(long id, long subpartID, bool doEmit);
+    void getDroppedItemViewLayerPlacementSignal(ModelPart * modelPart, ViewLayer::ViewLayerPlacement &);
+    void packItemsSignal(int columns, const QList<long> & ids, QUndoCommand *parent, bool doEmit);
 
 protected slots:
-	void itemAddedSlot(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, SketchWidget * dropOrigin);
+	void itemAddedSlot(ModelPart *, ItemBase *, ViewLayer::ViewLayerPlacement, const ViewGeometry &, long id, SketchWidget * dropOrigin);
 	void itemDeletedSlot(long id);
 	void clearSelectionSlot();
 	void itemSelectedSlot(long id, bool state);
@@ -543,7 +579,7 @@ protected slots:
 	void toggleLayerVisibility();
 	void wireConnectedSlot(long fromID, QString fromConnectorID, long toID, QString toConnectorID);
 	void wireDisconnectedSlot(long fromID, QString fromConnectorID);
-	void changeConnectionSlot(long fromID, QString fromConnectorID, long toID, QString toConnectorID, ViewLayer::ViewLayerSpec, bool connect, bool updateConnections);
+	void changeConnectionSlot(long fromID, QString fromConnectorID, long toID, QString toConnectorID, ViewLayer::ViewLayerPlacement, bool connect, bool updateConnections);
 	void navigatorScrollChange(double x, double y);
 	void restartPasteCount();
 	void dragIsDoneSlot(class ItemDrag *);
@@ -571,6 +607,7 @@ protected slots:
     void deleteTemporary();
     void canConnect(Wire * from, ItemBase * to, bool & connect);
     long swapStart(SwapThing & swapThing, bool master);
+    virtual void getDroppedItemViewLayerPlacement(ModelPart * modelPart, ViewLayer::ViewLayerPlacement &);
 
 public slots:
 	void changeWireColor(const QString newColor);
@@ -581,7 +618,7 @@ public slots:
 	void incInstanceTitle(long id);
 	void showPartLabel(long id, bool showIt);
 	void checkSticky(long id, bool doEmit, bool checkCurrent, CheckStickyCommand *);
-	void resizeBoard(long id, double w, double h);
+	virtual ItemBase * resizeBoard(long id, double w, double h);
 	void resizeJumperItem(long id, QPointF pos, QPointF c0, QPointF c1);
 	void disconnectAllSlot(QList<ConnectorItem *>, QHash<ItemBase *, SketchWidget *> & itemsToDelete, QUndoCommand * parentCommand);
 	void setResistance(long itemID, QString resistance, QString pinSpacing, bool doEmit);
@@ -594,6 +631,9 @@ public slots:
 	virtual void changeBoardLayers(int layers, bool doEmit);
 	void updateConnectors();
 	void ratsnestConnect(long id, const QString & connectorID, bool connect, bool doEmit);
+    void cleanupRatsnests(bool doEmit);
+    void addSubpart(long id, long subpartid, bool doEmit);
+    void packItems(int columns, const QList<long> & ids, QUndoCommand *parent, bool doEmit);
 
 protected:
 	enum StatusConnectStatus {
@@ -605,7 +645,7 @@ protected:
 protected:
 	QPointer<class ReferenceModel> m_referenceModel;
 	QPointer<SketchModel> m_sketchModel;
-	ViewLayer::ViewIdentifier m_viewIdentifier;
+	ViewLayer::ViewID m_viewID;
 	class WaitPushUndoStack * m_undoStack;
 	class SelectItemCommand * m_holdingSelectItemCommand;
 	class SelectItemCommand * m_tempDragWireCommand;
@@ -671,14 +711,15 @@ protected:
 	QList<QGraphicsItem *> m_inFocus;
 	QString m_viewName;
 	bool m_movingByArrow;
-	int m_arrowTotalX;
-	int m_arrowTotalY;
+	double m_arrowTotalX;
+	double m_arrowTotalY;
 	bool m_movingByMouse;
 	bool m_alignToGrid;
 	bool m_showGrid;
 	double m_gridSizeInches;
     QString m_gridSizeText;
 	QPointer<ItemBase> m_alignmentItem;
+	QPointer<ItemBase> m_originatingItem;
 	QPointF m_alignmentStartPoint;
 	double m_zoom;
 	bool m_draggingBendpoint;
@@ -691,6 +732,8 @@ protected:
 	QPointer<QGraphicsSvgItem> m_movingItem;
 	QList< QPointer<ConnectorItem> > m_ratsnestUpdateDisconnect;
 	QList< QPointer<ConnectorItem> > m_ratsnestUpdateConnect;
+	QList< QPointer<ConnectorItem> > m_ratsnestCacheDisconnect;
+	QList< QPointer<ConnectorItem> > m_ratsnestCacheConnect;
 	QList <ItemBase *> m_checkUnder;
 	bool m_addDefaultParts;
 	QPointer<ItemBase> m_addedDefaultPart;
@@ -703,10 +746,13 @@ protected:
 	RoutingStatus m_routingStatus;
 	bool m_anyInRotation;
     bool m_pasting;
+	QPointer<class ResizableBoard> m_resizingBoard;
+    QList< QPointer<ItemBase> > m_squashShapes;
 
 public:
-	static ViewLayer::ViewLayerID defaultConnectorLayer(ViewLayer::ViewIdentifier viewId);
+	static ViewLayer::ViewLayerID defaultConnectorLayer(ViewLayer::ViewID viewId);
 	static const int PropChangeDelay;
+    static bool m_blockUI;
 
 protected:
 	static const int MoveAutoScrollThreshold;
