@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 6561 $:
+$Revision: 6984 $:
 $Author: irascibl@gmail.com $:
-$Date: 2012-10-12 22:39:11 +0200 (Fri, 12 Oct 2012) $
+$Date: 2013-04-22 23:44:56 +0200 (Mo, 22. Apr 2013) $
 
 ********************************************************************/
 
@@ -30,15 +30,23 @@ $Date: 2012-10-12 22:39:11 +0200 (Fri, 12 Oct 2012) $
 #include "textutils.h"
 #include "misc.h"
 #include "../installedfonts.h"
+
 //#include "../debugdialog.h"
 
 #include <QRegExp>
 #include <QBuffer>
 #include <QFile>
+#include <QSettings>
+#include <QUrl>
+#include <QUuid>
+#include <QCryptographicHash>
 
 #include <qmath.h>
 #include <qnumeric.h>
 
+QSet<QString> InstalledFonts::InstalledFontsList;
+QMultiHash<QString, QString> InstalledFonts::InstalledFontsNameMapper;   // family name to filename; SVG files seem to have to use filename
+
 const QString TextUtils::CreatedWithFritzingString("Created with Fritzing (http://www.fritzing.org/)");
 const QString TextUtils::CreatedWithFritzingXmlComment("<!-- " + CreatedWithFritzingString + " -->\n");
 
@@ -53,7 +61,7 @@ const QRegExp TextUtils::floatingPointMatcher(RegexFloatDetector);
 static const QRegExp HexExpr("&#x[0-9a-fA-F];");   // &#x9; &#xa; &#xd;
 static const QRegExp Xmlns("xmlns=([\"|'])[^\"']*\\1");
 
-static const ushort MicroSymbolCode = 181;
+const ushort TextUtils::MicroSymbolCode = 181;
 const QString TextUtils::MicroSymbol = QString::fromUtf16(&MicroSymbolCode, 1);
 
 const QString TextUtils::AdobeIllustratorIdentifier = "Generator: Adobe Illustrator";
@@ -70,7 +78,7 @@ QSet<QString> getAttrFontFamilies(const QString &fileContent) {
 	/*
 	 * font-family defined as attr example:
 
-<text xmlns="http://www.w3.org/2000/svg" font-family="DroidSans"
+<text xmlns="http://www.w3.org/2000/svg" font-family="Droid Sans"
 id="text2732" transform="matrix(1 0 0 1 32.2012 236.969)"
 font-size="9.9771" >A0</text>
 
@@ -102,7 +110,7 @@ FixedFontsHash fixFontsMapping(const QSet<QString> fontsTofix, const QString & d
 	QSet<QString> tempFontsToFix;
 	tempFontsToFix = fontsTofix - InstalledFonts::InstalledFontsList;
 	foreach (QString fontFileName, InstalledFonts::InstalledFontsNameMapper.values()) {
-		// SVG uses filename which may not match family name (e.g. "DroidSans" and "Droid Sans")
+		// SVG uses filename which may not match family name (e.g. "Droid Sans" and "Droid Sans")
 		tempFontsToFix.remove(fontFileName);
 	}
 
@@ -140,6 +148,22 @@ bool fixUnavailableFontFamilies(QString &fileContent, const QString & destFont)
 	return fixedFonts.size() > 0;
 }
 
+QString  makeDashString(bool dashed, const QVector<qreal> & pattern, double dpi, double printerScale)
+{
+    QString dash;
+    if (dashed && pattern.count() > 0) {
+        dash = "stroke-dasharray='";
+        foreach (qreal p, pattern) {
+            dash += QString::number(p * dpi / printerScale);
+            dash += ",";
+        }
+        dash.chop(1);
+        dash.append("'");
+    }
+
+    return dash;
+}
+
 ///////////////////////////////////////
 
 
@@ -210,11 +234,7 @@ double TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrat
 		divisor = 6.0;
 	}
 	else {
-		for (int ix = string.count() - 1; ix >= 0; ix--) {
-			if (string.at(ix).isDigit()) break;
-			
-			string.chop(1);
-		}
+        chopNotDigits(string);
 		divisor = 90.0;			// default to Qt's standard internal units if all else fails
 		chop = 0;
 	}
@@ -232,6 +252,16 @@ double TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrat
 	return result / divisor;
 }
 
+void TextUtils::chopNotDigits(QString & string) {
+	for (int ix = string.count() - 1; ix >= 0; ix--) {
+        QChar ch = string.at(ix);
+		if (ch.isDigit()) return;
+        if (ch == '.') return;
+			
+		string.chop(1);
+	}
+}
+
 bool TextUtils::squashElement(QDomDocument & doc, const QString & elementName, const QString &attName, const QRegExp &matchContent) {
     bool result = false;
     QDomElement root = doc.documentElement();
@@ -271,7 +301,7 @@ QString TextUtils::replaceTextElement(const QString & svg, const QString & id, c
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		replaceChildText(doc, node, newValue);
+		replaceChildText(node, newValue);
 		return doc.toString();
 	}
 		
@@ -293,7 +323,7 @@ QByteArray TextUtils::replaceTextElement(const QByteArray & svg, const QString &
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		replaceChildText(doc, node, newValue);
+		replaceChildText(node, newValue);
 		return doc.toByteArray();
 	}
 		
@@ -316,7 +346,7 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 		foreach (QString id, hash.keys()) {
 			if (node.attribute("id").compare(id) != 0) continue;
 
-			replaceChildText(doc, node, hash.value(id));
+			replaceChildText(node, hash.value(id));
 			changed = true;
 			break;
 		}
@@ -328,18 +358,19 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 }
 
 
-void TextUtils::replaceElementChildText(QDomDocument & doc, QDomElement & root, const QString & elementName, const QString & text) {
+void TextUtils::replaceElementChildText(QDomElement & root, const QString & elementName, const QString & text) {
 	QDomElement element = root.firstChildElement(elementName);
 	if (element.isNull()) {
-		doc.createElement(elementName);
+		element = root.ownerDocument().createElement(elementName);
 		root.appendChild(element);
 	}
-	replaceChildText(doc, element, text);
+	replaceChildText(element, text);
 }
 
-void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QString & text) {
+void TextUtils::replaceChildText(QDomNode & node, const QString & text) {
 	node.normalize();
 
+
 	QDomNodeList childList = node.childNodes();
 	for (int j = 0; j < childList.count(); j++) {
 		QDomNode child = childList.item(j);
@@ -349,7 +380,7 @@ void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QStr
 		}
 	}
 
-	QDomText t = doc.createTextNode(text);
+	QDomText t = node.ownerDocument().createTextNode(text);
 	node.appendChild(t);
 }
 
@@ -561,7 +592,11 @@ bool TextUtils::pxToInches(QDomElement &elem, const QString &attrName, bool isIl
 	return false;
 }
 
-QString TextUtils::svgMatrix(QMatrix & matrix) {
+QString TextUtils::svgMatrix(const QMatrix & matrix) {
+	return QString("matrix(%1, %2, %3, %4, %5, %6)").arg(matrix.m11()).arg(matrix.m12()).arg(matrix.m21()).arg(matrix.m22()).arg(matrix.dx()).arg(matrix.dy());
+}
+
+QString TextUtils::svgMatrix(const QTransform & matrix) {
 	return QString("matrix(%1, %2, %3, %4, %5, %6)").arg(matrix.m11()).arg(matrix.m12()).arg(matrix.m21()).arg(matrix.m22()).arg(matrix.dx()).arg(matrix.dy());
 }
 
@@ -952,11 +987,12 @@ bool TextUtils::fixInternalUnits(QString & svg)
 		result = true;
 		if (firstTime) {
 			// assumes width dpi = height dpi
-			size = parseForWidthAndHeight(svg, viewBox);
+			size = parseForWidthAndHeight(svg, viewBox, true);
 			if (size.width() == 0) {
 				// svg is messed up
 				return false;
 			}
+            firstTime = false;
 		}
 
 		QString old = findInternalUnits.cap(1) + findInternalUnits.cap(2); 
@@ -972,7 +1008,7 @@ bool TextUtils::fixInternalUnits(QString & svg)
 		result = true;
 		if (firstTime) {
 			// assumes width dpi = height dpi
-			size = parseForWidthAndHeight(svg, viewBox);
+			size = parseForWidthAndHeight(svg, viewBox, true);
 			if (size.width() == 0) {
 				// svg is messed up
 				return false;
@@ -1064,6 +1100,44 @@ bool TextUtils::fixViewBox(QDomElement & root) {
 	return true;
 }
 
+double TextUtils::getStrokeWidth(QDomElement & element, double defaultValue)
+{
+    bool ok;
+	double sw = element.attribute("stroke-width").toDouble(&ok);
+    if (ok) return sw;
+
+    QString stroke = element.attribute("stroke");
+    if (stroke == "none") return 0;
+
+    QDomElement parent = element.parentNode().toElement();
+    while (!parent.isNull()) {
+        sw = parent.attribute("stroke-width").toDouble(&ok);
+        if (ok) return sw;
+
+        stroke = parent.attribute("stroke");
+        if (stroke == "none") return 0;
+
+        parent = parent.parentNode().toElement();
+    }
+
+    //QString text;
+    //QTextStream stream(&text);
+    //element.save(stream, 0);
+    //DebugDialog::debug(QString("no circle stroke width set in %1: %2").arg(filename).arg(text));
+    
+    // default if there is no value to inherit
+    element.setAttribute("stroke-width", defaultValue);
+    return defaultValue;
+
+	//QString strokewidth("stroke-width");
+	//QString s = element.attribute("style");
+	//SvgFileSplitter::fixStyleAttribute(connectorElement, s, strokewidth);
+	//sw = connectorElement.attribute("stroke-width").toDouble(&ok);
+	//if (!ok) {
+		//return false;
+	//}
+}
+
 bool TextUtils::fixStrokeWidth(QDomDocument & svgDoc) {
     bool result = false;
 
@@ -1079,21 +1153,20 @@ bool TextUtils::fixStrokeWidth(QDomDocument & svgDoc) {
 
         QString stroke, strokeWidth;
         stroke = element.attribute("stroke");
-        strokeWidth = element.attribute("stroke-width");
         if (stroke.isEmpty()) {
             QString style = element.attribute("style");
             if (style.contains("stroke")) {
                 fixStyleAttribute(element);
                 stroke = element.attribute("stroke");
-                strokeWidth = element.attribute("stroke-width");
             }
         }
+        strokeWidth = element.attribute("stroke-width");
 
         if (stroke.isEmpty()) continue;
         if (stroke == "none") continue;
         if (!strokeWidth.isEmpty()) continue;
 
-        element.setAttribute("stroke-width", 1);
+        getStrokeWidth(element, 1);
         result = true;
     }
 
@@ -1353,27 +1426,32 @@ double TextUtils::getViewBoxCoord(const QString & svg, int coord)
 	return c.toDouble();
 }
 
-QString TextUtils::makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+QString TextUtils::makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> & pattern) 
 {
 	p1.setX(p1.x() * dpi / printerScale);
 	p1.setY(p1.y() * dpi / printerScale);
 	p2.setX(p2.x() * dpi / printerScale);
 	p2.setY(p2.y() * dpi / printerScale);
 
+    QString dash = makeDashString(dashed, pattern, dpi, printerScale);
+
 	QString stroke = (blackOnly) ? "black" : colorString;
-	return QString("<line stroke-linecap='round' stroke='%6' x1='%1' y1='%2' x2='%3' y2='%4' stroke-width='%5' />")
+	return QString("<line stroke-linecap='round' stroke='%6' x1='%1' y1='%2' x2='%3' y2='%4' stroke-width='%5' %7/>")
 					.arg(p1.x())
 					.arg(p1.y())
 					.arg(p2.x())
 					.arg(p2.y())
 					.arg(width * dpi / printerScale)
-					.arg(stroke);
+					.arg(stroke)
+                    .arg(dash)                
+                   ;
 }
 
-QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QString colorString, double dpi, double printerScale, bool blackOnly, bool dashed, const QVector<qreal> & pattern) 
 {
-	QString stroke = (blackOnly) ? "black" : colorString;
-	return QString("<path stroke-linecap='round' fill='none' stroke-width='%1' stroke='%2' d='M%3,%4C%5,%6, %7,%8 %9,%10' />")
+    QString dash = makeDashString(dashed, pattern, dpi, printerScale);
+    QString stroke = (blackOnly) ? "black" : colorString;
+	return QString("<path stroke-linecap='round' fill='none' stroke-width='%1' stroke='%2' d='M%3,%4C%5,%6, %7,%8 %9,%10' %11/>")
 					.arg(width * dpi / printerScale)
 					.arg(stroke)
 					.arg(poly.at(0).x() * dpi / printerScale)
@@ -1384,6 +1462,7 @@ QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QStr
 					.arg(poly.at(2).y() * dpi / printerScale)
 					.arg(poly.at(3).x() * dpi / printerScale)
 					.arg(poly.at(3).y() * dpi / printerScale)
+                    .arg(dash)
 					;
 }
 
@@ -1564,16 +1643,42 @@ int TextUtils::getPinsAndSpacing(const QString & expectedFileName, QString & spa
 QSizeF TextUtils::parseForWidthAndHeight(const QString & svg)
 {
 	QRectF viewBox;
-    return parseForWidthAndHeight(svg, viewBox);
+    return parseForWidthAndHeight(svg, viewBox, false);
 }
 
-QSizeF TextUtils::parseForWidthAndHeight(const QString & svg, QRectF & viewBox)
+QString TextUtils::parseForModuleID(const QString & fzp)
+{
+	QXmlStreamReader streamReader(fzp);
+    streamReader.setNamespaceProcessing(false);
+
+	while (!streamReader.atEnd()) {
+        switch (streamReader.readNext()) {
+            case QXmlStreamReader::StartElement:
+			    if (streamReader.name().toString().compare("module") == 0) {
+				    return streamReader.attributes().value("moduleId").toString();
+                }
+                break;
+            default:
+                break;
+        }
+    }
+
+    return "";
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(const QString & svg, QRectF & viewBox, bool getViewBox)
 {
 	QXmlStreamReader streamReader(svg);
-    return parseForWidthAndHeight(streamReader, viewBox);
+    return parseForWidthAndHeight(streamReader, viewBox, getViewBox);
 }
 
-QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg, QRectF & viewBox)
+QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg)
+{
+	QRectF viewBox;
+    return parseForWidthAndHeight(svg, viewBox, false);
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg, QRectF & viewBox, bool getViewBox)
 {
     svg.setNamespaceProcessing(false);
 
@@ -1604,21 +1709,30 @@ QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg, QRectF & viewBo
 
 				size.setWidth(w);
 				size.setHeight(h);
-				viewBox.setRect(0, 0, w, h);
 
-				QString vb = svg.attributes().value("viewBox").toString();
-				QStringList vbs = vb.split(QRegExp(",| "));
-				if (vbs.count() == 4) {
-					bool ok = false;
-					double d[4];
-					for (int i = 0; i < 4; i++) {
-						d[i] = vbs.at(i).toDouble(&ok);
-						if (!ok) break;
-					}
-					if (ok) {
-						viewBox.setRect(d[0], d[1], d[2], d[3]);
-					}
-				}
+                if (getViewBox) {
+                    bool gotViewBox = false;
+				    QString vb = svg.attributes().value("viewBox").toString();
+				    QStringList vbs = vb.split(QRegExp(",| "));
+				    if (vbs.count() == 4) {
+					    bool ok = false;
+					    double d[4];
+					    for (int i = 0; i < 4; i++) {
+						    d[i] = vbs.at(i).toDouble(&ok);
+						    if (!ok) break;
+					    }
+					    if (ok) {
+                            gotViewBox = true;
+						    viewBox.setRect(d[0], d[1], d[2], d[3]);
+					    }
+				    }
+
+                    if (!gotViewBox) {
+                        chopNotDigits(hs);
+                        chopNotDigits(ws);
+                        viewBox.setRect(0, 0, ws.toDouble(), hs.toDouble());
+                    }
+                }
 
 				return size;
 			}
@@ -1634,7 +1748,10 @@ QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg, QRectF & viewBo
 void TextUtils::gornTree(QDomDocument & doc)
 {
     QDomElement root = doc.documentElement();
+    QString oldid = root.attribute("id");
     root.setAttribute("id", "0");
+    root.setAttribute("gorn", "0");
+    root.setAttribute("oldid", oldid);
     gornTreeAux(root);
 }
 
@@ -1647,35 +1764,42 @@ void TextUtils::gornTreeAux(QDomElement & root) {
         QString newid = QString("%1.%2").arg(prefix).arg(index++);
         child.setAttribute("id", newid);
         child.setAttribute("gorn", newid);
-        if (!oldid.isEmpty()) child.setAttribute("oldid", oldid);
+        child.setAttribute("oldid", oldid);
         gornTreeAux(child);
         child = child.nextSiblingElement();
     }
 }
 
 bool TextUtils::elevateTransform(QDomElement & root) {
-    bool result = false;
-    if (root.hasChildNodes()) {
-        QDomElement child = root.firstChildElement();
-        while (!child.isNull()) {
-            QDomElement next = child.nextSiblingElement();
-            result |= elevateTransform(child);
-            child = next;
-        }
-        return result;
+    QList<QDomElement> transforms;
+    collectTransforms(root, transforms);
+    if (transforms.length() == 0) return false;
+
+    foreach (QDomElement element, transforms) {
+        QString transform = element.attribute("transform");
+        element.removeAttribute("transform");
+        QDomElement g = element.ownerDocument().createElement("g");
+        g.setAttribute("transform", transform);
+        element.parentNode().insertBefore(g, element);
+        g.appendChild(element);
     }
 
-    QString transform = root.attribute("transform");
-    if (transform.isEmpty()) return result;
-
-    root.removeAttribute("transform");
-    QDomElement g = root.ownerDocument().createElement("g");
-    g.setAttribute("transform", transform);
-    root.parentNode().insertBefore(g, root);
-    g.appendChild(root);
     return true;
 }
 
+void TextUtils::collectTransforms(QDomElement & root, QList<QDomElement> & transforms) {
+    QString transform = root.attribute("transform");
+    if (!transform.isEmpty()) {
+        transforms.append(root);
+    }
+
+    QDomElement child = root.firstChildElement();
+    while (!child.isNull()) {
+        collectTransforms(child, transforms);
+        child = child.nextSiblingElement();
+    }
+}
+
 bool TextUtils::fixFonts(QString & svg, const QString & destFont) {
 	bool changed = removeFontFamilySingleQuotes(svg);
 	changed |= fixUnavailableFontFamilies(svg, destFont);
@@ -1719,4 +1843,61 @@ void TextUtils::fixStyleAttribute(QDomElement & element, QString & style, const
 		style.remove(sw);
 		element.setAttribute(attributeName, value);
 	}
-}
\ No newline at end of file
+}
+
+QString TextUtils::getRandText() {
+	QString rand = QUuid::createUuid().toString();
+	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
+	return randext;
+}
+
+/*QString TextUtils::getBase64RandText() {
+	QString rand = QUuid::createUuid().toString();
+	QString randext = QCryptographicHash::hash(rand.toAscii(),QCryptographicHash::Md4).toHex();
+	return randext;
+}*/
+
+
+bool TextUtils::ensureViewBox(QDomDocument doc, double dpi, QRectF & rect, bool convert, double & w, double & h, bool getwh) {
+    bool isIllustrator = TextUtils::isIllustratorDoc(doc);
+
+    QDomElement root = doc.documentElement();
+	QString viewBox = root.attribute("viewBox");
+	if (viewBox.isEmpty() || getwh) {
+		bool ok1, ok2;
+        if (convert) {
+		    w = TextUtils::convertToInches(root.attribute("width"), &ok1, isIllustrator) * dpi;
+		    h = TextUtils::convertToInches(root.attribute("height"), &ok2, isIllustrator) * dpi;
+        }
+        else {
+		    w = root.attribute("width").toDouble(&ok1);
+		    h = root.attribute("height").toDouble(&ok2);
+        }
+		if (!ok1 || !ok2) {
+			return false;
+		}
+    }
+
+	if (viewBox.isEmpty()) {
+		root.setAttribute("viewBox", QString("0 0 %1 %2").arg(w).arg(h));
+        rect.setRect(0, 0, w, h);
+        return true;
+	}
+
+    QStringList coords = viewBox.split(QRegExp(" |,"));
+    if (coords.count() != 4) return false;
+
+    rect.setRect(coords.at(0).toDouble(), coords.at(1).toDouble(), coords.at(2).toDouble(), coords.at(3).toDouble());
+    return true;
+}
+
+QString TextUtils::findAnchor(const QDomElement & text) {
+    if (text.isNull()) return "start";
+
+    QString anchor = text.attribute("text-anchor");
+    if (!anchor.isEmpty()) return anchor;
+
+    return findAnchor(text.parentNode().toElement());
+}
+
+
